#!/bin/bash
#
# COPYRIGHT INFORMATION - DO NOT REMOVE
#
# This file is part of the MagicSpam (TM) Project and is
# Copyright (c) LinuxMagic Inc. 2018-2019 All Rights Reserved
#
# This file contains Original Code as created by LinuxMagic Inc.
#
# The Original Code is distributed on an 'AS IS' basis,
# WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
# HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
# ENJOYMENT OR NON-INFRINGEMENT.
#
# Do NOT download, distribute, use or alter this software or file in any
# way without express written permission from LinuxMagic Inc. or its parent
# company Wizard Tower TechnoServices signed by an authorized company officer.
#
# Author(s): Deion Yu <deiony@linuxmagic.com>
#
# $Id$
#
# NOTE: Minimum bash version?
#

# Globals
DEBUG=0

LOG_MSG_FMT="%s[%s]: %s - %s\n"
function info_msg {
    printf "INFO: $LOG_MSG_FMT" ${BASH_SOURCE[1]##*/} ${BASH_LINENO[0]} ${FUNCNAME[1]} "${@}"
}
function error_msg {
    printf "ERROR: $LOG_MSG_FMT" ${BASH_SOURCE[1]##*/} ${BASH_LINENO[0]} ${FUNCNAME[1]} "${@}" 1>&2
}
function debug_msg {
    if [ $DEBUG -ne 0 ]; then
        printf "DEBUG: $LOG_MSG_FMT" ${BASH_SOURCE[1]##*/} ${BASH_LINENO[0]} ${FUNCNAME[1]} "${@}"
    fi
}

function check_prerequiste {
    local required optional utility

    # the required and optional utilities that we need
    # NOTE: we assume that the 'which' utility exists
    required=( sed grep od )
    optional=( uuidgen systemctl service cagefsctl md5sum )

    # ensure that each required utility exists and is executable
    for utility in ${required[@]}; do
        initialize_utility $utility
        if [ $? -ne 0 ]; then
            error_msg "[$utility] is missing or not executable"
            return 1
        fi
    done

    # attempt to initialize the optional utilities
    for utility in ${optional[@]}; do
        initialize_utility $utility
    done

    return 0
}

function initialize_utility {
    local utility=$1

    # sanity check
    if [ -z "$utility" ]; then
        return 1
    fi

    # ensure that each required utility exists and is executable
    local path=$(which $utility 2> /dev/null)
    if [ -z "$path" ] || [ ! -x "$path" ]; then
        return 1
    fi

    # set the path to the utility as a global variable
    eval ${utility^^}=$path
    return 0
}

# Replacing INTERPRETER and MS_PHP_BASE_PATH placeholders
function replace_placeholder {
    local placeholder=$1
    local target=$2
    local file=$3

    # sanity check
    if [ -z "$placeholder" ] || [ -z "$target" ] || [ -z "$file" ]; then
        error_msg "Missing function parameters"
        return 1
    fi

    # ensure that we have all the required utilities
    if [ -z "$GREP" ] || [ -z "$SED" ]; then
        error_msg "Missing required utilities"
        return 1
    fi

    # ensure that the file exists
    if [ ! -f "$file" ]; then
        error_msg "[$file] does not exist"
        return 1
    fi

    # ensure that the placeholder exists in the file...
    $GREP -qs $placeholder $file
    if [ $? -ne 0 ]; then
        # ...but it is not an error in the case that it doesn't
        debug_msg "[$placeholder] does not exist in [$file]"
        return 0
    fi

    # replace the placeholder with the target in the file
    # NOTE: we assume that the placeholder and target are escaped correctly
    $SED -i -e "s|$placeholder|$target|g" $file
    if [ $? -ne 0 ]; then
        error_msg "Failed to replace [$placeholder] with [$target] in [$file]"
        return 1
    fi

    # success, placeholder has been replaced with target in file
    return 0
}

####
# sets the permission and ownership of the target file or directory
#
# @param   target       the target file or directory
# @param   mode         the mode to set
# @param   user         the user name to set
# @param   group        the group name to set (OPTIONAL)
#
# NOTE: the ownership of a directory will be set recursively
#
function set_permission_and_ownership
{
    local target=$1
    local mode=$2
    local user=$3
    local group=$4

    # sanity check
    if [ -z "$target" ] || [ -z "$mode" ] || [ -z "$user" ]; then
        error_msg "Missing function parameters"
        return 1
    fi

    # set group name to user name when not specified
    if [ -z "$group" ]; then
        group=$user
    fi

    # ensure that the user exists
    local uid=$(id -u $user 2> /dev/null)
    if [ -z "$uid" ]; then
        error_msg "The user [$user] does not exist"
        return 1
    fi

    # ensure that the group exists
    local gid=$(id -g $group 2> /dev/null)
    if [ -z "$gid" ]; then
        error_msg "The group [$group] does not exist"
        return 1
    fi

    # ensure that the target file or directory exists
    if [ ! -e "$target" ]; then
        error_msg "Failed to find target [$target]"
        return 1
    fi

    # the ownership of directories will be set recursively
    local recursive=""
    if [ -d "$target" ]; then
        recursive="-R"
    fi

    # set the ownership
    chown $recursive $user:$group $target
    if [ $? -ne 0 ]; then
        error_msg "Failed to set ownership of [$target] to [$user:$group]"
        return 1
    fi

    # set the permission
    chmod $mode $target
    if [ $? -ne 0 ]; then
        error_msg "Failed to set permission of [$target] to [$mode]"
        return 1
    fi

    return 0
}

####
# initialize a directory with given permission and ownership
#
# @param   directory    the directory to initialize
# @param   mode         the mode to set
# @param   user         the user name to set
# @param   group        the group name to set (OPTIONAL)
#
function initialize_directory
{
    local directory=$1
    local mode=$2
    local user=$3
    local group=$4

    # sanity check
    if [ -z "$directory" ] || [ -z "$mode" ] || [ -z "$user" ]; then
        error_msg "Missing function parameters"
        return 1
    fi

    # set group name to user name when not specified
    if [ -z "$group" ]; then
        group=$user
    fi

    # make the directory if it doesn't already exist
    if [ ! -d "$directory" ]; then
        # make the parent directories as well if they do not exist
        mkdir -p $directory
        if [ $? -ne 0 ]; then
            error_msg "Failed to make directory [$directory]"
            return 1
        fi
    fi

    # set the permission and ownership of directory
    # NOTE: any parent directories that may have been created will have the
    #       default permission and ownership
    set_permission_and_ownership $directory $mode $user $group
    if [ $? -ne 0 ]; then
        error_msg "Failed to set permission/ownership of [$directory]"
        return 1
    fi

    return 0
}

####
# initialize control file with default file with given permission and ownership
#
# @param   control      the control file to initialize
# @param   default      the default file to initialize the control file with
# @param   mode         the mode to set
# @param   user         the user name to set
# @param   group        the group name to set (OPTIONAL)
#
function initialize_control_with_file
{
    local control=$1
    local default=$2
    local mode=$3
    local user=$4
    local group=$5

    # sanity check
    if [ -z "$control" ] || [ -z "$default" ] || [ -z "$mode" ] || [ -z "$user" ]; then
        error_msg "Missing function parameters"
        return 1
    fi

    # set group name to user name when not specified
    if [ -z "$group" ]; then
        group=$user
    fi

    # ensure that the parent directory exists
    if [ ! -d "$(dirname $control)" ]; then
        error_msg "The parent directory for $control does not exist"
        return 1
    fi

    # ensure that the default file exists and is not empty
    if [ ! -f "$default" ] || [ ! -s "$default" ]; then
        error_msg "Failed to find default file [$default]"
        return 1
    fi

    # initialize the control file only if it doesn't exists
    if [ ! -f "$control" ]; then
        # copy the default file to the control file
        cp -f $default $control
        if [ $? -ne 0 ]; then
            error_msg "Failed to copy [$default] to [$control]"
            return 1
        fi
    fi

    # set the permission and ownership of control file
    set_permission_and_ownership $control $mode $user $group
    if [ $? -ne 0 ]; then
        error_msg "Failed to set permission/ownership of [$control]"
        return 1
    fi

    return 0
}

####
# initialize control file with default value with given permission and ownership
#
# @param   control      the control file to initialize
# @param   default      the default value to initialize the control file with
# @param   mode         the mode to set
# @param   user         the user name to set
# @param   group        the group name to set (OPTIONAL)
#
function initialize_control_with_value
{
    local control=$1
    local default=$2
    local mode=$3
    local user=$4
    local group=$5

    # sanity check
    if [ -z "$control" ] || [ -z "$default" ] || [ -z "$mode" ] || [ -z "$user" ]; then
        error_msg "Missing function parameters"
        return 1
    fi

    # set group name to user name when not specified
    if [ -z "$group" ]; then
        group=$user
    fi

    # ensure that the parent directory exists
    if [ ! -d "$(dirname $control)" ]; then
        error_msg "The parent directory for $control does not exist"
        return 1
    fi

    # initialize the control file only if it doesn't exists
    if [ ! -f "$control" ]; then
        # write the default value in the control file
        echo "$default" > $control
        if [ $? -ne 0 ]; then
            error_msg "Failed to write [$default] into [$control]"
            return 1
        fi
    fi

    # set the permission and ownership of control file
    set_permission_and_ownership $control $mode $user $group
    if [ $? -ne 0 ]; then
        error_msg "Failed to set permission/ownership of [$control]"
        return 1
    fi

    return 0
}

# Initializing shadow file with UUID/MAC
function initialize_shadow_file {
    local shadow=$1

    # sanity check
    if [ -z "$shadow" ]; then
        error_msg "Missing function parameters"
        return 1
    fi

    # ensure that the parent directory exists
    if [ ! -d "$(dirname $shadow)" ]; then
        error_msg "The parent directory for $shadow does not exist"
        return 1
    fi

    # check whether a non-empty shadow file already exists
    if [ -f "$shadow" ] && [ -s "$shadow" ]; then
        echo "Found system id... preserving"
        return 0
    fi

    local uuid=""
    # generate the uuid for the shadow file
    # XXX: should we try to use the MAC as the uuid?
    if [ ! -z "$UUIDGEN" ] && [ -x "$UUIDGEN" ]; then
        # attempt 1: generate uuid using uuidgen
        uuid=$($UUIDGEN)
    elif [ -f "/proc/sys/kernel/random/uuid" ]; then
        # attempt 2: generate uuid using the system uuid file
        uuid=$(cat "/proc/sys/kernel/random/uuid")
    fi

    # check that we managed to generate a uuid
    if [ -z "$uuid" ]; then
        error_msg "Failed to generate uuid"
        return 1
    fi

    # write the uuid into the shadow file
    echo $uuid > $shadow
    if [ $? -ne 0 ]; then
        error_msg "Failed to write uuid to shadow file"
        return 1
    fi

    return 0
}

# Initializing magicspam services
function initialize_service {
    local service=$1
    local level=$2
    local directory=$3
    local boot=$4

    # sanity check
    if [ -z "$service" ] || [ -z "$level" ] || [ -z "$directory" ]; then
        error_msg "Missing function parameters"
        return 1
    fi

    # ensure that the service file exists
    if [ ! -x "/etc/init.d/$service" ]; then
        error_msg "Missing [$service] service file"
        return 1
    fi

    # determine where the RC directory is on this system
    local pre="/etc"
    if [ -d "/etc/rc.d/rc2.d" ]; then
        pre="/etc/rc.d"
    fi

    # set up the RC links for the service
    for i in {0..6}; do
        case $i in
        [2-5])
            ln -sf /etc/init.d/$service $pre/rc$i.d/S$level$service
            ;;
        [016])
            ln -sf /etc/init.d/$service $pre/rc$i.d/K$level$service
            ;;
        esac
    done

    # initialize service for systemd if necessary
    if [ ! -z "$SYSTEMCTL" ] && [ -x "$SYSTEMCTL" ] && [ -d "/etc/systemd/system" ]; then
        if [ -f "$directory/$service.service" ]; then
            cp -a $directory/$service.service /etc/systemd/system
            if [ ! -z "$boot" ]; then
                $SYSTEMCTL daemon-reload > /dev/null 2>&1
                $SYSTEMCTL enable $service.service > /dev/null 2>&1
            fi
        fi
    fi

    return 0
}

# Starting magicspam services
function start_service {
    local service=$1

    # sanity check
    if [ -z "$service" ]; then
        error_msg "Missing function parameters"
        return 0
    fi

    # precedence 1: systemctl
    if [ ! -z "$SYSTEMCTL" ] && [ -x "$SYSTEMCTL" ]; then
        $SYSTEMCTL restart $service.service
        return $?
    fi

    # precedence 2: upstart
    if [ ! -z "$SERVICE" ] && [ -x "$SERVICE" ]; then
        $SERVICE $service start
        return $?
    fi

    # precedence 3: sysv-init
    if [ -x "/etc/init.d/$service" ]; then
        /etc/init.d/$service start
        return $?
    fi

    error_msg "Unable to find method to start $service service"
    return 1
}

# Stopping magicspam services
function stop_service {
    local service=$1

    # sanity check
    if [ -z "$service" ]; then
        error_msg "Missing function parameters"
        return 1
    fi

    # clean up systemd if exists
    if [ ! -z "$SYSTEMCTL" ] && [ -x "$SYSTEMCTL" ]; then
        # if systemctl thinks its running
        $SYSTEMCTL is-active $service.service >/dev/null 2>&1

        if [ $? -eq 0 ]; then
            # then stop it
            $SYSTEMCTL stop $service.service
            if [ $? -eq 0 ]; then
                return 0
            fi
        fi
    fi
    # if we are here, then either systemd is not installed,
    # systemd did not think the service was running,
    # or systemctl stop failed.

    # upstart service attempt
    if [ ! -z "$SERVICE" ] && [ -x "$SERVICE" ]; then
        # if upstart thinks its running
        $SERVICE $service status >/dev/null 2>&1

        if [ $? -eq 0 ]; then
            # then stop it
            $SERVICE $service stop
            if [ $? -eq 0 ]; then
                return 0
            fi
        fi
    fi

    # and finally if we are here, then we need to resort to direct init script
    if [ -x /etc/init.d/$service ]; then
        /etc/init.d/$service stop >/dev/null 2>&1
        if [ $? -eq 0 ]; then
            return 0
        fi
    fi

    # if we are here, then we found no way to stop the service
    echo "Failed to stop service $service using OS service management systems. You may need to manually kill the process"
    return 1
}

# Deinitialzing magicspam services
function deinit_service {
    local service=$1
    local level=$2

    if [ -z "$service" ] || [ -z "$level" ]; then
        error_msg "Missing function parameters"
        return 1
    fi

    # determine where the RC directory is on this system
    local pre=/etc
    if [ -d "/etc/rc.d/rc2.d" ]; then
        pre=/etc/rc.d
    fi

    # delete the RC links for the service
    for i in {0..6}; do
        case $i in
        [2-5])
            rm -f $pre/rc$i.d/S$level$service
            ;;
        [016])
            rm -f $pre/rc$i.d/K$level$service
            ;;
        esac
    done

    # now for the systemd stuff
    if [ ! -z "$SYSTEMCTL" ] && [ -x "$SYSTEMCTL" ] && [ -d "/etc/systemd/system" ]; then
        if [ -f "/etc/systemd/system/$service.service" ]; then
            rm -f /etc/systemd/system/$service.service
        fi
        $SYSTEMCTL disable $service.service > /dev/null 2>&1
        if [ -L "/etc/systemd/system/multi-user.target.wants/$service.service" ]; then
            rm -f /etc/systemd/system/multi-user.target.wants/$service.service
        fi
    fi
}

# Add user and group
function add_user_and_group {
    local user=$1
    local group=$2

    # sanity check
    if [ -z "$user" ] || [ -z "$group" ]; then
        error_msg "Missing function parameters"
        return 1
    fi

    # ensure that the group exists...
    local gid=$(id -g $group 2> /dev/null)
    if [ -z "$gid" ]; then
        # ...otherwise, add the group
        groupadd $group
        if [ $? -ne 0 ]; then
            error_msg "Failed to add group [$group]"
            return 1
        fi
    fi

    # ensure that the user exists...
    local uid=$(id -u magicspam 2> /dev/null)
    if [ -z "$uid" ]; then
        # ...otherwise, add the user under the group
        useradd -g $group -d /var/run/$user -M $user
        if [ $? -ne 0 ]; then
            error_msg "Failed to add user [$user]"
            return 1
        fi
    fi

    return 0
}

# Exempting magicspam user from CageFS
function exempt_magicspam_cagefs {
    # additional method for exempting the magicspam user
    if [ ! -z "$CAGEFSCTL" ] && [ -x "$CAGEFSCTL" ]; then
        # do not care whether this succeeds as it could get overwritten
        $CAGEFSCTL --disable magicspam
    fi

    return 0
}

function get_randomized_cron_time {
    local max=$1
    local frequency=$2

    # sanity check
    if [ -z "$max" ] || [ -z "$frequency" ]; then
        error_msg "Missing function parameters"
        return 1
    fi

    # validate the frequency
    if [ $frequency -lt 1 ] || [ $frequency -ge $max ]; then
        error_msg "Invalid frequency, frequency must be between 1-$max: $frequency"
        return 1
    fi

    # calculate the period between each time value
    local period=$((max / frequency))

    # calculate the range of acceptable starting time values
    local range=$((max - (period * (frequency - 1))))

    # calculate a random start time value within the accpetable range
    local time=$((`$OD -N 1 /dev/urandom | head -n1 | cut -d' ' -f2` % $range))

    # build the time string with the following properties:
    # 1. the starting time value is between the acceptable range
    # 2. the number of time values is equal to the frequency
    # 3. each time value is spaced equally apart by period
    local time_string="$time"
    for i in $(seq 2 $frequency); do
        time=$((time + period))
        time_string="$time_string,$time"
    done

    echo $time_string
    return 0
}

# Generating randomized time for certain cronjobs
function add_randomized_time_cronjob {
    local cron=$1
    local job=$2
    local user=$3
    local frequency=$4

    # sanity check
    if [ -z "$cron" ] || [ -z "$job" ] || [ -z "$user" ] || [ -z "$frequency" ]; then
        error_msg "Missing function parameters"
        return 1
    fi

    # validate that the cronfile exists
    if [ ! -f "$cron" ]; then
        error_msg "[$cron] is missing"
        return 1
    fi

    # validate that the cronjob file exists
    if [ ! -f "$job" ]; then
        error_msg "[$job] is missing"
        return 1
    fi

    # validate that the user exists
    if [ -z "$(id -u $user)" ]; then
        error_msg "[$user] is missing"
        return 1
    fi

    # generate a random minute and hour
    local minute=$(get_randomized_cron_time 60 1)
    local hours=$(get_randomized_cron_time 24 $frequency)
    if [ -z "$minute" ] || [ -z "$hours" ]; then
        error_msg "Failed to generate randomized cron time"
        return 1
    fi

    # append the randomized time cronjob to the cronfile
    echo "$minute $hours * * * $user $job" >> $cron
    return 0
}

