/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2014-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): William Storey <william@linuxmagic.com>
 *
 * $Id: bms-lists.js 5107 2015-09-18 17:58:01Z shaun $
 */

"use strict";

if (!MS) {
    var MS = {};
}
if (!MS.BMSLists) {
    MS.BMSLists = {};
}

//! form name with the list inputs.
MS.BMSLists.form = 'bmsListsForm';

//! get a list's slider "frame" element (where the click handling occurs).
/*!
 * @param int list The list number
 *
 * @return mixed Object, or undefined if not found.
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * a list is associated with a slider "frame" container... find it.
 *
 * we do this right now by finding the list's input, and then
 * finding its parent frame div.
 *
 * the HTML currently is:
 * <div class="centerColumn">
 *   <div class="slider-frame"> <-- this now has the onclick
 *     <span class="slider-button"></span>
 *   </div>
 *   <input> <!-- the list's input. -->
 * </div>
 */
MS.BMSLists.findListSliderFrame = function(list) {
    if (list === undefined) {
        MS.log("findListSliderFrame: invalid parameter");
        return undefined;
    }
    // we need the list input first.
    var input = this.findListInput(list);
    if (!input) {
        MS.log("findListSliderFrame: list input not found");
        return undefined;
    }
    // its container <div>: class="centerColumn"
    var parentDiv = input.parentElement;
    if (!parentDiv) {
        MS.log("findListSliderFrame: list input parent not found");
        return undefined;
    }
    // it should have a slider frame div, and the list input.
    if (parentDiv.children.length !== 3) {
        MS.log("findListSliderFrame: unexpected parent child count: "
            + parentDiv.children.length);
        return undefined;
    }
    // get the slider frame div. this is what we want to treat as
    // the slider (to trigger our onclick).
    var sliderFrame = parentDiv.children[0];
    return sliderFrame;
};

//! get a list's slider element itself.
/*!
 * @param int list The list number
 *
 * @return mixed Object, or undefined if not found.
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * a list is associated with a slider span... find it.
 *
 * we do this right now by finding the list's input, and then
 * finding its sibling slider span.
 * the HTML currently is:
 * <div class="centerColumn">
 *   <div class="slider-frame"> <-- this now has the onclick
 *     <span class="slider-button"></span>
 *   </div>
 *   <input> <!-- the list's input. -->
 * </div>
 */
MS.BMSLists.findListSliderOff = function(list) {
    return MS.BMSLists.findListSlider(list, 0);
};

MS.BMSLists.findListSliderFlag = function(list) {
    return MS.BMSLists.findListSlider(list, 1);
};

MS.BMSLists.findListSliderOn = function(list) {
    return MS.BMSLists.findListSlider(list, 2);
};

MS.BMSLists.findListSlider = function(list, slot) {
    if (list === undefined || slot === undefined) {
        MS.log("findListSlider: invalid parameter");
        return undefined;
    }
    // find its div.slider-frame.
    var sliderFrame = this.findListSliderFrame(list);
    // then slider is a span.slider-button - the single child of the frame.
    if (sliderFrame.children.length !== 3) {
        MS.log("findListSlider: unexpected frame child count: "
            + sliderFrame.children.length);
        return undefined;
    }
    var slider = sliderFrame.children[slot];
    return slider;
};

//! get a list's hidden input element.
/*!
 * @param int list The list number
 *
 * @return mixed Object, the element, or undefined if not found.
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * each list has a single hidden input with the name bmslist[<number>].
 */
MS.BMSLists.findListInput = function(list) {
    if (list === undefined) {
        MS.log("findListInput: invalid parameter");
        return undefined;
    }
    var listForm = document.forms.namedItem(this.form);
    if (!listForm) {
        MS.log("findListInput: list form not found");
        return undefined;
    }
    // look for the inputs for this list.
    var inputName = 'bmslist[' + list + ']';
    var inputs = listForm.elements.namedItem(inputName);
    if (!inputs) {
        MS.log("findListInput: list inputs not found");
        return undefined;
    }
    // namedItem() can return a collection or a single element... we want
    // exactly one, so if it's a collection, we can't find it.
    // we tell if it's a collection if it has the length property.
    // TODO: is there a better way to do this? well, this works!
    // right now we should only have one - its a single input for the slider.
    if (inputs.length !== undefined) {
        MS.log("findListElement: unexpected number of inputs: "
            + inputs.length);
        return undefined;
    }
    // this is a single object!
    return inputs;
};

// attach our callbacks.
// what we do here:
// - if you enable 'mipspace-all', then we set mipspace-{worst,poor,pros}
//   disabled (because they are contained in mipspace-all.
// - if you enable any mipspace-{worst,poor,pros}, then we disable
//   mipspace-all, because it contains all of the {worst,poor,pros}.
MS.addDocumentOnLoad(function() {
    // we need 3 elements for each list:
    // - its input - this is the hidden input that holds the actual value.
    //   we need this so we can tell what value the list is set to for
    //   the purposes of deciding if any sliders need to be adjusted on
    //   clicks.
    // - its slider "frame" - this is where the onclick events occur and
    //   where we need to attach our onclick to.
    // - its slider - this is the actual slider itself which we need so
    //   that we can toggle it if necessary.
    
    // mipspace-all
    var mipspaceAllInput = MS.BMSLists.findListInput(35);
    if (!mipspaceAllInput) {
        MS.log("mipspace-all input not found");
        return;
    }
    var mipspaceAllSliderFrame = MS.BMSLists.findListSliderFrame(35);
    if (!mipspaceAllSliderFrame) {
        MS.log("mipspace-all slider frame not found");
        return;
    }
    var mipspaceAllSliderOff = MS.BMSLists.findListSliderOff(35);
    if (!mipspaceAllSliderOff) {
        MS.log("mipspace-all off slider not found");
        return;
    }
    var mipspaceAllSliderFlag = MS.BMSLists.findListSliderFlag(35);
    if (!mipspaceAllSliderFlag) {
        MS.log("mipspace-all flag slider not found");
        return;
    }
    var mipspaceAllSliderOn = MS.BMSLists.findListSliderOn(35);
    if (!mipspaceAllSliderOn) {
        MS.log("mipspace-all on slider not found");
        return;
    }

    // mipspace-worst
    var mipspaceWorstInput = MS.BMSLists.findListInput(40);
    if (!mipspaceWorstInput) {
        MS.log("mipspace-worst input not found");
        return;
    }
    var mipspaceWorstSliderFrame = MS.BMSLists.findListSliderFrame(40);
    if (!mipspaceWorstSliderFrame) {
        MS.log("mipspace-worst slider frame not found");
        return;
    }
    var mipspaceWorstSliderOff = MS.BMSLists.findListSliderOff(40);
    if (!mipspaceWorstSliderOff) {
        MS.log("mipspace-worst off slider not found");
        return;
    }
    var mipspaceWorstSliderFlag = MS.BMSLists.findListSliderFlag(40);
    if (!mipspaceWorstSliderFlag) {
        MS.log("mipspace-worst flag slider not found");
        return;
    }
    var mipspaceWorstSliderOn = MS.BMSLists.findListSliderOn(40);
    if (!mipspaceWorstSliderOn) {
        MS.log("mipspace-worst on slider not found");
        return;
    }

    // mipspace-poor
    var mipspacePoorInput = MS.BMSLists.findListInput(41);
    if (!mipspacePoorInput) {
        MS.log("mipspace-poor input not found");
        return;
    }
    var mipspacePoorSliderFrame = MS.BMSLists.findListSliderFrame(41);
    if (!mipspacePoorSliderFrame) {
        MS.log("mipspace-poor slider frame not found");
        return;
    }
    var mipspacePoorSliderOff = MS.BMSLists.findListSliderOff(41);
    if (!mipspacePoorSliderOff) {
        MS.log("mipspace-poor OFF slider not found");
        return;
    }
    var mipspacePoorSliderFlag = MS.BMSLists.findListSliderFlag(41);
    if (!mipspacePoorSliderFlag) {
        MS.log("mipspace-poor FLAG slider not found");
        return;
    }
    var mipspacePoorSliderOn = MS.BMSLists.findListSliderOn(41);
    if (!mipspacePoorSliderOn) {
        MS.log("mipspace-poor ON slider not found");
        return;
    }

    // mipspace-pros
    var mipspaceProsInput = MS.BMSLists.findListInput(42);
    if (!mipspaceProsInput) {
        MS.log("mipspace-pros input not found");
        return;
    }
    var mipspaceProsSliderFrame = MS.BMSLists.findListSliderFrame(42);
    if (!mipspaceProsSliderFrame) {
        MS.log("mipspace-pros slider frame not found");
        return;
    }
    var mipspaceProsSliderOff = MS.BMSLists.findListSliderOff(42);
    if (!mipspaceProsSliderOff) {
        MS.log("mipspace-pros OFF slider not found");
        return;
    }
    var mipspaceProsSliderFlag = MS.BMSLists.findListSliderFlag(42);
    if (!mipspaceProsSliderFlag) {
        MS.log("mipspace-pros FLAG slider not found");
        return;
    }
    var mipspaceProsSliderOn = MS.BMSLists.findListSliderOn(42);
    if (!mipspaceProsSliderOn) {
        MS.log("mipspace-pros ON slider not found");
        return;
    }

    // for mipspace-all, if it is set to enabled, then all the others
    // should become disabled.
    var mipspaceAllCb = function() {
        // we only do anything if we are set to enabled (true).
        if (mipspaceAllInput.value !== 'true') {
            return;
        }
        // set all non-mipspace-all to disabled.
        mipspaceWorstSliderFlag.className = "slider-inactive";
        mipspaceWorstSliderOn.className = "slider-inactive";
        mipspaceWorstSliderOff.className = "slider-button";
        mipspaceWorstInput.value = 'false';

        mipspacePoorSliderFlag.className = 'slider-inactive';
        mipspacePoorSliderOn.className = 'slider-inactive';
        mipspacePoorSliderOff.className = 'slider-button';
        mipspacePoorInput.value = 'false';

        mipspaceProsSliderFlag.className = 'slider-inactive';
        mipspaceProsSliderOn.className = 'slider-inactive';
        mipspaceProsSliderOff.className = 'slider-button';
        mipspaceProsInput.value = 'false';
    };
    MS.addElementOnClick(mipspaceAllSliderFrame, mipspaceAllCb);

    // for mipspace-{worst,poor,pros}, if any are set enabled, we disable
    // mipspace-all.
    var mipspaceOtherCb = function() {
        // we only do anything if we're set to enabled (true).
        if (  mipspaceWorstInput.value !== 'true'
           && mipspaceWorstInput.value !== 'flag'
           && mipspacePoorInput.value !== 'true'
           && mipspacePoorInput.value !== 'flag'
           && mipspaceProsInput.value !== 'true'
           && mipspaceProsInput.value !== 'flag') {
            return;
        }
        // only turn off mips all IF it is fully on. If it is set to flag, we will leave it alone.
        if (mipspaceAllInput.value == 'true') {
            mipspaceAllSliderOn.className = 'slider-inactive';
            mipspaceAllSliderOff.className = 'slider-button';
            mipspaceAllInput.value = 'false';
        }
    };
    MS.addElementOnClick(mipspaceWorstSliderFrame, mipspaceOtherCb);
    MS.addElementOnClick(mipspacePoorSliderFrame, mipspaceOtherCb);
    MS.addElementOnClick(mipspaceProsSliderFrame, mipspaceOtherCb);

    // and ensure we're in a consistent state when we first load
    // the page.
    // if mipspace-all is enabled, then disable the rest.
    // if it's disabled, then we don't need to do anything else (i.e.,
    // we don't need to disable it if another mipspace list is enabled, because
    // we already know it's disabled!)
    if (mipspaceAllInput.value === 'true') {
        // set all non-mipspace-all to disabled.
        sliderTurnOnOff(mipspaceWorstInput, mipspaceWorstSlider, false,
            MS.intl('on'), MS.intl('off'));
        sliderTurnOnOff(mipspacePoorInput, mipspacePoorSlider, false,
            MS.intl('on'), MS.intl('off'));
        sliderTurnOnOff(mipspaceProsInput, mipspaceProsSlider, false,
            MS.intl('on'), MS.intl('off'));
    }
});

//! make a AJAX to lookup an IP against BMS lists 
/*!
 * @param object the event that triggered 
 * @param url    the URL for AJAX request, empty string to use calling page
 *
 * @return void 
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
MS.BMSLists.lookupIp = function(event, url) { 
    var inputEl = document.getElementById('ip-address');
    if (inputEl === null) {
        MS.log("Unable to find the IP address input");
        return;
    }

    // Clear previous messages
    MS.clearNoticesAndErrors();
    var messageEl = document.getElementById('bms-lookup-message');
    if (messageEl !== null) {
        messageEl.innerHTML = '';
    }

    var ipValue = inputEl.value.trim();
    if (MS.validateIdV4(ipValue) !== true) {
        MS.log("The IP address is not valid");
        MS.buildNotice(MS.i18n['ipAddressInvalid'], 'errors');
        return;
    }   

    // clear any previous responses by reseting the class name for 
    // all the span tag with the helplink
    var helpLinksSpan = document.getElementsByClassName('helplink');
    for (var i in helpLinksSpan) {
        var element = helpLinksSpan[i];
        // if there was a result then probably we have another class here
        // we want to change only the one from the BMS list
        if (element.tagName == 'SPAN' && element.parentElement.parentElement.className === 'descHelp') {
            element.className = 'helplink';
        }
    }

    if (!url) {
        url = window.location.pathname;
    }

    var module = "module=bmsLists";
    if (window.location.href.split('?')[1].split('=')[1] == "feIpReputation") {
        module = "module=feIpReputation&submodule=bmsLists";
    }

    var data_string = module + "&ms_action=lookup_ip&output_only=1&ip_address=" + encodeURIComponent(ipValue); 

    // Make the Ajax request
    ajaxRequest(url, MS.BMSLists.lookupCB, data_string, MS.showError);
};

//! keypress callback for the IP address input 
/*!
 * @param event the event handler
 * @param url   the URL for AJAX request, empty string to use calling page
 *
 * @return void 
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
MS.BMSLists.keyLookupIp = function(event, url) {
    // check the event and the key pressed if enter, then call the click callback function
    if (event !== undefined && typeof event === 'object' && event.keyCode === 13) {
        MS.BMSLists.lookupIp(event, url);
    }
};


//! BMS lookup callback if the response was success
/*!
 * @param object the AJAX response object 
 *
 * @return void 
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
MS.BMSLists.lookupCB= function(responseObj) {
    // sanity check
    if (responseObj === undefined || typeof responseObj !== 'object') {
        MS.log("displayIpCheckResult: sanity check failed");
        return;
    }

    // hide the magicspam loading icon
    var msloading = document.getElementById('magicspam-loading');
    if (msloading !== null && msloading.style.display != 'none') {
        msloading.style.display = 'none';
    }

    if (!responseObj.response) {
        return;
    }
    var data;
    if (typeof responseObj.response === 'string') {
        data = JSON.parse(responseObj.response);
    } else {
        // already JSON
        data = responseObj.response;
    }
    // make sure that we have the expected properties 
    if (!data.hasOwnProperty('status') || !data.hasOwnProperty('message') || !data.hasOwnProperty('data')) {
        MS.log("displayIpCheckResult: sanity check failed");
        return;
    } 
    // if the status is TRUE then the IP is listed 
    if (data.status === true) {
        var messageEl = document.getElementById('bms-lookup-message');
        if (messageEl !== null) {
            messageEl.innerHTML = data.message;
        }

        if (data.data.length > 0) {
            // loop over each element and add the class that will show the green circle 
            for (var i in data.data) {
                var el = document.getElementById('list-number-' + data.data[i]);
                if (el !== null) {
                    el.className += ' green-light';
               }
            }
        }
    // The IP is not listed or there was a problem with the result
    } else {
        MS.showError(data.message);
    }    
};

//! Helper function to show a error message
/*!
 * @param string error the error message that you want to display
 *
 * @return void
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
MS.showError = function(error) {
    if (error === undefined || typeof error !== 'string') {
        return;
    }

    // hide the magicspam loading icon
    var msloading = document.getElementById('magicspam-loading');
    if (msloading !== null && msloading.style.display != 'none') {
        msloading.style.display = 'none';
    }
    MS.buildNotice(error, 'errors');
};
