/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2011-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: magicspam.js 10644 2019-11-14 19:27:57Z deiony $
 */

// set up a namespace to hold our globals and functions to try to avoid
// naming collisions.
// TODO: I recall there was some discussion about whether to use this style...
//   should we stop? I think there's mostly benefits and not much changed
//   from using regular globals.
if (!MS) {
    var MS = {};
}

//! debug mode on or off.
MS.debug = false;

//! log a message to the console (in debug mode).
/*!
 * @params string msg
 *
 * @returns void
 *
 * @author William Storey <william@linuxmagic.com>
 */
MS.log = function(msg) {
    if (msg === undefined) {
        return;
    }
    if (!this.debug) {
        return;
    }
    if (!window || !window.console || !window.console.log) {
        return;
    }
    window.console.log(msg);
};

//! wrapper function to build the notices or error messages
/*!
 * @param string message that you want to display
 *        string type the type of message (errors or notices)
 *
 * @return void
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
MS.buildNotice = function(message, type) {
    // sanity check
    if (message === undefined || typeof message !== 'string' 
        || (type !== 'notices' &&  type !== 'errors')) {
        MS.log("Function called incorrectly!");
        return;
    }

    // clear any pre existing notices or errors
    MS.clearNoticesAndErrors();

    // retrieve target to prepend the notice
    var target = document.getElementById('magicspam-wrapper');
    if (target === null) {
        return;
    }

    // build the notice
    var li = document.createElement("li");
    li.innerHTML = message;

    var ul =  document.createElement("ul");
    ul.appendChild(li);

    var div = document.createElement("div");
    div.setAttribute('id', type);
    div.appendChild(ul);

    // prepend notice to the targer
    target.insertBefore(div, target.firstChild);
};

//! helper function that will remove any existing notices or errors 
/*!
 *
 * @return void
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
MS.clearNoticesAndErrors = function() {
    // remove notices
    var notices= document.getElementById('notices');
    if (notices !== null) {
        notices.remove();
    }
    // remove errors
    var errors= document.getElementById('errors');
    if (errors !== null) {
        errors.remove();
    }
}; 

//! utility function to validate a IP address on FrontEnd
/*!
 * @param string the IP address that we want to validate
 *
 * @return bool whatever the IP is valid 
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 * 
 * Note: this function assume that the IP address was trimmed of any whitespace 
 */
MS.validateIdV4 = function(ip) {
    if (ip === undefined || typeof ip !== 'string' || ip == '') {
        return false;
    }
    
    // regex to validate and IP address on the Front End
    return ip.match(/^([0-9]{1,3}\.){3}[0-9]{1,3}$/) === null ? false : true;    
};

//! when the window loads call this function
/*!
 * @param function func run when the window loads
 *
 * @return none
 *
 * @author Anonymous
 *
 * TODO: consider replacing this with addDocumentOnLoad() in this file - I
 *   think that's a more standard way to hook into onload.
 */
function addLoadEvent(func) {
   var oldonload = window.onload;
   if (typeof window.onload != 'function') {
        window.onload = func;
   } else {
        window.onload = function() {
            oldonload();
            func();
        };
    }
}

//! method to hide the 'loading' element
function hideLoading() {
    var ele = document.getElementById('magicspam-loading');
    if (ele) {
        ele.style.display = 'none';
    }
}

function showLoading() {
    var ele = document.getElementById('magicspam-loading');
    if (ele) {
        ele.style.display = 'block';
    }
}

//! set a slider to on or off.
/*!
 * @param Element input The form input for the slider
 * @param Element slider The slider span
 * @param bool on True to turn on, or false to turn off.
 * @param string onHtml The html label to show for on
 * @param string offHtml The html label to show for on
 *
 * @returns void
 *
 * @author Shaun A. Johnson <shaun@linuxmagic.com>
 * @author William Storey <william@linuxmagic.com>
 */
function sliderTurnOnOff(input, slider, on, onHtml, offHtml) {
    if (!input || !slider || on === undefined || onHtml === undefined
        || offHtml === undefined) {
        MS.log('sliderTurnOnOff: invalid parameter');
        return;
    }
    if (on == true) {
        slider.className = 'slider-button on';
        slider.innerHTML = onHtml;
        input.value = 'true';
        return;
    }

    slider.className = 'slider-button';
    slider.innerHTML = offHtml;
    input.value = 'false';
}

//! method to 'toggle' three buttons
/*!
 * @param   object    the 'calling' element a <span> element
 * @param   string    mode ('true', 'flag', 'false') what button we want to activate 
 * @param   string    the 'label' for the new activated button
 * @param   formInput the 'hidden' form element id where we actually keep the status ('true', 'flag', 'false')
 *
 * @returns void
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
function toggleButton(caller, mode, label, formInput) {  
    if (caller === undefined || mode === undefined || label === undefined || formInput === undefined) {
        MS.log('.toggleButton: invalid parameter');
        return;
    }
    // grab the current class name
    var currentclass = caller.className;

    // if the class has been flagged as 'disabled' then we return right
    // away since toggle is not supported. This is to bypass cases
    // where someone may try to bypass the interface safeties with
    // firebug or similar.
    if (currentclass.match(/(?:^|\s)disabled(?!\S)/)) {
        MS.log("toggleButton: disabled!");
        return;
    }
    // if we have this class, that means that we press on the button it self 
    // nothing to do
    if (currentclass.match(/(?:^|\s)slider-button(?!\S)/)) {
        MS.log("toggleButton: we press on the button it self");
        return;
    }

    // detect how we want to switch the slider and also where 
    // we came from to perform the slider animation
    switch (mode) {
        case 'true':
            MS.log("toggleButton: turning on");
            caller.setAttribute('class', 'slider-button on');
            caller.innerHTML = label;  
            // we want to detect where was the button previous to know 
            // where we should start the animation
            if (caller.previousElementSibling.className.match(/(?:^|\s)slider-button(?!\S)/)) {
                // deactivate the previous active button
                caller.previousElementSibling.setAttribute('class', 'slider-inactive');
            } else {
                // deactivate the previous active button
                caller.previousElementSibling.previousElementSibling.setAttribute('class', 'slider-inactive');
            }
            // update the input value 
            document.getElementById(formInput).value = 'true';
        break;
        case 'flag':
            MS.log("toggleButton: turning flag");
            caller.setAttribute('class', 'slider-button flag');
            caller.innerHTML = label;  
            if (caller.previousElementSibling.className.match(/(?:^|\s)slider-button(?!\S)/)) {
		// deactivate the previous active button
                caller.previousElementSibling.setAttribute('class', 'slider-inactive');
            } else {
                // deactivate the previous active button
                caller.nextElementSibling.setAttribute('class', 'slider-inactive');
            }
            // update the input value 
            document.getElementById(formInput).value = 'flag';
        break;
        case 'false':
            MS.log("toggleButton: turning off");
            caller.setAttribute('class', 'slider-button');
            caller.innerHTML = label;  
            if (caller.nextElementSibling.className.match(/(?:^|\s)slider-button(?!\S)/)) {
                // deactivate the previous active button
                caller.nextElementSibling.setAttribute('class', 'slider-inactive');
            } else {
                // deactivate the previous active button
                caller.nextElementSibling.nextElementSibling.setAttribute('class', 'slider-inactive');
            }
            // update the input value 
            document.getElementById(formInput).value = 'false';            
        break;
        default :
            MS.log("toggleButton: This mode is not implemented mode: " + mode);
        break;

    }
}

//! method to 'toggle' the on/off status of a slider element
/*!
 * @param   object  the 'calling' slider element to change
 * @param   object  the (hidden) form element name to toggle
 * @param   object  the master form name to operate with
 * @param   string  the 'on' label (this is to use PHP i18n)
 * @param   string  the 'off' label (this is to use PHP i18n)
 *
 * @returns void
 *
 * @author Shaun A. Johnson <shaun@linuxmagic.com>
 */
function toggleSlider(caller, elementName, formName, on, off) {
    // validate the arguments
    if (!caller || !elementName || !formName || !on || !off) {
        MS.log("toggleSlider: invalid parameter");
        return;
    }
    // grab the current class name
    var currentclass = caller.className;
    // and grab the form data
    var formobj = document.forms[formName];
    if (!formobj) {
        MS.log("toggleSlider: form not found");
        // do we alert?
        return;
    }

    // now we grab the form element
    var elementobj = formobj.elements[elementName];
    if (!elementobj) {
        MS.log("toggleSlider: form element not found");
        // do we alert an error?
        return;
    }

    // if the class has been flagged as 'disabled' then we return right
    // away since toggle is not supported. This is to bypass cases
    // where someone may try to bypass the interface safeties with
    // firebug or similar.
    if (currentclass.match(/(?:^|\s)disabled(?!\S)/)) {
        MS.log("toggleSlider: disabled!");
        return;
    }
    // if it is currently set as 'on' then we want to turn it off
    if (currentclass.match(/(?:^|\s)on(?!\S)/)) {
        MS.log("toggleSlider: turning off");
        sliderTurnOnOff(elementobj, caller, false, on, off);
    } else  {
        MS.log("toggleSlider: turning on");
        sliderTurnOnOff(elementobj, caller, true, on, off);
    }
}

//! action function to toggle the visibility of a 'inline' tab widget.
/*!
 * @param   object  the 'calling' href link - this is to be able to swap out labels
 * @param   string  the identifier for the tab widget we are toggling
 * @param   string  the 'collapse' label name (this is to use PHP backend i18n)
 * @param   string  the 'expand' label name (this is to use PHP backend i18n)
 * @param   string  the optional image prefix path
 */
function toggleSection(caller, tabId, imgpath) {
    // sanity check
    if (!caller || !tabId || !tabId.length) {
        return;
    }
    // convert the 'tabid' to an object
    var tab = document.getElementById(tabId);
    if (!tab) {
        return;
    }
    // grab the current tab classes
    var classes = tab.className;

    // and now we want to fetch the image...
    var images  = caller.getElementsByTagName('img');

    // if the classes include active...
    var n = classes.search('Active');

    var imagename = 'images/';
    if (imgpath != null) {
        imagename = imgpath;
    }
    if (n == -1) {
        // not active.. so.. let's make it active
        tab.className = 'collapsibleModule collapsibleModuleActive';
        imagename += 'collapse-icon.png';
    } else {
        tab.className = 'collapsibleModule';
        imagename += 'expand-icon.png';
    }
    if (images) {
        images[0].src = imagename;
    }
}

//! action function to toggle the visibility of a 'inline' tab widget and also remember the state
//! of the toggle.
/*
 * @param   object  the 'calling' href link - this is to be able to swap out labels
 * @param   string  the identifier for the tab widget we are toggling
 * @param   string  the state of the identifier for the tab widget we are toggling
 *
 * @author  Reece Burborough <reeceb@linuxmagic.com>
 * @author  Robert Bisewski <robertb@linuxmagic.com>
 */
function toggleSectionAndUpdateState(caller, tabId, value, prefix) {

    // Input validation 
    if (!caller || !tabId || !value) {
        return;
    }

    // Run the function to toggle the given tab 
    toggleSection(caller, tabId, prefix);

    // Determines the next state to toggle to 
    if (value == '1') {
        value = 0;
    } else {
        value = 1;
    }

    // AJAX string is generated here
    var ajax_string = "ms_action=remember_expand_state&output_only=1&module=";
    ajax_string += tabId + "&changed_module=" + tabId + "&expanded=" + value;

    // Having generated the string, go ahead and make the AJAX request
    ajaxRequest(window.location.pathname, function() {}, ajax_string, function() {});
}



//! get a rule's slider element.
/*!
 * @param string name The rule name (internal, e.g. block_lists)
 *
 * @return mixed Object, or undefined if not found.
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * a rule is associated with a slider span... find it.
 *
 * we do this right now by finding the rule's input, and then
 * finding its sibling slider span.
 * the HTML currently is:
 * <div class="centerColumn">
 *   <div class="slider-frame">
 *     <span class="slider-button" onclick..."></span>
 *   </div>
 *   <input> <!-- the rule's input. -->
 * </div>
 */
MS.findRuleSlider = function(name) {
    if (name === undefined) {
        this.log("findRuleSlider: invalid parameter");
        return undefined;
    }
    // we need the rule input first.
    var input = this.findRuleInput(name);
    if (!input) {
        this.log("findRuleSlider: rule input not found");
        return undefined;
    }
    // its container <div>
    var parentDiv = input.parentElement;
    if (!parentDiv) {
        this.log("findRuleSlider: rule input parent not found");
        return undefined;
    }
    // it should have a slider frame div, the rule input, and the 'original value' input
    if (parentDiv.children.length !== 3) {
        this.log("findRuleSlider: unexpected parent child count: "
            + parentDiv.children.length);
        return undefined;
    }
    // get the slider frame div.
    var sliderFrame = parentDiv.children[0];
    // it should have the slider button span.
    if (sliderFrame.children.length !== 1) {
        this.log("findRuleSlider: unexpected frame child count: "
            + sliderFrame.children.length);
        return undefined;
    }
    // and get the slider button span!
    var slider = sliderFrame.children[0];
    return slider;
};

//! get a rule's hidden input element.
/*!
 * @param string name The rule name (internal name)
 *
 * @return mixed Object, the element, or undefined if not found.
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * each rule has a single hidden input with the name rules[<name>].
 */
MS.findRuleInput = function(name) {
    if (name === undefined) {
        this.log("findRuleInput: invalid parameter");
        return undefined;
    }
    var ruleForm = document.forms.namedItem('serverPoliciesForm');
    if (!ruleForm) {
        this.log("findRuleInput: rule form not found");
        return undefined;
    }
    // look for the inputs for this rule.
    var inputName = 'rules[' + name + ']';
    var inputs = ruleForm.elements.namedItem(inputName);
    if (!inputs) {
        this.log("findRuleInput: rule inputs not found");
        return undefined;
    }
    // namedItem() can return a collection or a single element... we want
    // exactly one, so if it's a collection, we can't find it.
    // we tell if it's a collection if it has the length property.
    // TODO: is there a better way to do this? well, this works!
    // right now we should only have one - its a single input for the slider.
    if (inputs.length !== undefined) {
        this.log("findListElement: unexpected number of inputs: "
            + inputs.length);
        return undefined;
    }
    // this is a single object!
    return inputs;
};

//! add an onclick callback.
/*!
 * @params element ele
 * @params Function fn
 *
 * @returns bool Whether successful
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * this is to try to make event attachment cross browser compatible.
 */
MS.addElementOnClick = function(ele, fn) {
    if (!ele || !fn) {
        this.log("addElementOnClick: invalid parameter");
        return false;
    }
    // IE8 does not have addEventListener
    if (!ele.addEventListener) {
        // to make 'this' be the element, we need to call the
        // function in this manner...
        ele.attachEvent('onclick', function(e) {
            fn.apply(ele, [e]);
        });
        this.log("addElementOnClick: attached (ie)");
        return true;
    }
    ele.addEventListener('click', fn, false);
    this.log("addElementOnClick: attached");
    return true;
};

//! add a submit event callback.
/*!
 * @params EventTarget ele The element to attach to. This should
 *   always be a form for this event.
 * @params Function fn The function to fire.
 *
 * @returns bool Whether successful
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * this is to attempt to make this attachment cross browser compatible.
 *
 * TODO: this has similarities with addElementOnClick() and
 *   addDocumentOnLoad()... consider consolidating.
 */
MS.addElementOnSubmit = function(ele, fn) {
    if (!ele || !fn) {
        MS.log("addElementOnSubmit: invalid parameter");
        return false;
    }
    // IE8 does not have addEventListener
    if (!ele.addEventListener) {
        // to make 'this' be the element, we need to call the
        // function in this manner...
        ele.attachEvent('onsubmit', function(e) {
            fn.apply(ele, [e]);
        });
        this.log("addElementOnSubmit: attached (ie)");
        return true;
    }
    ele.addEventListener('submit', fn, false);
    this.log("addElementOnSubmit: attached");
    return true;
};

//! add an on document load callback.
/*!
 * @params Function fn
 *
 * @returns bool Whether successful
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * this is to try to make event attachment cross browser compatible.
 */
MS.addDocumentOnLoad = function(fn) {
    if (!fn) {
        this.log("addDocumentOnLoad: invalid parameter");
        return false;
    }

    // IE8 does not have addEventListener().
    if (!document.addEventListener) {
        document.attachEvent('onreadystatechange',
            function() {
                if (document.readyState !== 'complete') {
                    return;
                }
                fn();
            }
        );
        this.log("addDocumentOnLoad: attached (ie)");
        return true;
    }
    document.addEventListener('DOMContentLoaded', fn, false);
    this.log("addDocumentOnLoad: attached");
    return true;
};

//! translate an i18n key.
/*!
 * @param string key The i18n key
 *
 * @returns string The translation if it's available. Otherwise we'll
 *   return the key or a placeholder if the parameter was invalid.
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * we place some i18n translations into javascript.
 * this function wraps access translations up.
 */
MS.intl = function(key) {
    if (typeof key !== 'string' || key.length === 0) {
        return 'intl key missing';
    }
    // we must have the i18n object.
    if (!MS.i18n) {
        return key;
    }
    // double check there's a translation
    if (typeof MS.i18n[key] !== 'string') {
        return key;
    }
    return MS.i18n[key];
};

//! escape a string for HTML
/*!
 * @param string s
 *
 * return string escaped for HTML
 *
 * we also manually replace ' and " with html entity equivalents as
 * we need this for placing the string in html attributes.
 * for some reason the innerHTML escape method does not do this.
 *
 * @author William Storey <william@linuxmagic.com>
 */
MS.escapeForHTML = function(s) {
    if (s === undefined || typeof s !== 'string') {
        return '';
    }
    var mydiv = document.createElement('textarea');
    mydiv.innerHTML = s;
    var escaped = mydiv.innerHTML;
    var attrEscaped = escaped.replace(/'/g, '&#39;').replace(/"/g, '&#34;');
    return attrEscaped;
};

//! convert HTML to text.
/*!
 * @param string html
 *
 * @return string Text
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * given a string such as "&amp;hi", we return "&hi".
 * why would we need to do this? consider the case of setting an attribute
 * value on an element: that expects you to give it raw text and then applies
 * html encoding itself... so you need to give it raw unencoded text.
 */
MS.HTMLToText = function(html) {
    if (html === undefined || typeof html !== 'string') {
        return '';
    }

    var mydiv = document.createElement('textarea');
    mydiv.innerHTML = html;
    return mydiv.value;
};

//! popup a new window and force focus it.
/*!
 * @param string url The URL to open
 * @param string windowName The name of the window to use (or re-use).
 *
 * @return void
 *
 * @author William Storey <william@linuxmagic.com>
 */
MS.popupAndFocus = function(url, windowName) {
    if (url === undefined || typeof url !== 'string'
        || url.length === 0 || windowName === undefined
        || typeof windowName !== 'string' || windowName.length === 0) {
        MS.log("popupAndFocus: invalid parameter");
        return;
    }
    var features = 'toolbar=1,scrollbars=1,location=1,status=1,menubar=1,resizable=1';
    var win = window.open(url, windowName, features);
    if (win) {
        win.focus();
    }
};

//! some browsers don't support getElementsByClassName().
/*!
 * @param Element The element to search from
 * @param string className The name of the class we want
 *
 * @return Array
 */
MS.getElementsByClassName = function(ele, className) {
    if (ele.getElementsByClassName) {
        return ele.getElementsByClassName(className);
    }
    var a = [];
    var re = new RegExp('(^| )'+className+'( |$)');
    var els = ele.getElementsByTagName("*");
    for (var i = 0, j = els.length; i < j; i++) {
        if (re.test(els[i].className)) {
            a.push(els[i]);
        }
    }
    return a;
};

// all pages will have a 'loading' icon shown... we hide it once we're done.
addLoadEvent(hideLoading);


// define 'trim' for browser compatibility - apparently IE7, 8, and possibly 9 failed to have
// this basic functionality...
if(typeof String.prototype.trim !== 'function') {
  String.prototype.trim = function() {
    return this.replace(/^\s+|\s+$/g, '');
  }
}

/* Pushes support back to IE9
 * Element.closest() polyfill
 * https://developer.mozilla.org/en-US/docs/Web/API/Element/closest#Polyfill
 */
if (!Element.prototype.closest) {
    if (!Element.prototype.matches) {
        Element.prototype.matches = Element.prototype.msMatchesSelector ||
            Element.prototype.webkitMatchesSelector;
    }

    Element.prototype.closest = function (s) {
        var el = this;
        var ancestor = this;

        if (!document.documentElement.contains(el)) {
            return null;
        }
        do {
            if (ancestor.matches(s)) {
                return ancestor;
            }
            ancestor = ancestor.parentElement;
        } while (ancestor !== null);
        return null;
    };
}
