<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2013-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *            Robert Bisewski <robertb@linuxmagic.com>
 *
 * $Id: logsearch.php 10924 2020-01-08 20:25:15Z deiony $
 */

// ensure we have the 'include' paths pointing back to our tree.
// NOTE: /usr/share/magicspam/php/ gets updated on package build.
define('BASE_DIR', '/usr/share/magicspam/php/');
set_include_path(BASE_DIR . "includes" . PATH_SEPARATOR . get_include_path());
set_include_path(BASE_DIR . "lib" . PATH_SEPARATOR . get_include_path());

// our base requirements
require_once('config.php');         // defines and suite specific utility path information
require_once('lib.licensemanager.php'); // for accessing the license manager.

// allow up to 5 minutes execution time - handy for CSV export
ini_set('max_execution_time', 300);

// we need to go full path on the next file due to
// possible conflicting file names in the panels we integrate with.
require_once(BASE_DIR . 'includes/functions.php');
// and we also load our 'suite specific' configuration as required.
if (file_exists(BASE_DIR . 'includes/suite_config.php')) {
    include_once(BASE_DIR . 'includes/suite_config.php');
}

// load up the MagicSpam locales, default to English
$locale = getControl("ms_locale", "en-US");
require_once(loadMSLocale());

// and start our session for persistent storage (if one isn't already started).
if (session_id() === '') {
    session_start();
}
// set up our global variables

// array of error messages stored through each module
$errors = array();
// array of informational messages each module may wish to report
$notices = array();
// flag indicative if we have been called in an ajax only manner
$output_only = false;
// our array of all modules that belong in this system.
$modules = array();
// grab the singleton for the license manager binary handle
$license = LicenseManager::getInstance();

// get our posted base level arguments
$action = isset($_REQUEST['ms_action']) ? $_REQUEST['ms_action'] : NULL;
if (!$action) {
    $action = 'search';
}
// load the modules.
foreach ($pages as $page) {
    loadModule($page);
}
// if we are running our own authentication management then...
if (isset($modules['login'])) {
    // get the base level information required for all: authenticated state, license status, administration email
    $authenticated = loginModule::isAuthenticated();

    if (!$authenticated) {
        header('Location: /index.php');
        exit;
    }
}
$module = 'log';

// grab the active module.
$activeModule = isset($modules[$module]) ? $modules[$module] : NULL;

// process any actions - setting any feedback that we might need.
if ($action) {
    if ($activeModule) {
        $activeModule->actionProcessor($action, $errors, $notices);
    }
}

// If there is no active module, we print out a warning message :w
if (!$activeModule) {
    $errors[] = intl('logSearch_module_not_found') . $module;
}

// Here's where we take the i18n and make 'em javascript
$i18n = array(
              "ls_alert_whitelist",
              "ls_alert_whitelist_domain",
              "ls_alert_blacklist",
              "ls_alert_blacklist_domain",
              "ls_alert_remove_whitelist",
              "ls_alert_remove_blacklist",
              "ls_allow_IP",
              "ls_allow_Sender",
              "ls_allow_SenderDomain",
              "ls_block_IP",
              "ls_block_Country",
              "ls_block_Sender",
              "ls_block_SenderDomain",
              "ls_remove_white_ip",
              "ls_remove_black_ip",
              "ls_allow_ip_ratelimit",
              "ls_allow_user_ratelimit",
              "ls_block_message",
              "ls_cancel"
             );

$i18n_labels = array();
foreach ($i18n as $label) {
    // NOTE: we require that label and its translation are utf8 encoded
    //   prior to json_encode().
    $i18n_labels[$label] = utf8_encode(intl($label));
}
$i18nJson = json_encode($i18n_labels);

// instead of calling the common renderHeader function,
// we manually generate our header here.
print <<<END
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
    <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
END;

// Attaches the title and the body tag
print "    <title>MagicSpam - " . intl('logSearchTitle') . "</title>";

// Attaches the essential style sheets
$logCSS = array(
    "magicspam.css",
    "log-search.css",
);
foreach (generateURIs($logCSS, "css", MAGICSPAM_VERSION) as $c) {
    print "    <link style type=\"text/css\" media=\"all\" rel=\"stylesheet\" href=\"$c?\">" . "\n";
}

// Attaches any needed javascript code
$logJS = array(
    "ajax.js",
    "css_browser_selector.js",
    "help-popup.js",
    "magicspam.js",
    "magicspam-menu.js",
    "log-search.js",
);
foreach (generateURIs($logJS, "js", MAGICSPAM_VERSION) as $j) {
    print "    <script type=\"text/javascript\" src=\"$j\"></script>\n";
}

// choose appropriate behavior when clicking on the 'Back to Log Search' link
// NOTE: the difference in behavior is based on whether interface is running on
//       a MMC or regular browser
$back_behavior = ($_SERVER['SERVER_PORT'] == 6242) ? "history.back();" : "window.close();";

// Since logsearch.php isn't of type module, we have to i18n thru here
print "<script type=\"text/javascript\">\n";
print "if (!MS) { var MS = {}; }\n";
print "if (!MS.i18n) { MS.i18n = {}; }\n";
print "(function() {\n";
print "    var i18n = " . $i18nJson . ";\n";
print "    for (var label in i18n) {\n";
print "        if (!i18n.hasOwnProperty(label)) {\n";
print "            continue;\n";
print "        }\n";
print "        MS.i18n[label] = i18n[label];\n";
print "   }\n";
print "})();\n";
print "</script>\n";

// Closes the head, opens the body
print "</head>" . "\n";
print "<body>" . "\n";

// Adds error / notice boxes
print "<div id=\"errors\" style=\"display: none;\">\n</div>\n";
print "<div id=\"notices\" style=\"display: none;\">\n</div>\n";

// Adds a box to the left-hand corner of the page to allow the user to return to the log search module
print "<div id=\"return_box\">";
print "<a href=\"#\" onclick=\"$back_behavior\" title=\"" . intl('logSearchReturnLink');
print "\">" . intl('logSearchReturnLink') . "</a>"; 
print "<span id=\"magicspam-loading\"><img src=\"" . IMAGEPATH . "spinner.gif\"/> " . intl('magicspam_loading') . "</span>\n";
print "</div>\n";

// Add replay results
print "<div id=\"magicspam-wrapper\">\n";
print "    <div id=\"replay-popup\">\n</div>\n";
print "</div>\n";

// Prints out the individual module
$activeModule->renderOutput($modules);
print "</body></html>";
