<?php
/*
 * vim: ft=php
 *
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2014 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): William Storey <william@linuxmagic.com>
 *            Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * $Id: ms_purge_notifications.php 3825 2014-09-29 21:32:38Z shaun $
 *
 * This program purges all feed entries that are older than a certain date.
 * It is intended to be run periodically to keep the cache clean.
 *
 * NOTE: You must be sure to run this as the system user that the interface
 *   uses or else there may be issues if we create feed files because the
 *   interface will then not be able to work with them!
 */

// ensure we have the 'include' paths pointing back to our tree.
define('BASE_DIR', 'MS_PHP_BASE_PATH');
set_include_path(BASE_DIR . "includes" . PATH_SEPARATOR . get_include_path());
set_include_path(BASE_DIR . "lib" . PATH_SEPARATOR . get_include_path());

// our base requirements.
// config.php defines and suite specific utility path information
// as well we need it so we can get at what our feed id is that we will work
// with as well as contains the information for how many days back we purge
// each feed.
// NOTE: switched to using full path to the file as config.php may be
//   elsewhere in our path I imagine.
require_once(BASE_DIR . '/includes/config.php');
// TODO: should we just go with a full path for include here too?
//   then we could avoid changing include path.
require_once('lib.msnotifications.php');

//! purge old notifications from a feed.
/*!
 * @param int $feedID The feed ID.
 * @param int $entryLifetimeDays The number of days we keep entries for.
 *
 * @return bool Whether successful
 *
 * read in notifications for the feed and re-save without
 * those that are too old.
 *
 * we load only those that are within our time period which causes us,
 * when we save, to rewrite the cache without those that are too old.
 */
function mspnPurgeFeed($feedID, $entryLifetimeDays) {
    if (!is_int($feedID) || !is_int($entryLifetimeDays)
        || $entryLifetimeDays <= 0) {
        header("HTTP/1.0 500 Invalid parameter to mspnPurgeFeed()");
        return FALSE;
    }
        
    // timestamp of N days back.
    $timestamp = time() - (60*60*24*$entryLifetimeDays);
    // we need to load up entries back to N days back from both
    // the read & unread locally cached files. then we re-save
    // which will rewrite the cache - and anything we hadn't
    // loaded will be purged.
    $notification = new MSNotifications($feedID);
    $unreadRepo = $notification->getLocal();
    // we don't need to actually store the entries - just load
    // so they get into the object for our save().
    $unreadRepo->getEntries($timestamp);
    $readRepo = $notification->getLocalRead();
    $readRepo->getEntries($timestamp);
    if (!$unreadRepo->save() || !$readRepo->save()) {
        header("HTTP/1.0 500 Failed to save notifications!");
        return FALSE;
    }
    return TRUE;
}

// if the request did not originate from cron job loop back, then exit immediately
if (  !isset($_SERVER['REMOTE_ADDR']) 
   || $_SERVER['REMOTE_ADDR'] !== '127.0.0.1') {
    exit(1);
}

// we currently hardcode the feeds we work with.
if (!defined('ALERT_FEED_ID')
 || !defined('ALERT_FEED_ENTRY_LIFETIME_DAYS')) {
    header("HTTP/1.0 500 Missing alert feed settings!");
    exit(1);
}
if (!mspnPurgeFeed(ALERT_FEED_ID, ALERT_FEED_ENTRY_LIFETIME_DAYS)) {
    // we should have logged.
    exit(1);
}
if (!defined('NEWS_FEED_ID')
    || !defined('NEWS_FEED_ENTRY_LIFETIME_DAYS')) {
    header("HTTP/1.0 500 Missing news feed settings!");
    exit(1);
}
if (!mspnPurgeFeed(NEWS_FEED_ID, NEWS_FEED_ENTRY_LIFETIME_DAYS)) {
    // we should have logged.
    exit(1);
}
header('HTTP/1.0 200 Ok');
exit(0);
