<?php
/*
 * vim: ft=php
 *
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Product line and is
 * Copyright (c) LinuxMagic Inc. 2014 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: post_system_alert.php 3825 2014-09-29 21:32:38Z shaun $
 *
 * This program provides a method to add notification system feed items
 * to a given feed.
 */

// ensure we have the 'include' paths pointing back to our tree.
// NOTE: MS_PHP_BASE_PATH gets updated on package build.
define('BASE_DIR', 'MS_PHP_BASE_PATH');
set_include_path(BASE_DIR . "includes" . PATH_SEPARATOR . get_include_path());
set_include_path(BASE_DIR . "lib" . PATH_SEPARATOR . get_include_path());

// our base requirements.
// config.php defines and suite specific utility path information
// as well we need it so we can get at what our feed id is that we will insert
// to.
// NOTE: switched to using full path to the file as config.php may be
//   elsewhere in our path I imagine.
require_once(BASE_DIR . '/includes/config.php');
// TODO: should we just go with a full path for include here too?
//   then we could avoid changing include path.
require_once('lib.msnotifications.php');

//! retrieve & validate command line arguments.
/*!
 * @return mixed Associative array where each argument is a key,
 *   or bool false if failure/invalid arguments.
 *   Keys are:
 *   title - string, post title
 *   author - string, author of post
 *   type - string, type (information, warning, etc)
 *   severity - string, severity (high, medium, etc)
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * read and parse command line arguments.
 *
 * NOTE: all arguments are currently optional, and we will set the
 *   default values here.
 *
 * as well, we don't enforce specific values on author/type/severity - they
 * are accepted as free form.
 */
function getArgs() {
    
    // title. optional currently.
    $title = 'New System Alert Message';
    if (array_key_exists('t', $_POST) && strlen($_POST['t']) > 0) {
        $title = $_POST['t'];
    }
    // author. also currently optional.
    $author = 'MagicSpam System';
    if (array_key_exists('a', $_POST) && strlen($_POST['a']) > 0) {
        $author = $_POST['a'];
    }
    // type. optional.
    // TODO: enforce a limited set of valid values for type?
    $type = 'warning';
    if (array_key_exists('c', $_POST) && strlen($_POST['c']) > 0) {
        $type = $_POST['c'];
    }
    // severity. optional.
    // TODO: enforce a limited set of valid values for severity?
    // rationale as high: coming from system, high severity!
    $severity = 'high';
    if (array_key_exists('s', $_POST) && strlen($_POST['s']) > 0) {
        $severity = $_POST['s'];
    }

    $short = NULL;
    $description = NULL;
    if (array_key_exists('short', $_POST) && strlen($_POST['short']) > 0) {
        $short = substr($_POST['short'], 0, 40);
    }
    if (array_key_exists('description', $_POST) && strlen($_POST['description']) > 0) {
        $description = $_POST['description'];
    }

    if (!$short || !$description) {
        return NULL;
    }
    return array(
        'title'    => $title,
        'author'   => $author,
        'type'     => $type,
        'severity' => $severity,
        'short' => $short,
        'description' => $description,
    );
}

//! program entry.
/*!
 * @return bool True if success, false if failure
 *
 * @author Shaun A. Johnson <shaun@linuxmagic.com>
 * @author William Storey <william@linuxmagic.com>
 *
 * generate and insert a notification.
 *
 * we read the content of the notice from stdin.
 *
 * your stdin stream should end with 'LM_EOF' to stop.
 */
function main() {
    // if the request did not originate from cron job loop back, then exit immediately
    if (  !isset($_SERVER['REMOTE_ADDR'])
       || $_SERVER['REMOTE_ADDR'] !== '127.0.0.1') {
        exit(1);
    }

    // grab our arguments.
    $args = getArgs();
    if ($args === FALSE) {
        header('HTTP/1.0 500 invalid data posted for alert');
        return FALSE;
    }

    // TODO: feed could be an argument too.
    if (!defined('ALERT_FEED_ID')) {
        header('HTTP/1.0 500 alert feed ID is not available!');
        return FALSE;
    }
    // catch exceptions, just in case... because there are cases in the
    // notification libraries that throw exceptions still.
    try {
        $notification = new MSNotifications(ALERT_FEED_ID);
        global $errors;
        if (isset($errors) && is_array($errors) && count($errors) > 0) {
            header("HTTP/1.0 500 Failed to create notification object");
            return FALSE;
        }
        // NOTE: addLocalEntry() at present doesn't indicate failure/success.
        $notification->addLocalEntry($args['title'], $args['author'],
            $args['type'], $args['severity'], $args['short'], $args['description']);
    } catch (Exception $e) {
        header('HTTP/1.0 500 Failed to add entry: ' . $e->getMessage());
        return FALSE;
    }

    header('HTTP/1.0 200 OK');
    return TRUE;
}
exit(main() ? 0 : 1);
