<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam(TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2011-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: functions.php 10395 2019-09-16 18:11:15Z deiony $
 */

include_once('lib.msnotifications.php');

//! utility function to render out the header with optional lists of CSS and JS
/*!
 * @param   array   list of the CSS files that are rendered into the output.
 * @param   array   list of the JS files that are rendered into the output.
 * @param   array   list of error messages (i.e. HTML strings via locales)
 * @param   array   list of notice messages (i.e. HTML strings via locales)
 * @param   string  stores the title of the page. Note this *must* be HTML
 *   encoded already!
 * @param   mixed   optional string prefix for the 'relative' path includes.
 *
 * @return void
 *
 * @author Shaun A. Johnson <shaun@linuxmagic.com>
 * @author William Storey <william@linuxmagic.com>
 *
 * SIDE EFFECT: Outputs to stdout.
 *
 * we start the HTML document (doctype, <html>, <head>, <body>)
 * if we are in 'full page' rendering mode.
 *
 * as well, we output <link> and <script> elements to include our
 * various assets, and show errors/notices.
 */
function renderHeader($css, $js, $errors, $notices, $title, $relprefix = NULL, $force_js_css = FALSE) {
    global $isFrontend;
    
    // SJ: normally I would run sanity checks on the arguments here, but since
    // they are all optional, we do that check inline.
    // part of our header will also include a status check.
    $enabled = getControl('enabled', 0, MASTERCONTROLDIR);

    // set default values if necessary
    if (!is_array($js)) {
        $js = array();
    }
    if (!is_array($css)) {
        $css = array();
    }
    if (is_null($relprefix)) {
        $relprefix = "";
    }

    // output the standard header information if we are in full page mode.
    if (msInFullPageMode()) {
        // TODO: confirm we actually want to change doctype like this. I think
        //   it's a good idea myself and it will be consistent with what Plesk
        //   sets for us when we wrap ourselves in its framework.
        //
        print "<!DOCTYPE html>\n";
        // IE conditionals so we can apply css more effectively.
        // these classes are the same as in Plesk11+.
        print '<html>' . "\n";
        print "<head>\n";
        print '    <meta charset="UTF-8">' . "\n";
        // tell IE to run in its most current standards mode, and activate
        // its chrome frame (browser addon) if it exists (seems to be
        // recommended and what Plesk 11 sends by default).
        print '    <meta http-equiv="X-UA-Compatible"';
        print ' content="IE=edge,chrome=1">' . "\n";
        // NOTE: we expect $title to be HTML encoded.
        print '    <title>MagicSpam - ' . $title . '</title>' . "\n";

        // followed by the optional list of CSS
        foreach (generateURIs($css, "${relprefix}css", MAGICSPAM_VERSION) as $c) {
            print '    <link style type="text/css" media="all"'
                . ' rel="stylesheet" href="' . $c . '">' . "\n";
        }
        // then the optional list of Javascript
        foreach (generateURIs($js, "${relprefix}js", MAGICSPAM_VERSION) as $j) {
            print '    <script type="text/javascript" src="' . $j . '"></script>' . "\n";
        }

        print "</head>\n";
        print "<body>\n";
    } else {
        if ($force_js_css) {
            foreach (generateURIs($css, "${relprefix}css", MAGICSPAM_VERSION) as $c) {
                print '    <link style type="text/css" media="all"'
                    . ' rel="stylesheet" href="' . $c . '">' . "\n";
            }
            foreach (generateURIs($js, "${relprefix}js", MAGICSPAM_VERSION) as $j) {
                print '    <script type="text/javascript" src="' . $j . '"></script>' . "\n";
            }
        }
        // if we *aren't* in full page mode, then show an HTML comment where our
        // HTML begins.
        // NOTE: starting with a newline to ensure we begin on a fresh line as
        //       we may be included starting on an existing line.
        print "\n" . '<!-- Code Section: CopyRight LinuxMagic Inc. All Rights';
        print ' Reserved -->' . "\n";
    }

    print "<div id=\"magicspam-wrapper\">\n";

    // This displays any possible error messages
    if (is_array($errors) && sizeof($errors) > 0) {
        print "    <div id=\"errors\">\n"
            . "        <ul><li>\n"
            . implode("</li><li>", $errors)
            . "        </li></ul>\n"
            . "    </div>\n";
    }

    // This displays any possible notice messages (e.g. success notices)
    if (is_array($notices) && sizeof($notices) > 0) {
        print "    <div id=\"notices\">\n"
            . "        <ul><li>\n"
            . implode("</li><li>", $notices)
            . "        </li></ul>\n"
            . "    </div>\n";
    }

    // and.. EVERY page will have our branding.
    print "    <table border=\"0\" cellspacing=\"2\" cellpadding=\"2\" id=\"magicspam-header\">\n";
    print "    <tr>\n";
    print "        <td><img src=\"" . IMAGEPATH . "magicspam-logo-small.png\" alt=\"MagicSpam\"/></td>\n";
    print "        <td class=\"";
    if ($enabled == "1") {
        print "green\">(" . intl('enabled') . ")</td>\n";
    } else {
        print "red\">";     
        if (!isset($isFrontend) || !$isFrontend) {
            print "<a title=\"" . intl('protection_disabled')
            . "\" href=\"?module=settings\" onclick=\"doFormSubmit({'hiddenFields':{'module':'settings','page':'system_settings'}}); return false;\">";
        }
        print "("
            . intl('disabled')
            . ")";
        if (!isset($isFrontend) || !$isFrontend) {
            print "</a>";
        } else {
            
        }
        print "</td>\n";
    }
    print "    <td id=\"magicspam-loading\"><img src=\"" . IMAGEPATH . "spinner.gif\"/> " . intl('magicspam_loading') . "</td>\n";
    print "    </tr>\n";
    print "    </table>\n";
}

//! utility function to render a given body template
/*!
 * @param   string  the template 'base' directory - may vary based on what is
 *   being rendered. This is relative to the BASE_DIR constant.
 * @param   string  the template 'file'.
 * @param   boolean whether to wrap the template with autogenerated comment
 *
 * If the template does not exist, nothing will be rendered.
 */
function renderTemplate($msname, $msvars = NULL, $mswrap = TRUE) {
    // set the variables if required.
    if (is_array($msvars)) {
        // curse through the array, (using 'ms' prefix to hopefully avoid
        // variable name overlap.
        foreach ($msvars as $mskey => $msval) {
            ${$mskey} = $msval;
        }
    }
    // sanity checks.
    if (!is_string($msname) || strlen($msname) == 0 ||
        !file_exists(MS_TEMPLATE_DIR . $msname)) {
        return;
    }
    // render the template
    if ($mswrap) {
        print "<!-- AUTOGENERATED TEMPLATE INCLUDE \"" . MS_TEMPLATE_DIR . "{$msname}\" -->\n";
    }
    include (MS_TEMPLATE_DIR . $msname);
    if ($mswrap) {
        print "<!-- END AUTOGENERATED TEMPLATE INCLUDE \"" . MS_TEMPLATE_DIR . "{$msname}\" -->\n\n";
    }
}

//! wrapper function to renderTemplate for returning template as a string
function getTemplate($msname, $msvars = NULL, $mswrap = TRUE) {
    ob_start();
    renderTemplate($msname, $msvars, $mswrap);
    return ob_get_clean();
}

//! utility function to render the end of content footer.
/*!
 * @return void
 *
 * @author Shaun A. Johnson <shaun@linuxmagic.com>
 * @author William Storey <william@linuxmagic.com>
 *
 * SIDE EFFECT: outputs HTML to stdout
 *
 * This function will write directly to STDOUT and is intended for use AFTER
 * all processing is complete as the final call.
 */
function renderFooter() {
    print "    <div id=\"ms_footer\">\n";
    print "        <p>Copyright&copy; " . date("Y") .  " <a href=\"http://www.linuxmagic.com\" onclick=\"window.open('http://www.linuxmagic.com'); return false;\">LinuxMagic&reg; Inc.</a>, All Rights Reserved.</p>\n";
    print "    </div><!-- end footer -->\n";
    print "  </div><!-- end wrapper -->\n";
    if (msInFullPageMode()) {
        print "</body>\n";
        print "</html>\n";
    }
    // if we *aren't* in full page mode, then show an HTML comment where our
    // HTML ends
    if (!msInFullPageMode()) {
        print '<!-- End LinuxMagic Code Section -->' . "\n";
    }
}

//!
/*!
 * TODO: document
 */
function renderMenu() {
    global $modules, $module;
    print "    <div id=\"menu\">\n";
    print "        <ul>\n";
    foreach ($modules as $ml => $moduleHandle) {
        if ($moduleHandle->onMenu()) {
            print "<li";
            if ($ml === $module) {
                print " class=\"active\"";
            }
            print ">\n";
            print " <a href=\"?module=" . htmlspecialchars(rawurlencode($moduleHandle->getLabel()));
            print "\">";
            print "  <div class=\"tabLeft\"></div>\n";
            print "  <div class=\"tabCenter\">\n";
            $label = intl($moduleHandle->getLabel());
            if ($ml === 'magicspamAlert' && $moduleHandle->countUnread() > 0) {
                $label .= ' <span class="alert">('
                        . htmlspecialchars($moduleHandle->countUnread())
                        . ')</span>';
            } elseif ($ml === 'packages' && !$moduleHandle->hasMS()) {
                $label .= ' <span class="alert">!</span>';
            }
            echo $label;
            print "  </div>\n";
            print "  <div class=\"tabRight\"></div>\n";
            print " </a>\n";
            print "</li>\n";
        }
    }
    print "       </ul>\n";
    print "    </div>\n";
}

//! utility rendering function to render a boolean slider control
/*!
 * @param   boolean enabled or disabled setting
 * @param   int locked status (1 cannot turn off, 2 cannot chnaged, 0 no lock)
 * @param   string  the associated form input name
 * @param   string  the form name
 *
 * @returns string  the html element to render
 */
function renderSliderThree($enabled, $locked, $formItem, $formName) {
    // validate arguments
    if (! isset($enabled)
       || !isset($locked)
       || !is_string($formItem)
       || strlen($formItem) < 1
       || !is_string($formName)
       || strlen($formName) < 1 ) {
        // the problem with integration frameworks is that there is no
        // reliable way to log events...
        return "";
    }
    // load our persistent labels
    $onlabel = intl('on');
    $offlabel = intl('off');
    $flaglabel = intl('flag');
    // initialize our return string
    $html = "";

    // prep the opening widget elements
    $html .= '<div class="slider-frame three">' . "\n";
    
    // init the class for the sliders    
    $offSpanClass = 'slider-inactive';
    $flagSpanClass = 'slider-inactive';
    $onSpanClass = 'slider-inactive';

    $offSpanAction = 'onclick="toggleButton(this, \'false\', \'' . $offlabel . '\', \'' . $formItem . '\')"';
    $flagSpanAction = 'onclick="toggleButton(this, \'flag\', \'' . $flaglabel . '\', \'' . $formItem . '\')"';
    $onSpanAction = 'onclick="toggleButton(this, \'true\', \'' . $onlabel . '\', \'' . $formItem . '\')"';

    // overwrites to disable changing and removing the action    
    if ($locked >= 1) {
        // we add the disabled class that will disable the click action
        $offSpanClass .= ' disabled ';
        $offSpanAction  = 'title="' . intl('enforce_description') . '"';
    }
    
    if ($locked >= 2) {
        $flagSpanClass .= ' disabled ';
        $flagSpanAction  = 'title="' . intl('enforce_description') . '"';
    }

    $inputValue = ''; 
    // setting the selected class and setting the orginal value and input value
    if ($enabled === TRUE) {
        if ($locked >= 2) {
            $onSpanClass = "slider-button on disabled";
        } else {
            $onSpanClass = "slider-button on";
        }
        // set the input value
        $inputValue = 'true';
    } elseif ($enabled === 'flag') {
        if ($locked >= 1) {
            $flagSpanClass = "slider-button flag disabled";
        } else {
            $flagSpanClass = "slider-button flag";
        }
        // set the input value
        $inputValue = 'flag';
    } else {
        $offSpanClass = "slider-button off";
        // set the input value
        $inputValue = 'false';
    }

    $html .= '  <span class="' . $offSpanClass. '  ' . $locked .'" ' . $offSpanAction . '>';
    $html .= $offlabel;
    $html .= '</span>' . "\n";

    $html .= '  <span class="' . $flagSpanClass. '" ' . $flagSpanAction . '>';
    $html .= $flaglabel;
    $html .= '</span>' . "\n";

    $html .= '  <span class="' . $onSpanClass. '" ' . $onSpanAction . '>';
    $html .= $onlabel;
    $html .= '</span>' . "\n";

    $html .= "</div>\n";
    $html .= '<input type="hidden" id="' . $formItem . '" name="' . $formItem . '" value="' . $inputValue . '" />' . "\n";
    $html .= '<input type="hidden" id="ori_' . $formItem . '" name="ori_' . $formItem . '" value="' . $inputValue . '" />' . "\n";

    return $html;
}

//! utility rendering function to render a boolean slider control
/*!
 * @param   boolean enabled or disabled setting
 * @param   boolean locked status (eg: can't change)
 * @param   string  the associated form input name
 * @param   string  the form name
 *
 * @returns string  the html element to render
 */
function renderSlider($enabled, $locked, $formItem, $formName, $test = 0) {
    // validate arguments
    if (  !is_bool($enabled)
       || !is_bool($locked)
       || !is_string($formItem)
       || strlen($formItem) < 1
       || !is_string($formName)
       || strlen($formName) < 1 ) {
        // the problem with integration frameworks is that there is no
        // reliable way to log events...
        return "";
    }
    // load our persistent labels
    $onlabel = intl('on');
    $offlabel = intl('off');

    // initialize our return string
    $html = "";

    // prep the opening widget elements
    $html .= '<div class="slider-frame" ';
    if (!$locked) {
        $html .= "onclick=\"toggleSlider(this.children[0], '{$formItem}', '{$formName}', '{$onlabel}', '{$offlabel}');\"";
    }
    $html .=  ">\n";
    $html .= '    <span class="slider-button';

    // set the on / off / locked class name
    if ($enabled) {
        $html .= ' on';
    }
    if ($locked) {
        $html .= ' disabled';
    }
    $html .= "\">";
    if ($enabled) {
        $html .= $onlabel;
    } else {
        $html .= $offlabel;
    }
    $html .= "</span>\n</div>\n";
    // if it is not locked...
    if (!$locked) {
        // add the form item
        $html .= '<input type="hidden" name="' . $formItem . '" value="';
        if ($enabled) {
            $html .= 'true';
        } else {
            $html .= 'false';
        }
        $html .= '"/>' . "\n";
        // and also add the 'original' non changing value for later comparison
        $html .= '<input type="hidden" name="ori_' . $formItem . '" value="';
        if ($enabled) {
            $html .= 'true';
        } else {
            $html .= 'false';
        }
        $html .= '"/>' . "\n";
    }
    return $html;
}

//!
/*!
 * TODO: document
 */
function loadModule($name) {
    global $modules, $errors, $ms_words;

    if (file_exists(MODULES . $name . 'Module.php')) {
        $class = $name . 'Module';
        $file = $name . 'Module.php';
        try {
            include_once(MODULES . $file);
            $modules[$name] = new $class($name);
            $moduleLocale = loadMSLocale($name);
            if (file_exists($moduleLocale)) {
                require_once($moduleLocale);
            }
        } catch (Exception $e) {
            $errors[] = $e->getMessage();
        }
    }
}

//! common utility function to load a locale specific file.
/*!
 * @param   string  (optional) the locale subdirectory - used for plugins..
 *
 * @returns string  the file to load
 *
 * NOTE: the function will try to lead the specified language, then 'default',
 *       then 'en-US' in order of search. Locale selection is based on global
 *       variable 'locale'.
 */
function loadMSLocale($subdir = NULL) {
    global $locale;
    $mainPath = LOCALES;
    if (is_string($subdir) && strlen($subdir) > 0) {
        $mainPath .= $subdir . "/";
    }
    // check one: exact match
    if (file_exists($mainPath . $locale . '.php')) {
        return $mainPath . $locale . '.php';
    }

    // check two: variant chars
    $langparts = preg_split('/[\-_]/', $locale);

    if (sizeof($langparts) != 2) {
        $langparts = array();
        $langparts[] = strtolower($locale);
        $langparts[] = strtoupper($locale);
    }

    if (file_exists($mainPath . $langparts[0] . '-' . strtoupper($langparts[1]) . '.php')) {
        return $mainPath . $langparts[0] . '-' . strtoupper($langparts[1]) . '.php';
    }

    // check three: partial match
    if (file_exists($mainPath . $langparts[0] . '-' . strtoupper($langparts[0]) . '.php')) {
        return $mainPath . $langparts[0] . '-' . strtoupper($langparts[0]) . '.php';
    }
    // fall back to default
    if (file_exists($mainPath . 'default.php')) {
        return $mainPath . 'default.php';
    }
    // if nothing else, then english it is
    return $mainPath . 'en-US.php';
}

//! common utility function to load a locale specific phrase/word.
/*!
 * @param   string  the 'key' to fetch for the word or phrase.
 *
 * @returns string  the translation string. This will be HTML.
 *
 * @author Shaun A. Johnson <shaun@linuxmagic.com>
 * @author William Storey <william@linuxmagic.com>
 *
 * NOTE: the return will be HTML encoded as though it were run through
 *   htmlspecialchars(). this means that you MUST not re-encode it.
 *   it also means that '"' will be encoded as &quot;, but "'" MAY appear
 *   exactly as is, so you should account for that.
 */
function intl($key) {
    // TODO: check $key is a string?

    global $ms_words;

    $ret = "";

    if ( isset($ms_words) && is_array($ms_words)
        && isset($key) && isset($ms_words[$key]) ) {
        $ret = $ms_words[$key];
    }
    if ($ret == "") {
        $ret = $key;
    }
    return $ret;
}

//! utility function to get a control file setting.
/*!
 * TODO: document
 */
function getControl($name, $default = NULL, $controldir = NULL) {
    global $home;
    // sanity check
    if (!is_string($name) || !strlen($name)) {
        // insane? then return the default
        return $default;
    }
    if ($controldir) {
        $controlFile = $controldir . "/" . $name;
    } else {
        if (isset($home)) {
            $controlFile = $home . "/magicspam/control/" . $name;
        } else {
            // set the control file path based on constants from config.php
            $controlFile = CONTROLDIR . "/" . $name;
        }
    }
    // make sure it exists
    if (!file_exists($controlFile)) {
        // not set? then return the default
        return $default;
    }
    // get the data
    $data = file_get_contents($controlFile);
    // if there was an error reading, then return default
    if ($data === FALSE) {
        return $default;
    }
    // return the data!
    return trim($data);
}

//! utility function to set a control file setting.
/*!
 * @param   string  the control name
 * @param   mixed   the value to set : should be numeric or string. Array won't
 *   work.
 *
 * @returns mixed   returns FALSE on unknown error.
 *                  returns string on known error.
 *                  returns TRUE on success.
 */
function setControl($name, $value) {
    global $home;
    // cast to string when it makes sense.
    if (is_numeric($value)) {
        $value = (string) $value;
    }
    // sanity check on arguments
    if (!is_string($name) || strlen($name) == 0 ||
        !is_string($value) || strlen($value) == 0) {
        return FALSE;
    }

    // determine the destination file path
    if (isset($home)) {
        $controlDir = $home . "/magicspam/control";
    } else {
        $controlDir = CONTROLDIR;
    }
    $controlFile = $controlDir . "/" . $name;

    // some sanity checks.
    if (file_exists($controlFile) && !is_writable($controlFile)) {
        return "Control file {$controlFile} is not writable";
    } else if (!is_writable($controlDir)) {
        return "Control directory " . $controlDir . " is not writable";
    }
    // open the file
    $fp = fopen($controlFile,"w");
    if (!$fp) {
        return "Error opening control file {$controlFile}";
    }
    // write it out
    if (fwrite($fp, $value) === FALSE) {
        fclose($fp);
        return "Unknown error writing to control file {$controlFile}";
    }

    // close it
    fclose($fp);

    // done
    return TRUE;
}

//! utility function to clear a control file.
/*!
 * @param   string  the control name
 *
 * @returns string  returns string on known error.
 *                  returns Okay on success.
 */
function clearControl($name) {
    global $home;
    // sanity check on arguments
    if (!is_string($name) || strlen($name) == 0) {
        return "Control file must have a valid name";
    }

    if (isset($home)) {
        $controlDir = $home . "/magicspam/control";
    } else {
        $controlDir = CONTROLDIR;
    }
    $controlFile = $controlDir . "/" . $name;

    // sanity check: does the file exist?  If not, there is nothing to clear.
    if (!file_exists($controlFile)) {
        return TRUE;
    }
    // some sanity checks.
    if (!is_writable($controlFile)) {
        return "Control file {$controlFile} is not writable";
    } else if (!is_writable($controlDir)) {
        return "Control directory " . $controlDir . " is not writable";
    }
    // open the file
    $fh = fopen($controlFile, "w");
    if (!$fh) {
        return "Error opening control file {$controlFile}";
    }
    ftruncate($fh, 0);
    fclose($fh);

    // done
    return TRUE;
}

//! utility function to convert command line arguments into PHP hash
/*!
 * @param   array   the command line 'argv' to convert
 *
 * @returns hash    the 'hashed' results hashed by label.
 */
function argv2array($argv) {
    // sanity check
    if (!is_array($argv)) {
        return NULL;
    }
    $opts = array();
    // shift of argv[0] - that is the command name itself.
    $argv0 = array_shift($argv);
    // loop through the rest of and set up as key/value pairs.
    while ( count($argv) ) {
        $key = array_shift($argv);
        $value = array_shift($argv);
        $opts[$key] = $value;
    }
    return $opts;
}

//! utility function to recursively remove a full directory.
/*!
 * @param   string  the directory to parse
 *
 * @returns boolean success or fail of operation.
 */
function recurseRmdir($dir) {
    // sanity check the directory
    if (!is_string($dir) || strlen($dir) == 0 ||
        !is_dir($dir) ) {
        return FALSE;
    }
    // secondary sanity check: this function is intended to be used
    // ONLY for removing magicspam subdirectories in individual home dirs.
    if (preg_match("/^\/home\/[^\s]+\/magicspam(\/.*)?$/", $dir) != 1) {
        return FALSE;
    }

    // open the directory
    $dh = opendir($dir);
    if (!$dh) {
        return FALSE;
    }

    // while we can read entries...
    while ( ($f = readdir($dh)) !== FALSE ) {
        // if the entry is not '.' or '..'
        if ($f != '.' && $f != '..') {
            // if its a directory recurse again
            if (is_dir($dir . "/" . $f)) {
                if (!recurseRmdir($dir . "/" . $f)) {
                    closedir($dh);
                    return FALSE;
                }
            } else {
            // otherwise unlink the file
                if (!unlink($dir . "/" . $f)) {
                    closedir($dh);
                    return FALSE;
                }
            }
        }
    }
    // finally remove this directory.
    closedir($dh);
    if (!rmdir($dir)) {
        return FALSE;
    }
    return TRUE;
}

//! helper function to quickly determine if this is freemium or not.
/*!
 * @returns boolean true if 'freemium' otherwise false.
 *
 * NOTE: this relies on the global $license handle to be initialized
 */
function isFreemium() {
    global $license, $license_set;

    // If no license is present, we default to restricting access
    if (!$license) {
        return FALSE;
    }

    // at this time, the only time we consider this 'freemium' is if there is
    // NO license at all - this includes cases where a license is registered
    // in the Plesk License Manager (requester check)
    if ($license->isNew() && strlen($license->getCode()) > 0) {
        return TRUE;
    }
    return FALSE;
}


//! helper function to quickly determine if this is professional or not.
/*!
 * @returns boolean true if 'professional' otherwise false.
 *
 * NOTE: this relies on the global $license handle to be initialized
 *
 * Author(s): Robert Bisewski <robertb@linuxmagic.com>
 */
function isProfessional() {
    global $license, $license_set;

    // if the config has a special deefine, then turn it on
    if (!defined('IS_PROFESSIONAL') || IS_PROFESSIONAL != 1) {
        return FALSE;
    }
    
    // pass through: defined and value 1
    return TRUE;
}


//! method to check if the input number is actually a valid number/integer
/*!
* @param     int    number to check
*
* @returns   bool   true if valid int, else false
*
* @author    Robert Bisewski <robertb@linuxmagic.com>
*/
function isValidNum($num) {
    // Checks if the numeral is blank
    $str = trim("" . $num);
    if (strlen($str) === 0) {
	return false;
    }

    // Checks if an invalid number
    if (!is_numeric($str)) {
	return false;
    }

    // Checks if less than 0
    if (intval($str) < 0) {
	return false;
    }

    // Number appears to be a valid integer
    return true;
}

//! determine if we are rendering output in full page mode or not.
/*!
 * @return bool True if full page mode enabled, false if not.
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * full page mode means magicspam is responsible for setting the doctype
 * and adding <html>,<head>,<body>, etc to the page.
 *
 * this is a configuration option. however I wrap it in a function as currently
 * we default (if the option is not specified) to this being enabled.
 */
function msInFullPageMode() {
    global $MS_FULL_PAGE_RENDERING;
    if (!isset($MS_FULL_PAGE_RENDERING)) {
        return TRUE;
    }
    return $MS_FULL_PAGE_RENDERING === TRUE;
}

//! utility helper function for generating 'help', 'description', 'small'
// listings
/*!
 * @param string $noteHTML Optional HTML string to include as a third div
 *   after the internal text.
 *
 * @return string HTML
 *
 * @author Anonymous
 * @author William Storey <william@linuxmagic.com>
 */
function renderListDescription($description, $internal, $href = NULL,
    $onclick = NULL, $target = NULL, $noteHTML = NULL) {
    // validate arguments
    if ( !is_string($description)
      || strlen($description) == 0
      || !is_string($internal)
      || strlen($internal) == 0) {
        // invalid arguments
        // XXX: log error?
        return "";
    }
    if (!is_null($href) && (!is_string($href) || strlen($href) == 0)) {
        return "";
    }
    if (!is_null($onclick) && (!is_string($onclick) || strlen($onclick) == 0)) {
        return "";
    }
    if (!is_null($target) && (!is_string($target) || strlen($target) == 0)) {
        return "";
    }
    // generate the help link
    $desc  = "<div class=\"descHelp\">\n";
    $desc .= "  <a href=\"";
    if (!is_null($href)) {
        $desc .= $href;
    } else {
        $desc .= "#";
    }
    $desc .= "\"";
    if (!is_null($onclick)) {
        $desc .= " onclick=\""
               . $onclick
               . "\"";
    }
    if (!is_null($target)) {
        $desc .= " target=\""
               . $target
               . "\"";
    }
    $desc .= ">\n";

    $listname = '';
    if (strpos($internal, ': ') !== FALSE) {
        list($notNeeded, $listname) = explode(': ', $internal);
        $listname = intval($listname);
    }
    $listname = 'list-number-' . $listname;

    $desc .= "  <span id=\"$listname\" class=\"helplink\"></span></a>\n";

    $desc .= "</div><!-- end descHelp -->\n";
    // then generate the two row description and 'internal'
    $desc .= "<div class=\"descText\">\n";
    $desc .= "  <span title=\"{$description}\">{$description}</span>\n";
    $desc .= "  <small title=\"{$internal}\">{$internal}</small>\n";
    if ($noteHTML !== NULL) {
        $desc .= "  <div class=\"noteHTML\">" . "\n";
        $desc .= "    " . $noteHTML . "\n";
        $desc .= "  </div><!-- end .noteHTML -->" . "\n";
    }
    $desc .= "</div><!-- end descText -->\n";
    return $desc;
}

//! utility function to determine if a given email address is RFC valid
/*!
 * @param   string  the address to validate
 *
 * @returns boolean true if its valid, false otherwise.
 *
 */
function isValidEmail($email_address) {
    $email_name_regex = "/^([0-9a-z_\\-\\.]+)$/i";
    $domain_regex = "/([0-9a-z][0-9a-z\\-]*\\.)+[a-z]{2,4}$/i";

    $email_name_part = '';
    $domain_part = '';

    $rtn = TRUE;
    // test one: do we have a length?
    if (!is_string($email_address) || strlen($email_address) == 0) {
        return FALSE;
    }

    // try to split up the address by '@' separator
    preg_match_all("/[@]/", $email_address, $matches);
    // if we have multiple '@' symbols then its invalid
    if (count($matches[0]) > 1) {
        return FALSE;
    }
    // if there is NO @ symbol then its invalid
    if (!preg_match("/@/", $email_address)) {
        $rtn = FALSE;
    }
    // split into component parts
    list($email_name_part, $domain_part) = explode("@", $email_address);

    // validate the name part against our regex of permitted characters
    if (!preg_match($email_name_regex, $email_name_part)) {
        return FALSE;
    }
    // validate the domain part against our regex of pattern / permitted chars
    if (!preg_match($domain_regex, "${domain_part}")) {
        return FALSE;
    }
    // all tests passed - return true
    return TRUE;
}

//! Simple utility function which generates Recommended / Enforced label
/*!
 * @param   string  specifies the type of label
 * @param   bool    specifies whether or not to show the recommended label 
 *
 * @returns string  HTML string of the label, else &nbsp;
 *
 * @author Robert Bisewski <robertb@linuxmagic.com>
 *         Rares Harnagea <rares@linuxmagic.com>
 */
function generateLabel($type = NULL, $show_recommended = FALSE) {
    if ($type === NULL || !is_string($type)) {
        return "&nbsp;";
    }

    // Returns a recommended label
    if ($type == "recommended" && $show_recommended == TRUE) {
        return '<span class="isdefault">' . intl('recommended') . '</span>';

    // Returns a enforced label 
    } elseif ($type == "enforced") {
        return '<span title="' . intl('enforce_description') .
                '" class="ispolicy">' . intl('policy') . '</span>';
    } elseif ($type == "restricted") {
        return '<span title="' . intl('restricted_description') .
                '" class="ispolicy">' . intl('restricted') . '</span>';
    }
    // return space
    return "&nbsp;";
}

function handleTimezone() {
    $php_tz = getControl('php_tz', NULL);
    if (!is_null($php_tz) && in_array($php_tz, timezone_identifiers_list())) {
        date_default_timezone_set($php_tz);
    }
}

function sendAjaxResponse($status, $message = "", $data = array()) {
    // sanity check
    if (!isset($status, $message, $data)
        || !WV::isBool($status)
        || !WV::isString($message)
        || !WV::isArray($data)) {
        return false;
    }

    // build the AJAX response
    $response = array(
        'status'    => $status,
        'message'   => $message,
        'data'      => $data,
    );

    // send the AJAX response
    echo json_encode($response);
}

//! generates the URIs for all the provided resources
/*!
 * @param   array               the resources to generate URIs
 * @param   string              the path to the resource
 * @param   string              the version to use in the query for cache busting
 *
 * @return  array               the URIs for the resources
 */
function generateURIs($resources, $path, $version) {
    // sanity check
    if (!WV::isArrayNotEmpty($resources)
        || !WV::isStringNotEmpty($path)
        || !WV::isStringNotEmpty($version)) {
        return array();
    }

    // initialize the URIs array
    $URIs = array();

    // build and store the URI to each resource
    foreach ($resources as $resource) {
        if (!WV::isStringNotEmpty($resource)) {
            continue;
        }
        $URIs[] = "$path/$resource?$version";
    }

    // return the URIs
    return $URIs;
}
