<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Deion Yu <deiony@linuxmagic.com>
 *
 * $Id: lib.logsearch.php 8545 2018-08-24 21:09:49Z deiony $
 */
require_once('lib.logsearchbase.php');

/*
 * Each of the columns contain content which is stored in specific
 * indices. If you change the order of the columns you will need to
 * adjust the below values to the new order.
 * NOTE: the components of log line by numeric index:
 *       0 -> datetime      e.g. 2014-12-04 14:00:51
 *       1 -> type          e.g. PASS, BLOCK
 *       2 -> rule          e.g. country_auth
 *       3 -> ip            e.g. 192.168.1.1
 *       4 -> country code  e.g. CA, **
 *       5 -> auth user     e.g. test@linuxmagic.dev
 */
define('LS_AUTH_INDEX_DATETIME',    0);
define('LS_AUTH_INDEX_TYPE',        1);
define('LS_AUTH_INDEX_RULE',        2);
define('LS_AUTH_INDEX_IP',          3);
define('LS_AUTH_INDEX_HOSTNAME',    4);
define('LS_AUTH_INDEX_COUNTRY',     5);
define('LS_AUTH_INDEX_USER',        6);

//! the auth logsearch filter types
define('LS_AUTH_FILTER_ALL',        0);
define('LS_AUTH_FILTER_PASS',       1);
define('LS_AUTH_FILTER_BLOCK',      2);

//! regex pattern for log lines
define('LS_AUTH_PATTERN_FMT', '/^(%s \d+:\d+:\d+) magicspam\-[a-zA-Z0-9]+\[\d+\]: (%s)(?:\[(.+)\])?: ip=\[(%s):([^:]+):([A-Z*]{2})\],auth=<(%s)\>$/');

class AuthLogSearch extends LogSearchBase
{
    //! the auth user
    protected $auth_user;

    //! the ip address
    protected $ip_address;

    // the filter type
    protected $filter;

    //! class constructor
    public function __construct() {
        parent::__construct();
        $this->auth_user = NULL;
        $this->ip_address = NULL;
        $this->filter = LS_AUTH_FILTER_ALL;
    }

    //! create the pattern for searching a log file
    protected function createLogSearchPattern() {
        $type_regex = $this->getFilterRegex();

        $date_regex = '\d{4}\-\d+\-\d+';

        $ip_regex = '.+';
        if (isset($this->ip_address)) {
            $ip_regex = preg_replace('/\*/', '.*', $this->ip_address);
        }

        $auth_user_regex = '.*';
        if (isset($this->auth_user)) {
            $auth_user_regex = preg_replace('/\*/', '.*', $this->auth_user);
        }

        $this->log_line_regex = sprintf(LS_AUTH_PATTERN_FMT,
            $date_regex,
            $type_regex,
            $ip_regex,
            $auth_user_regex
        );

        return TRUE;
    }

    //! create the pattern for skipping a log file based on date (overloaded)
    protected function createDatePattern() {
        $this->date_regex = '/^(\d{4}\-\d+\-\d+)/';
        return TRUE;
    }

    //! retrieve authentication country statistics
    /*!
     * @param   int             number of log days to parse (0 = current day)
     * @param   int             number of countries to be displayed
     *
     * @returns array           array with all the country stats
     *
     * @author  Lucas Nincao <lucasn@linuxmagic.com>
     *
     * NOTE: returns PASS and BLOCK auth country statistics in the array
     */
    public function getTopAuthStatsByCountryRange($start = 0, $stop = 0 ,$numSelections = 5) {
        // mslog files dir
        $baseFile = LOGDIR . "msauthlog";

        // associtative arrays to store all the authentication stats by country
        $authStats = array('PASS' => array(),
                           'BLOCK' => array()
                           );

        // find all the desired msauthlog files
        $files = $this->findLogFilesRange($baseFile, $start, $stop);
        if ($files === FALSE) {
            return FALSE;
        }

        // create the log search regex pattern
        $this->createLogSearchPattern();

        // loop through each eligible magicspam auth log file
        foreach ($files as $file) {
            // open the log file with the reverse reader
            $reader = new ReverseReader($file, $this->read_buffer_size);

            // read in each line from the file in the reverse order
            while (($line = $reader->readLine()) !== FALSE) {

                // parse the log line into its individual components
                $row = $this->parseLogLine($line);
                if ($row === FALSE) {
                    continue;
                }

                // assign friendly names for easier manipluation
                $countryCode = $row[LS_AUTH_INDEX_COUNTRY];
                $authType = $row[LS_AUTH_INDEX_TYPE];

                // keep count of countries based on the auth type
                // NOTE: auth status will be either PASS or BLOCK
                $this->incrementStat($authStats[$authType], $countryCode);
            }

            unset($reader);
        }

       // sort and slice the array
       $this->prepareMultiArrayResults($authStats, $numSelections);

       return $authStats;
    }

    public function getTopAuthStatsByCountry($numSelections = 5) {
        $this->getTopAuthStatsByCountryRange(0, 0, $numSelections);
    }

    protected function incrementStat(&$stats, $stat) {
        $stats[$stat] = isset($stats[$stat]) ? $stats[$stat] + 1 : 1;
    }

    //! Sorts array by values and keys and slices to the requested number of entries
    /*!
     * @param   array           the multi array
     * @param   int             the requested number of entries
     *
     * @author  Lucas Nincao <lucasn@linuxmagic.com>
     *
     * NOTE: the array is sorted by values in descending order followed by
     *       keys in the ascending order
     */
     protected function prepareMultiArrayResults(&$arr, $num) {
        foreach ($arr as $key => $data) {
            if (!empty($data)) {
                array_multisort(array_values($arr[$key]), SORT_DESC,
                    array_keys($arr[$key]), SORT_ASC,
                    $arr[$key]);
                $arr[$key] = array_slice($arr[$key], 0, $num);
            }
        }
    }

    // Helper Functions
    protected function getFilterRegex() {
        switch($this->filter) {
            case LS_AUTH_FILTER_PASS:
                $regex = "PASS";
                break;
            case LS_AUTH_FILTER_BLOCK:
                $regex = "BLOCK";
                break;
            case LS_AUTH_FILTER_ALL:
            default:
                $regex = "PASS|BLOCK";
                break;
        }
        return $regex;
    }

    //! setter for auth user
    public function setAuthUser($auth_user) {
        // ensure the auth user is a non-empty string
        if (!isset($auth_user) || !WV::isStringNotEmpty($auth_user)) {
            return FALSE;
        }
        // trim and check to ensure the auth user was not just whitespace
        $auth_user = trim($auth_user);
        if ($auth_user === '') {
            return FALSE;
        }
        // set the auth user class variable
        $this->auth_user = $auth_user;
        return TRUE;
    }

    //! setter for ip address
    public function setIpAddress($ip_address) {
        // ensure the ip address is a non-empty string
        if (!isset($ip_address) || !WV::isStringNotEmpty($ip_address)) {
            return FALSE;
        }
        // trim and check to ensure the ip address was not just whitespace
        $ip_address = trim($ip_address);
        if ($ip_address === '') {
            return FALSE;
        }
        // set the ip address class variable
        $this->ip_address = $ip_address;
        return TRUE;
    }

    //! setter for filter
    public function setFilter($filter) {
        // ensure the filter is a positive numeric
        if (!isset($filter) || !WV::isIntPositiveOrZero($filter)) {
            return FALSE;
        }
        // convert numeric string into integer
        $filter = intval($filter);
        // ensure that the filter type is known
        if ($filter !== LS_AUTH_FILTER_ALL &&
            $filter !== LS_AUTH_FILTER_PASS &&
            $filter !== LS_AUTH_FILTER_BLOCK) {
            return FALSE;
        }
        // set the filter class variable
        $this->filter = $filter;
        return TRUE;
    }
}
