<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2014 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Sean West <sean@linuxmagic.com>
 *            Shaun Johnson <shaun@linuxmagic.com>
 *
 * $Id: lib.cmdwrapper.php 6285 2017-03-30 17:24:13Z shaun $
 */

/*
 *
 * This is a base class for use with any/all classes that
 * act as wrappers to a backend executable. This class simply offers
 * a common 'exec' for binaries. All other logic should be abstracted
 * in the extending classes.
 *
 * TODO: add methods for execWrite in the event we need to feed data via STDIN.
 *
 * This file is part of the MagicSpam(TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2011 All Rights Reserved
 *
 * Author(s): Sean West <sean@linuxmagic.com>
 *            Shaun Johnson <shaun@linuxmagic.com>
 */
class cmdWrapper {
    // executable binary path
    protected $bin;

    // buffer for the 'last' error message
    protected $lastError;

    //! validates and stores a command
    /*!
     * @param   string  command path
     *
     * Being a constructor, an Exception will be thrown if there are
     * any errors with object creation.
     *
     * @author    Sean West <sean@linuxmagic.com>
     *            Shaun Johnson <shaun@linuxmagic.com>
     */
    protected function __construct($command) {
        // ensure the command argument is a string and has actual content
        if (!is_string($command) || strlen($command) === 0) {
            throw new Exception("invalid command");
        }
        // ensure that the command exists, and can be executed.
        if (!is_executable($command)) {
            throw new Exception("command {$command} is not executable");
        }
        // store the command for future use.
        $this->bin = $command;
        // and init the error message buffer
        $this->lastError = "";
    }

    //! accessor to get the last error message.
    /*!
     * @returns string  the message buffer.
     *
     * @author    Sean West <sean@linuxmagic.com>
     *            Shaun Johnson <shaun@linuxmagic.com>
     */
    public function error() {
        return $this->lastError;
    }

    //! internal function for running the command.
    /*!
     * @param array $args Arguments to pass to the command. These should
     *   each indicate a parameter and not yet be escaped.
     * @param string &$buffer (optional) buffer for command output (or errors).
     *
     * @author Sean West <sean@linuxmagic.com>
     *         Shaun Johnson <shaun@linuxmagic.com>
     *         William Storey <william@linuxmagic.com>
     *
     * @returns integer the exit code of the program,
     *          or -1 if fatal errors preparing.
     */
    protected function execRead($args = NULL, &$buffer = NULL) {
        $result = "";
        // we trust that the safety checks on
        // the constructor ensure there is a valid command...
        $cmd = escapeshellcmd($this->bin);
        // append optional arguments as required.
        if ($args !== NULL && is_array($args)) {
            foreach ($args as $arg) {
                $cmd .= ' ' . self::escapeArgument($arg);
            }
        }
        $cmd .= ' 2>&1';

        // open the pipe
        $fh = popen($cmd, "r");
        if (!$fh) {
            if (!is_null($buffer)) {
                $buffer = "failed to run command $cmd";
            }
            return -1;
        }
        // read any/all output the command may give
        while ($line = fgets($fh)) {
            $result .= $line;
        }
        // and store it into the (optional) buffer.
        if (!is_null($buffer)) {
            $buffer = trim($result);
        }
        // get the exit code and close the pipe.
        $exit_code = pclose($fh);
        // done!
        return $exit_code;
    }

    //! Function to escape a string to be used as a shell argument.
    /*!
     * @param	string the argument that will be escaped
     *
     * @return	string the escaped argument
     *
     * NOTE: This function acts as a wrapper to PHP standard escapeshellarg.
     *       It exists for the purpose of working around Windows Specific
     *       quirks wherein '%' characters get replaced by a space.
     */
    private static function escapeArgument($argument) {
        // IF this installation is detected as being windows
        if (defined('ISWINDOWS') && ISWINDOWS === 1) {
            // replace % with a unique string that windows PHP won't try
            // to mess with...
            $hackstring = preg_replace('/%/', 'MSPCTHACK20170124', $argument);

            // now we escape the argument
            $escapedArg = escapeshellarg($hackstring);

            // and unhack our casts
            return preg_replace('/MSPCTHACK20170124/', '%', $escapedArg);
        }
        return escapeshellarg($argument);
    }
}
