<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2011-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *            Logan Borean <loganb@linuxmagic.com>
 *
 * $Id: lib.monthstats.php 4655 2015-01-19 23:34:05Z shaun $
 *
 * monthStats is intended to act as a simple(r) method for
 * interacting with the statistics as generated by MagicSpam
 * binaries. This class will handle the loading and rendering
 * of the GUI chart view.
 */
require_once('functions.php');
require_once('lib.rcpthourlycounter.php');

class HourlyStats {
    private $hourCounter;

    //! Initialize the hourly counter
    /*!
     *
     * @author Logan Borean <loganb@linuxmagic.com>
     */
    public function __construct() {
        $this->hourCounter = new RCPTHourlyCounter();
        $this->hourCounter->load();
    }

    //! Displays a data table related to hourly stats
    /*!
     * @author Logan Borean<loganb@linuxmagic.com>
     *
     * this method generates HTML to show a graph of stats including
     * the counters for ham, spam, and exemptions.
     *
     * note that even if there are no stats recorded yet for the month
     * we do still want to render a graph - just with nothing in it.
     */
    public function render() {
        $max = $this->hourCounter->get_greatest();

        $total_number = 0;
        $total_count = 0;
        $day_count = 0;

        // set the max 'height' for the table.
        // and the full expected widget width;
        $tableheight = 150;
        $tablewidth = 31 * 27;

        // set flags for detecting PHP timezone error
        $all_empty = TRUE;
        $all_wrong = TRUE;

        // render the outside wrapper
        echo "<div id=\"hourlyStats\">\n";
        for ($c = 23; $c >= 0; $c--) {
            $hour = date('G', strtotime("$c hours ago"));
            $hour_unix = date('U', strtotime("$c hours ago"));
                
            $dayDat = $this->hourCounter->get_value($hour, RCPT_TYPE_ALL);

            echo "<div id=\"row_{$c}\" class=\"hourlyStatDayRow\" onclick=\"getHourlyLogs($hour)\">\n";

            if (is_array($dayDat) && isset($dayDat['timestamp']) && abs($hour_unix - $dayDat['timestamp']) < 3600) {
                $curHour = date('ga', $dayDat['timestamp']);
                $all_wrong = FALSE;
            } else {
                // determine whether error was due to invalid counter information
                if (isset($dayDat['timestamp'])) {
                    $all_empty = FALSE;
                }

                // initialize an empty counter array for the hour
                $dayDat = array();
                $dayDat['timestamp'] = mktime($hour);
                $dayDat['exempt'] = 0;
                $dayDat['ip_ratelimit'] = 0;
                $dayDat['user_ratelimit'] = 0;
                $dayDat['spam'] = 0;
                $dayDat['ham'] = 0;

                $curHour = date('ga', $dayDat['timestamp']);
            }

            // the actual day of the month label
            echo "        <div class=\"hourlyStatDayRowLabel \" >"  .  $curHour . "</div>\n";
            // now.. render the inner 'colored' bars.
            $this->renderDayStat($dayDat, $max, $tableheight - 25);
            // and.. render the mouse hover tooltip.
            $this->renderToolTip($dayDat, $hour);
            echo "    </div> <!-- end row_{$hour} -->\n";

            if (is_array($dayDat)) {

                // Determines the day count
                $day_count = $dayDat['exempt'] + $dayDat['ip_ratelimit']
                           + $dayDat['user_ratelimit'] + $dayDat['spam']
                           + $dayDat['ham'];

                $total_number += $day_count;
                if ($day_count > 0) {
                    $total_count++;
                }
            }
        }
        echo "</div> <!-- end hourlyStats -->\n";

        // Renders the Monthly Stats Legend
        $average = ($total_count != 0) ? (int) ($total_number / $total_count) : 0;
        renderTemplate('hourlyStatsLegend', array(
            'average'      => $average,
            'title'        => intl('hourlyStatsLegendTitle'),
            'php_tz_error' => ($all_wrong && !$all_empty),
        ));
    }

    //! Get the width of the stats table, useful for aligning blurb / legend
    /*!
     * @author Robert Bisewski <robertb@linuxmagic.com>
     */
    public function getWidth() {
        return $this->num_records * 27;
    }

    //! create a table and insert mail data into it.
    /*!
     * @param $data array is inserted into two lists, ham for good, spam for bad
     * @param $max int data modifier
     * @param $maxheight int data modifier
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     */
    private function renderDayStat($data, $max, $maxheight) {
        if (!is_array($data) || !is_numeric($max) ||
            $max <= 0 || !is_numeric($maxheight) || $maxheight == 0) {
            return;
        }

        // Grabs each of data types
        $pct_ham       = intval($data['ham'] / $max * $maxheight);
        $pct_spam      = intval($data['spam'] / $max * $maxheight);
        $pct_exempt    = intval($data['exempt'] / $max * $maxheight);
        $pct_ratelimit = intval(($data['ip_ratelimit'] + $data['user_ratelimit']) / $max * $maxheight);

        // Checks if graph is set to display exemption / rate limits,
        // otherwise prints ham / spam stats
        echo "<div class=\"hourlyStatHamBar\" style=\"height: {$pct_ham}px; top: -{$pct_ham}px;\">&nbsp;</div>\n";
        echo "<div class=\"hourlyStatSpamBar\" style=\"height: {$pct_spam}px; top: -{$pct_spam}px;\">&nbsp;</div>\n";
        echo "<div class=\"hourlyStatExemptBar\" style=\"height: {$pct_exempt}px; top: -{$pct_exempt}px;\">&nbsp;</div>\n";
        echo "<div class=\"hourlyStatRatelimitBar\" style=\"height: {$pct_ratelimit}px; top: -{$pct_ratelimit}px;\">&nbsp;</div>\n";
    }

    //! displays the data in a table
    /*!
     * @params $data array good data and bad data
     * @params $day int day in question
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     */
    private function renderToolTip($data, $day) {
        if (!is_array($data) || !is_numeric($day)) {
            return;
        }
        // XXX: use of 'number_format' does not take client environment locale into account....
        $monthHeader = date('F j, ga', $data['timestamp']);
        $dataTable = new magicspamDataTable();
        $dataTable->addRow( array (
                                    intl('hourlyStatTotalLabel'),
                                    number_format($data['ham'] +
                                    $data['spam'] +
                                    $data['exempt'] +
                                    $data['ip_ratelimit'] +
                                    $data['user_ratelimit'])
                                  ) );
        $dataTable->addRow( array (intl('hourlyStatHamLabel'), number_format($data['ham'])));
        $dataTable->addRow( array (intl('hourlyStatSpamLabel'), number_format($data['spam'])));
        $dataTable->addRow( array (intl('hourlyStatExemptLabel'), number_format($data['exempt'])));
        $dataTable->addRow( array (intl('hourlyStatRatelimitLabel'), number_format($data['ip_ratelimit'] + $data['user_ratelimit'])));
        // TODO: shall we also add in ip/user ratelimit counts?

        // Alters the style of the tooltip to prevent it from reaching the end of the browser
        $tooltip_suffix = ($day > 25) ? "_left" : "";

        // Prints out the tool tip data
        echo "<div class=\"hourlystats_tooltip" . $tooltip_suffix;
        echo "\"> <!-- hourlystats_tooltip -->\n";
        echo "    <div class=\"tooltip_title\">$monthHeader</div>\n";
        echo $dataTable->getRenderString();
        echo "</div><!-- end hourlystats_tooltip-->\n";
    }

    //! Displays data in a table with a label attached
    /*!
     * @params $label the data's label
     * @params $val the number to be displayed
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     */
    private function statLine($label, $val) {
        if (!is_string($label) || !is_string($val)) {
            return "";
        }
        $line = sprintf("<div class=\"hourlyStatDayLine\"><span class=\"hourlyStatLabel\">%s: </span><span class=\"hourlyStatNumber\">%s</span></div>", $label, $val);
        return $line;
    }
}
