<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2014 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Sean West <sean@linuxmagic.com>
 *            Shaun Johnson <shaun@linuxmagic.com>
 *
 * $Id: lib.licensemanager.php 6308 2017-04-12 20:52:29Z shaun $
 *
 * This is a container to access license information.
 * This class allows for the reading of License information in conjunction with information
 * pulled from license manager based on the license code.  Licenses are described using 5 variables:
 * $state, $code, $expiry, $nextupdate, $requester.
 */
include_once('lib.cmdwrapper.php');

class LicenseManager extends cmdWrapper {
    // internal private singleton.
    static private $object = NULL;

    // array for understanding the license manager status codes
    static private $state_codes = array (
        0 => 'valid',
        1 => 'pending',
        2 => 'expired',
        3 => 'new',
        255 => 'corrupt',
    );

    // the full encoded license key
    private $code;
    // the expiry date of the license
    private $expiry;
    // the expected date for next update
    private $nextupdate;
    // the license requester (used for server backend data types)
    private $requester;
    // the current state (see state_codes above)
    private $state;
    // the product sku - used to determine and print friendly types.
    private $sku;
    // the license type
    private $license_type;

    //! class constructor
    /*!
     * @author    Sean West <sean@linuxmagic.com>
     *            Shaun Johnson <shaun@linuxmagic.com>
     */
    public function __construct() {
        // instantiate back to the parent.
        parent::__construct(LICENSE_MANAGER);
        // initialize the data
        $this->code = NULL;
        $this->expiry = NULL;
        $this->nextupdate = NULL;
        $this->requester = NULL;
        $this->sku = NULL;
        $this->license_type = NULL;
        $this->state = self::state('corrupt');

        // and load
        $this->load();
    }

    //! method to clear out existing loaded data.
    public function clear() {
        $this->code = NULL;
        $this->expiry = NULL;
        $this->nextupdate = NULL;
        $this->requester = NULL;
        $this->sku = NULL;
        $this->license_type = NULL;
        $this->state = self::state('corrupt');
    }

    //! static function to get the current singleton instance
    /*!
     * @returns LicenseManager  the active singleton instance.
     *
     * @author    Sean West <sean@linuxmagic.com>
     *            Shaun Johnson <shaun@linuxmagic.com>
     * If the instance is not yet created, then this will create it.
     */
    static public function getInstance() {
        if (is_null(self::$object)) {
            self::$object = new LicenseManager;
        }
        return self::$object;
    }

    //! access function to update the license with a optional new license key.
    /*!
     * @param   string  (optional) the license key to fetch.
     *
     * @returns integer the license state after update, or -1 on errors.
     *
     * @author    Sean West <sean@linuxmagic.com>
     *            Shaun Johnson <shaun@linuxmagic.com>
     * Errors will retain a detailed error message that can be fetched by
     * a call to the 'error()' function.
     */
    public function update($code = NULL) {
        $args = array("update");
        $buffer = "";
        $rv = -1;

        // append the optional code to update with.
        if (is_string($code) && strlen($code) > 0) {
            $args[] = $code;
        }
        // attempt to run the execution
        $rv = $this->execRead($args, $buffer);
        if ($rv !== 0) {
            $this->lastError = $buffer;
            return -1;
        }
        // then reload
        $this->load();

        // act of reload will set the internal state, so we return that.
        return $this->state;
    }

    //! access function to update the addon license with a optional new key
    /*!
     * @param   string  (optional) the license key to fetch.
     *
     * @returns integer the license state after update, or -1 on errors.
     *
     * @author    Sean West <sean@linuxmagic.com>
     *            Shaun Johnson <shaun@linuxmagic.com>
     * Errors will retain a detailed error message that can be fetched by
     * a call to the 'error()' function.
     */
    public function updateAddon($code = NULL) {
        $args = array("wlupdate");
        $buffer = "";
        $rv = -1;

        // append the optional code to update with.
        if (is_string($code) && strlen($code) > 0) {
            $args[] = $code;
        }
        // attempt to run the execution
        $rv = $this->execRead($args, $buffer);
        if ($rv !== 0) {
            $this->lastError = $buffer;
            return -1;
        }
        // then reload
        $this->load();

        // act of reload will set the internal state, so we return that.
        return $this->state;
    }

    //! access function to 'remove' an existing license (only used by freemium for now)
    public function removeLicense() {
        $args = array("remove");
        $buffer = "";
        $rv = -1;

        $rv = $this->execRead($args, $buffer);
        if ($rv !== 0) {
            $this->lastError = $buffer;
            return -1;
        }

        // then reload
        $this->load();

        // act of reload will set the internal state, so we return that.
        return $this->state;
    }

    //! access function to 'load' the current license data.
    /*!
     * @returns integer the current license status code OR -1 on errors.
     * @author    Sean West <sean@linuxmagic.com>
     *            Shaun Johnson <shaun@linuxmagic.com>
     *
     * Any errors that occur will store a message that can be retrieved
     * by a call to 'error()' function.
     */
    public function load() {
        $buffer = "";
        $rv = $this->execRead(array("data"), $buffer);
        $this->state = $rv;
        if ($rv === self::state('corrupt')) {
            if (strlen($buffer) > 0) {
                $this->lastError = $buffer;
            } else {
                $this->lastError = "License file is corrupt";
            }
            return -1;
        }
        $linearray = explode("\n", $buffer);
        foreach ($linearray as $line) {
            trim($line);
            $resarray = explode(":", $line);

            if (count($resarray) == 7) {
                $this->expiry = $resarray[0];
                $this->code = $resarray[2];
                $this->sku = (int)$resarray[3];
                $this->nextupdate = $resarray[4];
                $this->requester = $resarray[5];
                $this->license_type = $resarray[6];
            }
        }
        // fallback specific to 'PLESK' skus from embedded licenses that aren't
        // updated yet
        if ($this->license_type == 'unknown') {
            $numsku = (int)$this->sku;
            if ($numsku > 11001 && $numsku <= 11006) {
                $this->license_type = 'Monthly';
            } else if ($numsku >= 11010 && $numsku <= 11011) {
                $this->license_type = 'Annual';
            } else if ($numsku == 10001) {
                $this->license_type = 'Monthly';
            }
        }
        return $rv;
    }

    //! access function to 'load' the addon license data.
    /*!
     * @returns integer the current license status code OR -1 on errors.
     *
     * Any errors that occur will store a message that can be retrieved
     * by a call to 'error()' function.
     */
    public function loadAddon() {
        $buffer = "";
        $rv = $this->execRead(array("wldata"), $buffer);
        $this->state = $rv;
        if ($rv === self::state('corrupt')) {
            if (strlen($buffer) > 0) {
                $this->lastError = $buffer;
            } else {
                $this->lastError = "License file is corrupt";
            }
            return -1;
        }
        $linearray = explode("\n", $buffer);
        foreach ($linearray as $line) {
            trim($line);
            $resarray = explode(":", $line);

            if (count($resarray) == 7) {
                $this->expiry = $resarray[0];
                $this->code = $resarray[2];
                $this->sku = (int)$resarray[3];
                $this->nextupdate = $resarray[4];
                $this->requester = $resarray[5];
                $this->license_type = $resarray[6];
            }
        }
        return $rv;
    }

    //! accessor function to determine if the state is of type 'valid'
    /*!
     * @returns boolean true or false if the license is valid and not pending expiry or otherwise.
     * @author    Sean West <sean@linuxmagic.com>
     *            Shaun Johnson <shaun@linuxmagic.com>
     */
    public function isValid() {
        if ($this->stateType('valid') || $this->stateType('pending')) {
            return TRUE;
        }
        return FALSE;
    }

    //! accessor function to determine if the state is of type 'expired'
    /*!
     * @returns boolean true or false if the license exists and is expired.
     * @author    Sean West <sean@linuxmagic.com>
     *            Shaun Johnson <shaun@linuxmagic.com>
     */
    public function isExpired() {
        return $this->stateType('expired');
    }

    //! access function to determine if the state is of type 'pending' expiry
    /*!
     * @returns boolean
     * @author    Sean West <sean@linuxmagic.com>
     *            Shaun Johnson <shaun@linuxmagic.com>
     */
    public function isPending() {
        return $this->stateType('pending');
    }

    //! access function to determine if the state is of type 'new'
    /*!
     * @returns boolean
     * @author    Sean West <sean@linuxmagic.com>
     *            Shaun Johnson <shaun@linuxmagic.com>
     */
    public function isNew() {
        return $this->stateType('new');
    }

    //! access function to get the license key 'code'.
    /*!
     * @returns string  the internal code of the license or NULL if it isn't loaded yet.
     * @author    Sean West <sean@linuxmagic.com>
     *            Shaun Johnson <shaun@linuxmagic.com>
     */
    public function getCode() {
        return $this->code;
    }

    //! access function to get the expiry date.
    /*!
     * @returns string  a formatted date string 'Month day_of_month, YYYY', or NULL if not set.
     * @author    Sean West <sean@linuxmagic.com>
     *            Shaun Johnson <shaun@linuxmagic.com>
     *
     * TODO: this function needs review for flexibility towards i18n formats of date strings.
     */
    public function getExpiry() {
        if (is_numeric($this->expiry)) {
            return date('F j, Y', $this->expiry);
        }
        return NULL;
    }

    //! access function to get the next update date
    /*!
     * @returns  string  a formatted date string 'Month day_of_month, YYYY', or NULL if not set.
     *
     * @author    Sean West <sean@linuxmagic.com>
     *            Shaun Johnson <shaun@linuxmagic.com>
     * TODO: this function needs review for flexibility towards i18n formats of date strings.
     */
    public function getNextUpdate() {
        if (is_numeric($this->nextupdate)) {
            return date('F j, Y', $this->nextupdate);
        }
        return NULL;
    }

    //! access function to get the license requestor code (required by license manager)
    /*!
     * @returns string  the requester code, or NULL if not set or empty.
     * @author    Sean West <sean@linuxmagic.com>
     *            Shaun Johnson <shaun@linuxmagic.com>
     */
    public function getRequester() {
        if (!is_string($this->requester) || strlen($this->requester) == 0) {
            return NULL;
        }
        return $this->requester;
    }

    //! access function to get the license sku type
    /*!
     * @returns char    the single letter code for use by the rest of the interface for processing.
     * @author    Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getSKUType() {
        return $this->license_type;
    }

    //! utility function to make comparisons of the license state easier.
    /*!
     * @param   string  the 'type' of the state to check.
     *
     * @returns boolean if you need this explained you probably shouldn't be reading this....
     * @author    Sean West <sean@linuxmagic.com>
     *            Shaun Johnson <shaun@linuxmagic.com>
     */
    private function stateType($type) {
        if (!is_string($type) || strlen($type) == 0) {
            return FALSE;
        }
        return ($this->state == self::state($type));
    }

    //! utility function to convert a string state to the numeric representation or vice versa.
    /*!
     * @param   mixed   the state string or numeric code to convert.
     *
     * @returns mixed   the converted value or FALSE on error
     * @author    Sean West <sean@linuxmagic.com>
     *            Shaun Johnson <shaun@linuxmagic.com>
     */
    static private function state($val) {
        // numeric code
        if (is_numeric($val)) {
            // run an array safety check
            if (isset(self::$state_codes[$val])) {
                // and return it if found
                return self::$state_codes[$val];
            } else {
                // otherwise return FALSE
                return FALSE;
            }
        } else if (is_string($val)) {
            // array_search returns FALSE if not found.
            return array_search($val, self::$state_codes);
        }
        // not a number, not a string, then this was misused.
        return FALSE;
    }
}
