<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2014-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Robert Bisewski <robertb@linuxmagic.com>
 *            Sean West <sean@linuxmagic.com>
 *            Shaun A Johnson <shaun@linuxmagic.com>
 *            Peter Chung <peterc@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: lib.logsearch.php 8545 2018-08-24 21:09:49Z deiony $
 */
require_once('lib.reversereader.php');
require_once('wizard/lib.wizard_validator.php');

//! the buffer size when reading logs
define('LS_READ_BUFFER_SIZE', 8192);
define('SEARCH_RESULTS_SANITY', 5000);

//! the max lengths for log search input fields
define('ADDRESS_SEARCH_MAX', 1024);
define('LSM_IP_ADDRESS_SEARCH_MAX', 40);
define('RESULTS_SEARCH_MAX', 6);

abstract class LogSearchBase
{
    // the read buffer size
    protected $read_buffer_size;

    //! the log line regex
    protected $log_line_regex;

    //! the date regex
    protected $date_regex;

    //! the date strings
    protected $startDate;

    protected $endDate;

    //! create the pattern for searching a log file
    abstract protected function createLogSearchPattern();

    //! class constructor
    public function __construct() {
        $this->read_buffer_size = LS_READ_BUFFER_SIZE;
        $this->log_line_regex = NULL;
        $this->date_regex = NULL;
        $this->starDate = NULL;
        $this->endDate = NULL;
    }

    //! create the pattern for skipping a log file based on date
    protected function createDatePattern() {
        $this->date_regex = NULL;
        return TRUE;
    }

    //! parse log files for the most recent filtered matches
    /*!
     * @param   string          the base name of the log file to parse
     * @param   integer         the number of log lines to parse, -1 for unlimited
     * @param   integer         the number of days back to parse
     * @param   array           the array of parsed log lines
     * @param   boolean         whether there were more results than requested
     *
     * @returns boolean         whether the log file was parsed successfully
     */
    public function parseLogFiles($logFile, $logMaxRows, $maxDays, &$data, &$moreThanRequested) {
        // sanity check
        if (!WV::isStringNotEmpty($logFile)
            || !WV::isInt($logMaxRows)
            || !WV::isIntPositive($maxDays)) {
            return FALSE;
        }

        // set the referenced variables to the default value
        $data = array();
        $moreThanRequested = FALSE;

        // find the log files ordered newest to oldest
        $files = $this->findLogFilesRange($logFile, 0, $maxDays);
        if ($files === FALSE) {
            return FALSE;
        }

        // not an error if we didn't find any log files
        if (empty($files)) {
            return TRUE;
        }

        // create the log search regex pattern
        $this->createLogSearchPattern();
        $this->createDatePattern();
        
        $start_date = strtotime($this->startDate);
        $end_date = strtotime($this->endDate);

        // and now we loop through the files
        foreach ($files as $file) {
            // no need to process the next log file as we have all we need
            if ($moreThanRequested === TRUE) {
                break;
            }

            // determine whether to skip log file based on the date
            if (isset($this->date_regex, $this->startDate)) {
                if ($this->skipLogFile($file)) {
                    continue;
                }
            }

            // read lines one by one moving backwards through the file
            $reader = new ReverseReader($file, $this->read_buffer_size);
            while (($line = $reader->readLine()) !== FALSE) {
                // no need to parse the next log line as we have all we need
                if ($logMaxRows !== -1 && count($data) >= $logMaxRows) {
                    $moreThanRequested = TRUE;
                    break;
                }

                // parse the log line into its individual components
                $row = $this->parseLogLine($line);
                if ($row === FALSE) {
                    // do not terminate as this process is a best effort
                    continue;
                }

                // if at least one date is passed in
                // check if each date is within the range passed in
                if (isset($this->startDate) && isset($row[0])) {
                    $row_date_tmp = strtotime($row[0]);

                    // check if row[0] has a valid date
                    if ($row_date_tmp !== FALSE) {
                        $row_date = strtotime(date('Y-m-d', $row_date_tmp)); 
                    } else {
                        continue;  
                    }

                    // if date is not in the given range, skip to next row
                    if (($row_date < $start_date) || ($row_date > $end_date)) { 
                        continue;
                    }
                }

                // store the parsed log line with the newest entry on top
                $data[] = $row;
            }
            unset($reader);
        }

        return TRUE;
    }

    //! find all the log files within the range of days
    /*!
     * @param   string          the base name of the log file
     * @param   integer         the start of the range of days
     * @param   integer         the end of the range of days
     *
     * @returns mixed           array of log files within the range, FALSE on error
     */
    protected function findLogFilesRange($logFile, $start, $end)
    {
        // sanity check
        if (!WV::isStringNotEmpty($logFile)
            || !WV::isIntPositiveOrZero($start)
            || !WV::isIntPositiveOrZero($end)
            || $start > $end) {
            return FALSE;
        }

        // initialize the array of log files to return
        $files = array();

        // calculate the range between the start and end
        $range = $end - $start + 1;

        // loop through the desired log files
        for ($i = $start; $i < $start + $range; $i++) {
            // build path to desired log file
            $path = $logFile;
            if ($i > 0) {
                $path .= ".$i";
            }

            // ensure that the log file exists
            if (!file_exists($path)) {
                continue;
            }

            // the log file is eligible
            $files[] = $path;
        }

        // we have got our array of log files that meet our criteria
        return $files;
    }

    //! parse log line into components according to log line regex
    /*!
     * @param   string          the log line to parse
     *
     * @returns mixed           the array of components, FALSE on error
     */
    protected function parseLogLine($line) {
        // ensure that the log line is a string
        if (!WV::isString($line)) {
            return FALSE;
        }

        // trim the log line and ignore when empty
        $line = trim($line);
        if (!strlen($line)) {
            return FALSE;
        }

        // perform regex match on log line with standard log line pattern
        if (preg_match($this->log_line_regex, $line, $matches) != 1) {
            return FALSE;
        }

        // remove the first capture group as it contains the entire log line
        array_shift($matches);

        // return the rest of the capture groups
        return $matches;
    }

    //! determine whether to skip log file based on date
    /*!
     * @param   string          the log file
     *
     * @returns boolean         whether to skip the log file
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    protected function skipLogFile($file) {
        // ensure that the log file is a non-empty string
        if (!WV::isStringNotEmpty($file)) {
            return FALSE;
        }
        // initialize reverse reader to get the last line
        $reader = new ReverseReader($file, $this->read_buffer_size);
        $reader->readLine();

        // retrieve the first (oldest) and last (newest) line in the log file
        $first_line = trim(fgets(fopen($file, 'r')));
        $last_line = trim($reader->readLine());
        if (!WV::isStringNotEmpty($first_line) || !WV::isStringNotEmpty($last_line)) {
            return FALSE;
        }

        // perform regex match on log lines with the date pattern
        if (preg_match($this->date_regex, $first_line, $first_matches) != 1
            || preg_match($this->date_regex, $last_line, $last_matches) != 1) {
            return FALSE;
        }

        // ensure that the expected capture groups are present
        if (sizeof($first_matches) != 2 || sizeof($last_matches) != 2) {
            return FALSE;
        }

        // convert dates into timestamp
        $old_date = strtotime($first_matches[1]);
        $new_date = strtotime($last_matches[1]);

        // ensure these are valid timestamp dates
        if ($old_date === FALSE || $new_date === FALSE) {
            return FALSE;
        }

        $start_date = strtotime($this->startDate);
        $end_date = strtotime($this->endDate);

        if ($start_date === FALSE || $end_date === FALSE) {
            return FALSE;
        }

        // return whether to skip the log file
        return ($old_date > $end_date || $start_date > $new_date);
    }

    //! setter for start and end dates
    public function setDate($startDate, $endDate = NULL) {
        // ensure the start date is a non-empty string
        if (!isset($startDate) || !WV::isStringNotEmpty($startDate)) {
            return FALSE;
        }  

        if (is_null($endDate)) {
            $endDate = $startDate; 
        } else if (!WV::isStringNotEmpty($endDate)) {
            return FALSE;        
        }

        // set the end date
        // set the start date
        $this->startDate = $startDate;
        $this->endDate = $endDate;
        return TRUE;
    }

}
