<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: lib.module.php 10931 2020-01-09 01:31:43Z deiony $
 *
 * This is the base definition for all top level modules in MagicSpam.
 * This class must never be used directly but rather be extended to define a
 * working whole.
 */
abstract class Module {
    //! variable to determine the internal 'label' that is used to load locales etc.
    protected $label;

    //! variable to determine if the module should render a menu link.
    protected $onMenu;

    //! variable for any submodules that may exist within this one.
    protected $submodules;

    //! internal variable for holding per module warnings
    protected $warning;

    //! internal variable for holding per module notices
    protected $notice;

    //! array of i18n labels to provide translations to javascript.
    protected $_i18n_js = array();

    //! variable to determine whether the module is migratable (default: FALSE)
    protected $migratable;

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     */
    public function __construct($l, $m) {
        $this->label = "";
        $this->warning = NULL;
        $this->notice = NULL;
        $this->submodules = array();
        if (is_string($l)) {
            $this->label = $l;
        }
        $this->onMenu = $m;
        $this->migratable = FALSE;
    }

    //! method to get the internal label of the instance.
    /*!
     * @returns string  the internal label
     */
    public function getLabel() {
        return $this->label;
    }

    //! method to determine if this module should include a link on the menu.
    /*!
     * @returns boolean
     */
    public function onMenu() {
        return $this->onMenu;
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     */
    abstract public function actionProcessor($action, &$errors, &$notices);

    //! method to render the full page module.
    abstract public function renderModule();

    //! any ajax calls should be handle by this method
    /*
     * @param array $modules the global modules array
     * @param string $controller_file the filename of the controller
     *
     * NOTES: Any submodules that has ajax calls implement this method to handle the ajax calls
     */
    public function renderOutput($modules) {
        $subModule = isset($_POST['submodule']) ? $_POST['submodule'] : NULL;
        if ( $subModule && isset($modules[$subModule]) ) {
            $modules[$subModule]->renderOutput($modules);
        }
    }

    //! generate HTML for this module/sub-module to provide i18n to javascript.
    /*!
     * @return void
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * SIDE EFFECT: Outputs HTML to stdout
     *
     * to provide i18n translations to javascript we generate <script>
     * elements.
     */
    public function renderI18nJS() {
        $html = '';
        $html .= $this->i18nJS();
        echo $html;
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    abstract public function getCSS();

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    abstract public function getJS();

    //! internal utility function for merging 'sub' modules CSS
    /*!
     * @param   string  the name of the submodule 
     *
     * @returns array   an array of all of the CSS files to include as dictated by submodules.
     */
    protected function subModuleCSS($submodule = NULL) {
        global $modules;

        // sanity check
        if (!WV::isArray($this->submodules) || !WV::isArray($modules)) {
            return array();
        }   

        // retrieve css for a specific submodule
        if (isset($submodule, $modules[$submodule]) && in_array($submodule, $this->submodules)) {
            return $modules[$submodule]->getCSS();
        }   

        // retrieve css for all the submodules
        $css = array();
        foreach ($this->submodules as $module) {
            if (isset($modules[$module])) {
                $css = array_merge($css, $modules[$module]->getCSS());
            }   
        }   
        return $css;
    }

    //! internal utility function for merging 'sub' modules JS
    /*!
     * @param   string  the name of the submodule 
     *
     * @returns array   an array of all of the JS files to include as dictated by submodules.
     */
    protected function subModuleJS($submodule = NULL) {
        global $modules;

        // sanity check
        if (!WV::isArray($this->submodules) || !WV::isArray($modules)) {
            return array();
        }

        // retrieve js for a specific submodule
        if (isset($submodule, $modules[$submodule]) && in_array($submodule, $this->submodules)) {
            return $modules[$submodule]->getJS();
        }

        // retrieve js for all the submodules
        $js = array();
        foreach ($this->submodules as $module) {
            if (isset($modules[$module])) {
                $js = array_merge($js, $modules[$module]->getJS());
            }
        }
        return $js;
    }

    //! generate a <script> element with requested i18n key/values.
    /*!
     * @return string HTML
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * we use this method to make i18n translations available to javascript.
     *
     * TODO: can we depend that json_encode() is available? PHP5.2.0+ according
     *   to php.net...
     */
    protected function i18nJS() {
        global $modules;

        // recursively call our submodules. submodules can be more than one
        // level deep (settings).
        $html = '';
        if (is_array($this->submodules) && is_array($modules)) {
            foreach ($this->submodules as $submoduleName) {
                if (!array_key_exists($submoduleName, $modules)) {
                    continue;
                }
                $submodule = $modules[$submoduleName];
                $html .= $submodule->i18nJS();
            }
        }
        // build the keys/values as an associative array that we'll convert
        // to json.
        $i18n = array();
        foreach ($this->_i18n_js as $label) {
            // NOTE: we require that label and its translation are utf8 encoded
            //   prior to json_encode().
            $i18n[$label] = utf8_encode(intl($label));
        }
        // if there are none, then we don't need to build anything.
        if (count($i18n) === 0) {
            return $html;
        }

        // generate our html.
        $html .= '<script type="text/javascript">' . "\n";
        // set up the object that holds our labels/translations if it's
        // not been setup before.
        $html .= 'if (!MS) { var MS = {}; }' . "\n";
        $html .= 'if (!MS.i18n) { MS.i18n = {}; }' . "\n";
        // get the json representation.
        // NOTE: we must escape '</' as '</script>' could end our
        //   script element. json_encode() does this for us.
        $i18nJson = json_encode($i18n);
        if ($i18nJson === FALSE) {
            $html .= '// json encoding error' . "\n";
            $html .= '</script>' . "\n";
            return $html;
        }
        // now merge the values into the MS.i18n object as properties.
        //
        // why am I merging this way and not just directly assigning
        // the object we made with json_encode() to MS.i18n? because I want
        // to consider the case that this method may have been called multiple
        // times by multiple modules in a single document, and so if we
        // directly assigned it, we'd be overwriting those from the last call.
        //
        // wrap everything up in a function so we don't clobber global scope.
        $html .= '(function() {' . "\n";
        $html .= '    var i18n = ' . $i18nJson . ';' . "\n";
        $html .= '    for (var label in i18n) {' . "\n";
        $html .= '        if (!i18n.hasOwnProperty(label)) {' . "\n";
        $html .= '            continue;' . "\n";
        $html .= '        }' . "\n";
        $html .= '        MS.i18n[label] = i18n[label];' . "\n";
        $html .= '    }' . "\n";
        $html .= '})();' . "\n";

        $html .= '</script>' . "\n";
        return $html;
    }

    //! determines whether the module is migratable
    /*!
     * @returns bool  whether the module is migratable
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    public function isMigratable() {
        return $this->migratable;
    }

    //! retrieve the migratable configurations of the module and its submodules
    /*!
     * @returns mixed the list of migratable configurations or FALSE if none
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     *
     * NOTE: modules that do not have submodules and performs the setting and
     *       reading of the configurations should override this function with
     *       logic for getting the migration configurations
     */
    public function getMigrationData() {
        // the list of all loaded Module objects
        global $modules;

        // the list of migratable configurations
        $configurations = array();

        // go through each submodule within the module
        foreach ($this->submodules as $submodule) {
            // skip submodules that are not loaded or migratable
            if (!array_key_exists($submodule, $modules)
                || !$modules[$submodule]->isMigratable()) {
                continue;
            }

            // retrieve the configuration of the migratable submodule
            $configuration = $modules[$submodule]->getMigrationData();
            if ($configuration === FALSE) {
                // XXX: maybe error handling, halt generate process?
                continue;
            }

            // append the configuration to the list of configurations
            $configurations[$submodule] = $configuration;
        }

        // return the list of migratable configurations or FALSE if none
        return !empty($configurations) ? $configurations : FALSE;
    }

    //! loads the migratable configurations of the module and its submodules
    /*!
     * @param   array  the list of submodules and their configuration
     *
     * @returns bool   whether the configurations were loaded successfully
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     *
     * NOTE: modules that do not have submodules and performs the setting and
     *       reading of the configurations should override this function with
     *       logic for loading the migration configurations
     */
    public function loadMigrationData($configurations) {
        // the list of all loaded Module objects
        global $modules;
        
        // sanity check
        if (!isset($configurations) || !WV::isArrayNotEmpty($configurations)) {
            return FALSE;
        }

        // go through the list of submodules and their configuration
        foreach ($configurations as $submodule => $configuration) {
            // skip submodules that are not loaded or migratable
            if (!array_key_exists($submodule, $modules)
                || !$modules[$submodule]->isMigratable()) {
                continue;
            }

            // load the confiugration for the migratable submodule
            $rv = $modules[$submodule]->loadMigrationData($configuration);
            if (!$rv) {
                // XXX: what do? best-attempt or do we halt?
            }
        }

        // success, the migratable configurations have been loaded
        return TRUE;
    }
}
