<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2011-2014 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * $Id: lib.monthstats.php 4655 2015-01-19 23:34:05Z shaun $
 *
 * monthStats is intended to act as a simple(r) method for
 * interacting with the statistics as generated by MagicSpam
 * binaries. This class will handle the loading and rendering
 * of the GUI chart view.
 */
require_once('functions.php');
require_once('lib.rcptcounter.php');

class monthStats {
    private $rcptcounter;
    private $month;
    private $monthMap;
    private $year;
    private $num_records;

    //! initialized the data related to date, also a constructor
    /*!
     * @params numeric $month The month to initialise for.
     * @params array $monthMap This is an associative array where the key
     *   is numeric and it associates with an internationalised month name.
     *   For example, 12 => December. See where we instantiate this class
     *   in monthStatsModule for an example.
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     */
    public function __construct($month, &$monthMap) {
        // TODO: parameter check? should we proceed with anything if not
        //   valid?

        $this->rcptcounter = new RCPTCounter($month);
        $timestamp = 0;
        if ($this->rcptcounter->load()) {
            // grab the first date.. that has data...
            $timestamp = 0;
            $c = 0;
            while ($c < 31 && ($timestamp == 0 || date('n', $timestamp) != $month)) {
                $timestamp = $this->rcptcounter->get_date($c);
                $c++;
            }
            // if there is nothing to load from, assume right now.
            if ($timestamp === FALSE || $timestamp == 0) {
                $timestamp = date('U');
            }
            // store for later use.
            $this->month = date('n', $timestamp);
            $this->year = date('Y', $timestamp);
        } else {
            $this->month = $month;
            $this->year = date('Y');
            $timestamp = strtotime($this->year . '-' . $this->month . '-01');
        }
        $this->monthMap = $monthMap;

        // we want the number of records to equal the number of days in the month.
        $this->num_records = date('t', $timestamp);
    }

    //! Displays a data table related to month stats
    /*!
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     *
     * this method generates HTML to show a graph of stats including
     * the counters for ham, spam, and exemptions.
     *
     * note that even if there are no stats recorded yet for the month
     * we do still want to render a graph - just with nothing in it.
     */
    public function render() {
        $max = $this->rcptcounter->get_greatest();

        $total_number = 0;
        $total_count = 0;
        $day_count = 0;

        // set the max 'height' for the table.
        // and the full expected widget width;
        $tableheight = 150;
        $tablewidth = 31 * 27;

        // render the outside wrapper
        echo "<div id=\"monthStats\">\n";
        for ($c = 1; $c <= 31; $c++) {
            $dayDat = $this->rcptcounter->get_value($c, RCPT_TYPE_ALL);

            // then the div for the 'row' (eg: day of the month box)
            // adjusts the calender based on month size
            if ($c <= $this->num_records) {
                echo "<div id=\"row_{$c}\" class=\"monthStatDayRow\">\n";
            } else {
                echo "<div id=\"row_{$c}\" class=\"monthStatDayRowDisabled\">\n";
            }

            // the actual day of the month label
            echo "        <div class=\"monthStatDayRowLabel\">{$c}</div>\n";
            // now.. render the inner 'colored' bars.
            $this->renderDayStat($dayDat, $max, $tableheight - 25);
            // and.. render the mouse hover tooltip.
            $this->renderToolTip($dayDat, $c);
            echo "    </div> <!-- end row_{$c} -->\n";

            if (is_array($dayDat)) {

                // Determines the day count
                $day_count = $dayDat['exempt'] + $dayDat['ip_ratelimit']
                           + $dayDat['user_ratelimit'] + $dayDat['spam']
                           + $dayDat['ham'];

                $total_number += $day_count;
                if ($day_count > 0) {
                    $total_count++;
                }
            }
        }
        echo "</div> <!-- end monthStats -->\n";

        // Renders the Monthly Stats Legend
        $average = ($total_count != 0) ? (int) ($total_number / $total_count) : 0;
        renderTemplate('monthStatsLegend', array(
            'average' => $average,
            'title'   => intl('monthStatsLegendTitle'),
        ));
    }

    //! Get for the month variable
    /*!
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getMonth() {
        return $this->month;
    }

    //! Get for the year variable
    /*!
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getYear() {
        return $this->year;
    }

    //! Get the width of the stats table, useful for aligning blurb / legend
    /*!
     * @author Robert Bisewski <robertb@linuxmagic.com>
     */
    public function getWidth() {
        return $this->num_records * 27;
    }

    //! create a table and insert mail data into it.
    /*!
     * @param $data array is inserted into two lists, ham for good, spam for bad
     * @param $max int data modifier
     * @param $maxheight int data modifier
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     */
    private function renderDayStat($data, $max, $maxheight) {
        if (!is_array($data) || !is_numeric($max) ||
            $max <= 0 || !is_numeric($maxheight) || $maxheight == 0) {
            return;
        }

        // Grabs each of data types
        $pct_ham       = intval($data['ham'] / $max * $maxheight);
        $pct_spam      = intval($data['spam'] / $max * $maxheight);
        $pct_exempt    = intval($data['exempt'] / $max * $maxheight);
        $pct_ratelimit = intval(($data['ip_ratelimit'] + $data['user_ratelimit']) / $max * $maxheight);

        // Checks if graph is set to display exemption / rate limits,
        // otherwise prints ham / spam stats
        echo "<div class=\"monthStatHamBar\" style=\"height: {$pct_ham}px; top: -{$pct_ham}px;\">&nbsp;</div>\n";
        echo "<div class=\"monthStatSpamBar\" style=\"height: {$pct_spam}px; top: -{$pct_spam}px;\">&nbsp;</div>\n";
        echo "<div class=\"monthStatExemptBar\" style=\"height: {$pct_exempt}px; top: -{$pct_exempt}px;\">&nbsp;</div>\n";
        echo "<div class=\"monthStatRatelimitBar\" style=\"height: {$pct_ratelimit}px; top: -{$pct_ratelimit}px;\">&nbsp;</div>\n";
    }

    //! displays the data in a table
    /*!
     * @params $data array good data and bad data
     * @params $day int day in question
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     */
    private function renderToolTip($data, $day) {
        if (!is_array($data) || !is_numeric($day)) {
            return;
        }
        // XXX: use of 'number_format' does not take client environment locale into account....
        $monthName = $this->monthMap[$this->month];
        $monthHeader = sprintf("%s %02d", $monthName, $day);
        $dataTable = new magicspamDataTable();
        $dataTable->addRow( array (
                                    intl('monthStatTotalLabel'),
                                    number_format($data['ham'] +
                                    $data['spam'] +
                                    $data['exempt'] +
                                    $data['ip_ratelimit'] +
                                    $data['user_ratelimit'])
                                  ) );
        $dataTable->addRow( array (intl('monthStatHamLabel'), number_format($data['ham'])));
        $dataTable->addRow( array (intl('monthStatSpamLabel'), number_format($data['spam'])));
        $dataTable->addRow( array (intl('monthStatExemptLabel'), number_format($data['exempt'])));
        $dataTable->addRow( array (intl('monthStatRatelimitLabel'), number_format($data['ip_ratelimit'] + $data['user_ratelimit'])));
        // TODO: shall we also add in ip/user ratelimit counts?

        // Alters the style of the tooltip to prevent it from reaching the end of the browser
        $tooltip_suffix = ($day > 25) ? "_left" : "";

        // Prints out the tool tip data
        echo "<div class=\"monthstats_tooltip" . $tooltip_suffix;
        echo "\"> <!-- monthstats_tooltip {$monthName} {$day} -->\n";
        echo "    <div class=\"tooltip_title\">$monthHeader</div>\n";
        echo $dataTable->getRenderString();
        echo "</div><!-- end monthstats_tooltip {$monthName} {$day} -->\n";
    }

    //! Displays data in a table with a label attached
    /*!
     * @params $label the data's label
     * @params $val the number to be displayed
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     */
    private function statLine($label, $val) {
        if (!is_string($label) || !is_string($val)) {
            return "";
        }
        $line = sprintf("<div class=\"monthStatDayLine\"><span class=\"monthStatLabel\">%s: </span><span class=\"monthStatNumber\">%s</span></div>", $label, $val);
        return $line;
    }
}
