<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Product line and is
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Sean West <sean@linuxmagic.com>
 *            Shaun A. Johnson <shaun@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: lib.msnotifications.php 10395 2019-09-16 18:11:15Z deiony $
 *
 * This is a wrapper class intended to retrieve system alerts
 * as posted by MagicSpam maintainers related to critical updates
 * etc.
 */
require_once(PHPDIR . 'lm-notification-client/class.LMNotifications.php');

class MSNotifications extends LMNotifications {
    //! integer. the feed id we work on.
    private $_feed_id;

    //! string. an error message with more details when an error occurs.
    public $_error;

    //! class constructor.
    /*!
     * @param int $feedID The feed id to work on.
     *
     * @author Sean West <sean@linuxmagic.com>
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     *
     * @constructor
     *
     * SIDE EFFECT: this constructor will append messages to the global 'errors'
     *   list if there are any issues with the parent constructor.
     */
    public function __construct($feedID) {
        global $errors;
        if (!isset($feedID) || !is_int($feedID)) {
            $errors[] = intl('msn_invalid_feed');
            return;
        }
        try {
            parent::__construct($feedID,
                'https://secure.wizard.ca/notifications/', CONTROLDIR);
        } catch (Exception $e) {
            $errors[] = $e->getMessage();
        }
        $this->_feed_id = $feedID;
    }

    //! method to get the number of unread notifications.
    /*!
     * @returns integer the number of unread notifications.
     *
     * @author Sean West <sean@linuxmagic.com>
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     */
    public function countUnread() {
        // check and and see if there are new notifications to grab.
        $this->getRecent();
        // NOTE: timestamp 0 to get all cached entries.
        return count($this->getLocal()->getEntries(0));
    }

    //! method to get the number of read notifications.
    /*!
     * @returns integer the number of read notifications.
     *
     * @author Sean West <sean@linuxmagic.com>
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     */
    public function countRead() {
        // check and and see if there are new notifications to grab.
        $this->getRecent();
        // NOTE: timestamp 0 to get all cached entries.
        return count($this->getLocalRead()->getEntries(0));
    }

    //! method to get the list of entries that have not yet been flagged as
    // 'read'.
    /*!
     * @returns array   list of the entries.
     *
     * @author Sean West <sean@linuxmagic.com>
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     */
    public function getUnreadEntries() {
        // check and see if there are new notifications to grab.
        $this->getRecent();
        // NOTE: timestamp 0 to get all cached entries.
        return $this->getLocal()->getEntries(0);
    }

    //! method to get the list of entries that have been flagged as read.
    /*!
     * @returns array   list of the entries;
     *
     * @author Sean West <sean@linuxmagic.com>
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     */
    public function getReadEntries() {
        // NOTE: timestamp 0 to get all cached entries.
        return $this->getLocalRead()->getEntries(0);
    }

    //! set a specific feed item as read.
    /*!
     * @param int $timestamp The timestamp of the item to set read. Note at
     *   this time this is what we use to uniquely identify a feed.
     *
     * @return bool True if success, false if failure.
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * we remove the feed item from the list of unread items, and add it to
     * te list of read items.
     *
     * we then save both lists.
     *
     * NOTE: we will set the instance variable $_error to give more details
     *   on error.
     *
     * TODO: these errors are not yet i18n'd
     */
    public function setItemRead($timestamp) {
        $this->_error = NULL;

        if (!isset($timestamp) || !is_int($timestamp)) {
            $this->_error = 'Invalid timestamp.';
            return FALSE;
        }
        // find the feed item in the unread items list, and rebuild its list of
        // items without the item.
        $unreadRepo = $this->getLocal();
        if (!$unreadRepo) {
            $this->_error = 'Local unread repository is not available.';
            return FALSE;
        }
        // NOTE: timestamp 0 to get all cached entries.
        $unreadEntries = $unreadRepo->getEntries(0);
        $unreadEntriesNew = array();
        $entryToSetRead = NULL;
        foreach ($unreadEntries as $entry) {
            if ($entry->timestamp == $timestamp) {
                $entryToSetRead = $entry;
                continue;
            }
            // NOTE: array must be associative by timestamp because that's what
            //   we expect to setEntries().
            $unreadEntriesNew[$entry->timestamp] = $entry;
        }
        // if we can't find it then there's nothing to do.
        if (!$entryToSetRead) {
            $this->_error = 'Unread entry not found with timestamp '
                . $timestamp . '.';
            return FALSE;
        }
        // now replace the set of 'unread' entries with an array without this
        // one.
        $unreadRepo->setEntries($unreadEntriesNew);

        // now add the item to the read repo.
        $readRepo = $this->getLocalRead();
        if (!$readRepo) {
            $this->_error = 'Local read repository is not available.';
            return FALSE;
        }
        // NOTE: timestamp 0 to get all entries. here we just want to load the
        //   entries (so when we save they will be retained)... we don't need to
        //   actually store the result.
        $readRepo->getEntries(0);
        // NOTE: addEntries() takes care of enforcing uniqueness for us (because
        //   it keys by timestamp).
        // NOTE: addEntries() expects to take an array, hence the kind of odd
        //   parameter.
        $readRepo->addEntries(array($entryToSetRead->timestamp => $entryToSetRead));
        // save the repo lists.
        if (!$readRepo->save() || !$unreadRepo->save()) {
            $this->_error = 'Failed to add entry to the read repository';
            return FALSE;
        }
        return TRUE;
    }

    //! utility function to fetch new notifications from upstream IF the last
    // check time exceeds 1 hour.
    /*!
     * @returns array   list of the most recent entries.
     *
     * @author Sean West <sean@linuxmagic.com>
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     */
    public function getRecent() {
        $timestamp = getControl($this->_feed_id . '.last', 0);
        if (!is_numeric($timestamp)) {
            // TODO: raise error, or use cache?
            return array();
        }
        $timestamp = intval($timestamp);
        $currentTime = time();
        if ($currentTime - $timestamp > 3600) {
            // XXX: should we really be using 'static' file names?
            setControl($this->_feed_id. '.last', (string) $currentTime);
            return parent::getRecent();
        }
        // TODO: should we return cached ones, or what?
        return array();
    }

    //! additional utility function to 'add' a notification
    /*!
     * @param string $title Title for the post.
     * @param string $author Author of the post.
     * @param string $type Post type. e.g., 'warning', 'information',
     *   'security'.
     * @param string $severity Severity e.g., 'high', 'medium', 'low'
     * @param string $shortDescription The short description. Should be one
     *   line or so.
     * @param string $description The full body of the post.
     *
     * @return void
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     *
     * @pre You must call this from a process that has same uid as the interface
     *   runs as because if we create the feed through this, then there it may
     *   be owned by the wrong user, and the interface and other processes may
     *   have issues working with the feed files.
     */
    public function addLocalEntry($title, $author, $type, $severity,
        $shortDescription, $description) {
        // validate
        if (  !is_string($title) || strlen($title) < 1
           || !is_string($author) || strlen($author) < 1
           || !is_string($type) || strlen($type) < 1
           || !is_string($severity) || strlen($severity) < 1
           || !is_string($shortDescription) || strlen($shortDescription) < 1
           || !is_string($description) || strlen($description) < 1) {
            return;
        }
        // TODO: should we enforce specific types & severities?
        // TODO: should we enforce a maximum length on short description?

        // create the entry object
        $entry = new LMNotificationsEntry();
        $entry->timestamp = time();
        $entry->title = $title;
        $entry->author = $author;
        $entry->type = $type;
        $entry->severity = $severity;
        $entry->short_description = $shortDescription;
        $entry->description = $description;

        // we can now add it.
        $localRepo = $this->getLocal();
        // first we want to load up *all* (hence timestamp 0) entries we have
        // stored, because when we save we will rewrite the list of entries,
        // so we want to be sure we have everything saved at once. getEntries()
        // loads them into the object for us (we don't need to store the return
        // value here because we don't need it).
        $localRepo->getEntries(0);
        // addEntries() expects an array, hence this odd calling parameter.
        $localRepo->addEntries(array($entry));
        // and save.
        if (!$localRepo->save()) {
            // TODO: do something?
        }
    }
}
