<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Product line and is
 * Copyright (c) LinuxMagic Inc. 2008-2017 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *            Robert Bisewski <robertb@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *            Kyle Macdonald <kylem@linuxmagic.com>
 *
 * $Id: lib.notification_module.php 7230 2017-11-17 23:54:28Z shaun $
 */
require_once('lib.module.php');
require_once('lib.msnotifications.php');

/*!
 * this class provides common functionality for displaying notification
 * feeds.
 * this is because we are planning to have at least two feeds and it seems
 * to me that they are going to need to have generally similar functionality.
 */
class NotificationModule extends Module {
    //! integer. the feed id we're working on. in the notification libraries
    // themselves this is referred to as the product.
    private $_feed_id;

    //! the internal handle to the 'notification' class
    private $_notification_handle;

    //! the internal count of unread messages: used for access to update menu
    // label
    private $_num_unread;

    //! the internal count of read messages: used for access to update menu
    // label
    private $_num_read;

    //! string. the submodule name. we need this to correctly direct
    // requests.
    protected $_submodule;

    //! string. the i18n key for 'success' after marking something read.
    // plural.
    protected $_msg_alert_marked_read;

    //! string. the i18n key for 'success' after marking something read.
    // singular.
    protected $_msg_alert_marked_read_single;

    //! string. the i18n key for the blurb of an alert section.
    protected $_msg_alert_blurb;

    //! string. the i18n key to show when no alerts to display.
    protected $_msg_alert_no_alerts;

    //! string. the i18n key to show when no new alerts to display.
    protected $_msg_alert_no_new_alerts;

    //! class constructor
    /*!
     * @param string $l the 'label' that is used for links, and locale.
     * @param int $feedID The feed ID to work with.
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     *
     * @constructor
     */
    public function __construct($l, $feedID) {
        // TODO: what should we do if this is invalid? we can't return a failure
        //   value (other than throwing an exception!)
        if (!is_int($feedID)) {
            return;
        }
        $showMenuLink = FALSE;
        parent::__construct($l, $showMenuLink);
        $this->_notification_handle = new MSNotifications($feedID);
        $this->_num_unread = $this->_notification_handle->countUnread();
        $this->_num_read = $this->_notification_handle->countRead();
    }

    //! handle a request action 'mark_read'.
    /*!
     * @param array $errors The array of errors we may set. This is by
     *   reference and should contain full messages.
     * @param array $notices The array of notices we may set. This is by
     *   reference and should contain full messages.
     *
     * @return void
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * we should have a set of feed items listed in the request.
     * we will mark each one of these as read internally.
     */
    public function _actionMarkRead(&$errors, &$notices) {
        if (!is_array($errors) || !is_array($notices)) {
            $errors[] = intl('sa_invalid_messages');
            return;
        }
        // we should have timestamp(s) identifying those we want to
        // mark read.
        if (!array_key_exists('feed_items', $_POST)
            || !is_array($_POST['feed_items'])
            || count($_POST['feed_items']) === 0) {
            $errors[] = intl('sa_no_alert_to_mark_read');
            return;
        }
        $timestamps = array();
        foreach ($_POST['feed_items'] as $timestamp) {
            if (!is_numeric($timestamp)) {
                $errors[] = intl('sa_invalid_alert_id');
                return;
            }
            $timestamps[] = intval($timestamp);
        }
        // now mark each one as read.
        // TODO: this could be done more efficiently - right now we're
        //   loading/saving alerts for each one.
        $notifications = $this->_notification_handle;
        if (!$notifications) {
            $errors[] = intl('sa_cant_access_alerts');
            return;
        }
        $count = 0;
        foreach ($timestamps as $timestamp) {
            if (!$notifications->setItemRead($timestamp)) {
                $errors[] = intl('sa_failed_to_mark_alert_read');
                // TODO: this may be too detailed for interface display.
                if ($notifications->_error !== NULL) {
                    $errors[] = htmlspecialchars($notifications->_error);
                }
                return;
            }
            $count++;
        }
        if ($count === 1) {
            $notices[] = intl($this->_msg_alert_marked_read_single);
        } else {
            $notices[] = intl($this->_msg_alert_marked_read);
        }
        // ensure we update our cached unread count or else we may have it
        // cached as an old value.
        $this->_num_unread = $this->_notification_handle->countUnread();
        $this->_num_read = $this->_notification_handle->countRead();
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        
	// sets the state of the collaspible modules
	if ($action === 'remember_expand_state') {
                $module = isset($_POST['changed_module']) ? $_POST['changed_module'] : NULL;
                $value  = isset($_POST['expanded']) ? $_POST['expanded'] : NULL;
                $var = $module . "__collapse";
                $_SESSION[$var] = $value;
        }

	if (!is_string($action) || strlen($action) === 0
            || !is_array($errors) || !is_array($notices)) {
            // NOTE: while not every request must have an action, if
            //   we're asked to process actions and these are not
            //   as expected, it's invalid.
            $errors[] = intl('sa_invalid_action');
            return;
        }

        if ($action === 'mark_upgrade_read') {
            $this->_actionMarkRead($errors, $notices);
            $notices = array(intl('pu_upgrade_complete'));
            return;
        }

        if ($action === 'mark_read') {
            return $this->_actionMarkRead($errors, $notices);
        }


        $errors[] = intl('sa_invalid_action');
    }

    //! method to render the full page module.
    /*!
     * @return void
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     *
     * SIDE EFFECT: Generates HTML to stdout
     */
    public function renderModule() {

        $this->_i18n_js[] = 'sa_read_less';
        $this->_i18n_js[] = 'sa_read_more';
        $this->_i18n_js[] = 'sa_show_read';
        $this->_i18n_js[] = 'sa_show_unread';

        //Variable used to determine if we should show / hide read messages
        $show_read = 0;
        $button_label = intl('sa_show_read');
        $foldoutTitle = "";

        // Determines whether showing un/read messages for news / system alerts
        if ($this->_submodule == 'systemAlert') {

            $show_read = (array_key_exists('show_read_systemAlert', $_POST)
                          && $_POST['show_read_systemAlert'] === '1') ? 0 : 1;
            $button_label = ($show_read) ? intl('sa_show_read') : intl('sa_show_unread');
            $foldoutTitle = intl('systemAlertHeader');

        } else if ($this->_submodule == 'news') {

            $show_read = (array_key_exists('show_read_news', $_POST)
                          && $_POST['show_read_news'] === '1') ? 0 : 1;
            $button_label = ($show_read) ? intl('sa_show_read') : intl('sa_show_unread');
            $foldoutTitle = intl('newsHeader');
        }

        // Determines the POST parameters
        $params = "{hiddenFields:{'module':'magicspamAlert',"
                . "'submodule':'$this->_submodule',"
                . "'show_read_$this->_submodule':'$show_read'}}";

        // the text we have for 'no alerts to show' varies depending on if there
        // are no alerts *at all* (no read, no new): 'no alerts to show', and
        // if there are no new alerts to show, but there *are* read alerts that
        // could be shown: 'no new alerts to show'.
        if ($this->countUnread() + $this->countRead() === 0) {
            $noAlertsText = intl($this->_msg_alert_no_alerts);
        } else {
            $noAlertsText = intl($this->_msg_alert_no_new_alerts);
        }

        // check if a relative path has been defined and then override
        // any currently existing path
        $image_path_override = "";
        if (defined('RELPREFIX')) {
            $image_path_override = RELPREFIX;
        }

        // Renders the alert template
        renderTemplate('alertTemplate', array(
            'submodule'            => $this->_submodule,
            'show_read'            => $show_read,
            'button_label'         => $button_label,
            'params'               => $params,
            'submodule_title'      => $foldoutTitle,
            'unreadCount'          => $this->countUnread(),
            '_msg_alert_blurb'     => intl($this->_msg_alert_blurb),
            '_msg_alert_no_alerts' => $noAlertsText,
            'override'             => $image_path_override,
            'image_path'           => IMAGEPATH,
        ));

        // Prints out all of the alert messages
        $this->_renderAlerts();

        // and now we close off the mess we made...
        echo "</div><!-- end collapsibleModule -->\n";
        echo "<div class=\"descBuffer\"></div>\n";
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getCSS() {
        return array();
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getJS() {
        return array('ajax.js');
    }

    //! method to get the number of unread messages
    /*!
     * @return int The number of unread messages.
     *
     * @author Anonymous
     * @author William Storey <william@linuxmagic.com>
     *
     * if you're wondering where this is used, we currently use it for knowing
     * how many unread alerts there are for the purposes of displaying in
     * the tabs.
     */
    public function countUnread() {
        return $this->_num_unread;
    }

    //! method to get the number of read messages
    /*!
     * @return int The number of read messages.
     *
     * @author William Storey <william@linuxmagic.com>
     */
    public function countRead() {
        return $this->_num_read;
    }

    //! Displays the alert messages
    /*!
     * @return void
     *
     * @author Robert Bisewski <robertb@linuxmagic.com>
     */
    private function _renderAlerts() {
        $entries  = $this->_notification_handle->getUnreadEntries();
        $entries2 = $this->_notification_handle->getReadEntries();

        // Validation
        if (sizeof($entries) > 0 && sizeof($entries2) == 0) {
            $this->_renderEntries($entries, TRUE);
            return;
        } else if (sizeof($entries) == 0 && sizeof($entries2) > 0) {
            $this->_renderEntries($entries2, FALSE);
            return;
        } else if (sizeof($entries) > 0 && sizeof($entries2) > 0) {
            $entityNum = $this->_renderEntries($entries, TRUE);
            $this->_renderEntries($entries2, FALSE, $entityNum);
            return;
        }
    }

    //! generate HTML listing a set of entries.
    /*!
     * @param array $entries An array of LMNotificationsEntry objects.
     * @param bool $unread True if listing unread, false otherwise. At this
     *   time we have different information to show depending on whether
     *   we are listing unread entries or not.
     *
     * @return void
     *
     * @author Robert Bisewski <robertb@linuxmagic.com>
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     */
    private function _renderEntries($entries, $unread, $entityNum = 0) {
        global $module;

        // Basic Validation
        if (!is_array($entries) || sizeof($entries) == 0
            || !is_bool($unread)) {
            return;
        }

        // Renders each of the entries
        foreach ($entries as $entry) {

            // Grabs the date and description
            $date = date("l, F j, Y", $entry->timestamp);
            $description = nl2br(htmlspecialchars($entry->description));
            $fullMessage = $description;

            // Determines if description is too massive to show all at once
            $showMore = FALSE;
            // NOTE: while we don't care about matches here, preg_match_all() in
            //   older PHP versions required that the parameter be given.
            $matches = array();
            if ((preg_match_all('/\n/', $entry->description, $matches) !== FALSE
               && preg_match_all('/\n/', $entry->description, $matches) > 10)
               || strlen($entry->description) > 1500) {
                $showMore = TRUE;
            }

            // Determines the shortened value of the message
            if (preg_match_all('/\n/', $entry->description, $matches) > 10) {
                $aLines = explode("\n", $entry->description);
                $shortDesc = "";
                for ($i = 0; $i < 10; $i++) {
                    $shortDesc .= $aLines[$i] . "\n";
                }
                $description = nl2br(htmlspecialchars($shortDesc));
            } else if (strlen($description) > 1500) {
                $description = nl2br(htmlspecialchars(substr($entry->description, 0, 1500) . "..."));
            }

            // Determines the color of the severity
            if ($entry->severity == "high") {
                $severityColor = "highSeverity";
            } else if ($entry->severity == "medium") {
                $severityColor = "mediumSeverity";
            } else if ($entry->severity == "low") {
                $severityColor = "lowSeverity";
            }

            // Params for the form submit
            $params = "{hiddenFields:{'module':'$module',"
                    . "'submodule':'$this->_submodule',"
                    . "'ms_action':'mark_read',"
                    . "'feed_items[]':'$entry->timestamp'}}";
            
            $type = htmlspecialchars($entry->type);
            $upgradeLink = '';

            // determine if we display the 'upgrade now' link
            $showUpgradeLink = FALSE;
            if (!defined('UPDATE_TOGGLE_FLAG')) {
                $showUpgradeLink = TRUE;
            } else if (defined('UPDATE_TOGGLE_FLAG') && file_exists(UPDATE_TOGGLE_FLAG)) {
                $toggleContents = file_get_contents(UPDATE_TOGGLE_FLAG);
                if (trim($toggleContents) == '1') {
                    $showUpgradeLink = TRUE;
                }
            } else if (defined('UPDATE_TOGGLE_FLAG')) {
                if (!isFreemium()) {
                    $showUpgradeLink = TRUE;
                }
            }
            // is PKG_UPDATER defined in this suite.. and if so, is the entry type 'upgrade'
            if ($showUpgradeLink && defined('PKG_UPDATER') && $entry->type == 'upgrade') {
                $upgradeLink = "&nbsp;<a href=\"#\" onclick=\"javascript:showUpgrade(" . $entry->timestamp . ");return false;\">" . intl('upgrade_now') . "</a>";
            }

            // Renders the alert template
            renderTemplate('alertBubble', array(
                                                'module'         => $module,
                                                'submodule'      => $this->_submodule,
                                                'title'          => htmlspecialchars($entry->title),
                                                'markAsRead'     => intl('sa_mark_read'),
                                                'upgradeLink'    => $upgradeLink,
                                                'date_label'     => intl('date'),
                                                'by_label'       => intl('by'),
                                                'type_label'     => intl('type'),
                                                'severity_label' => intl('severity'),
                                                'readMore'       => intl('sa_read_more'),
                                                'severity_color' => $severityColor,
                                                'unread'         => $unread,
                                                'params'         => $params,
                                                'showMore'       => $showMore,
                                                'fullMessage'    => $fullMessage,
                                                'submodule'      => $this->_submodule,
                                                'entityNum'      => $entityNum,
                                                'timestamp'      => htmlspecialchars($entry->timestamp),
                                                'author'         => htmlspecialchars($entry->author),
                                                'date'           => htmlspecialchars($date),
                                                'type'           => $type,
                                                'severity'       => htmlspecialchars($entry->severity),
                                                'short_desc'     => htmlspecialchars($entry->short_description),
                                                'desc'           => $description
                                               ));

            // Increments the entityNum, which is used to track which message is expanding / collapsing
            $entityNum++;
        }

        // Returns the entity number
        return $entityNum;
    }
}
