<?php
/* vim:et:sw=4:ts=4
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2008-2013 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc. 
 *
 * The Original Code is distributed on an 'AS IS' basis, 
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC 
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET 
 * ENJOYMENT OR NON-INFRINGEMENT. 
 *
 * Do NOT download, distribute, use or alter this software or file in any 
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *
 * $Id: lib.ratelimitcontrol.php 3825 2014-09-29 21:32:38Z shaun $
 */


require_once('lib.module.php');
require_once('lib.datatable.php');
require_once('functions.php');

class rateLimitControl extends Module {
    // internal flag to indicate the 'prefix' requirements for the rate limiter type.
    private $prefix;

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   string  the 'prefix' that is used to differentiate the rate limiter type.
     *                  NOTE: should be of type 'rl' or 'ra'
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    public function __construct($l, $p) {
        // we need access to the word list because we are loading a locale explicitly here.
        global $ms_words;

        parent::__construct($l, FALSE);

        // this should never happen!
        if (   !is_string($p)
            || ($p !== 'ra' && $p !== 'rl') ) {
            // XXX: we only throw exceptions in constructors - after that we require error handling!
            throw new Exception("ratelimitControl called with illegal prefix {$p}");
        }
        // set the prefix
        $this->prefix = $p;

        // in this case we have to explicitly load the locale because the module is not being
        // 'loaded' per se...
        $moduleLocale = loadMSLocale('rateLimitControl');
        if (file_exists($moduleLocale)) {
            require_once($moduleLocale);
        }
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $module;

        // no action = nothing to do so let's not waste time then.
        if (!is_string($action) || strlen($action) == 0) {
            return;
        }

        // based on the 'type' of rate limiter we are controlling we set the control file vars.
        $count_control = $flag_control = $block_control = "";

        if ($this->prefix == 'rl') {
            $count_control = RATELIMITER_COUNT;
            $flag_control = RATELIMITER_FTO;
            $block_control = RATELIMITER_BTO;
        }
        if ($this->prefix == 'ra') {
            $count_control = AUTHLIMITER_COUNT;
            $flag_control = AUTHLIMITER_FTO;
            $block_control = AUTHLIMITER_BTO;
        }
        // we only serve up one action: save!
        if ($action == 'save') {
            // grab the posted data
            $count_limit = isset($_POST['count_limit']) ? $_POST['count_limit'] : 0;
            $flag_timeout = isset($_POST['flag_timeout']) ? $_POST['flag_timeout'] : 0;
            $block_timeout = isset($_POST['block_timeout']) ? $_POST['block_timeout'] : 0;

            $error_detected = FALSE;

            // convert the posted time values to seconds
            $flag_timeout = $this->minutesToSeconds($flag_timeout);
            $block_timeout = $this->minutesToSeconds($block_timeout);

            // now we start our chained 'save' event
            $rv = setControl($count_control, "$count_limit");
            if ($rv !== TRUE) {
                $errors[] = $rv;
                $error_detected = TRUE;
            } else {
                $rv = setControl($flag_control,"$flag_timeout");
                if ($rv !== TRUE) {
                    $errors[] = $rv;
                    $error_detected = TRUE;
                } else {
                    $rv = setControl($block_control, "$block_timeout");
                    if ($rv !== TRUE) {
                        $errors[] = $rv;
                        $error_detected = TRUE;
                    }
                }
            }
            // if there were no errors, then post a friendly save message!
            if (!$error_detected) {
                $notices[] = intl('saved');
            }
        }
    }

    //! method to render the full page module.
    /*!
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    public function renderModule() {
        global $module;

        // based on the 'type' of rate limiter we are controlling we set the control file vars.
        $count_control = $flag_control = $block_control = $title_control = $desc_control = $submodule = $page = "";

        if ($this->prefix == 'rl') {
            $count_control = RATELIMITER_COUNT;
            $flag_control = RATELIMITER_FTO;
            $block_control = RATELIMITER_BTO;
            $title_control = 'rl_ip_control_title';
            $desc_control = 'rl_ip_control_desc';
            $submodule = 'ipRateLimitControl';
            $page = 'ip_rl';
        }
        if ($this->prefix == 'ra') {
            $count_control = AUTHLIMITER_COUNT;
            $flag_control = AUTHLIMITER_FTO;
            $block_control = AUTHLIMITER_BTO;
            $title_control = 'rl_auth_control_title';
            $desc_control = 'rl_auth_control_desc';
            $submodule = 'userRateLimitControl';
            $page = 'user_rl';
        }

        // now we grab the control file values
        $count_limit = trim(getControl($count_control, 0));
        $flag_timeout = $this->secondsToMinutes(trim(getControl($flag_control, 0)));
        $block_timeout = $this->secondsToMinutes(trim(getControl($block_control, 0)));

        // and render out the template.
        renderTemplate('ratelimiterControlModule', array (
                                                 'module'             => $module,
                                                 'submodule'          => $submodule,
                                                 'page'               => $page,
                                                 'title'              => intl($title_control),
                                                 'desc'               => intl($desc_control),
                                                 'rl_message_limit'   => intl('rl_message_limit'),
                                                 'rl_count_limit'     => intl('rl_count_limit'),
                                                 'rl_counting_period' => intl('rl_counting_period'),
                                                 'rl_flag_timeout'    => intl('rl_flag_timeout'),
                                                 'rl_expiry_time'     => intl('rl_expiry_time'),
                                                 'rl_block_timeout'   => intl('rl_block_timeout'),
                                                 'count_limit'        => $count_limit,
                                                 'flag_timeout'       => $flag_timeout,
                                                 'block_timeout'      => $block_timeout,
                                                 'save_key'           => intl('save'),
                                               ));
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    public function getCSS() {
        return array('ratelimiter.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    public function getJS() {
        return array('limit-integer.js');
    }

    //! internal utility function to convert seconds to minutes
    /*!
     * @param $arg int amount in seconds to be converted
     * @return float amount in minutes
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    private function secondsToMinutes($arg) {
        if (!is_numeric($arg)) {
            return 0;
        }
        return round($arg / 60);
    }

    //! internal utility function to convert minutes to seconds
    /*!
     * @param number of minutes to be converted to seconds
     * @return amount in minutes
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    private function minutesToSeconds($arg) {
        if (!is_numeric($arg)) {
            return 0;
        }
        return $arg * 60;
    }
}

?>
