<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2017 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Logan Borean <loganb@linuxmagic.com>
 *
 * $Id: lib.rcptdaycounter.php 4802 2017-07-07 23:19:54Z logan $
 *
 * RCPTCounter is a simple method for obtaining stored data
 * counts from the magicspam engine.
 */

require_once('lib.rcptcounter.php');

class RCPTHourlyCounter extends RCPTCounter
{


    //! constructor
    /*!
     * @param   integer the day to load.
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     *
     */
    public function __construct() {
        // important that this comes before the rest
        parent::__construct(null);
        
        // reset these even though they are set in the parent
        $this->_m_strError = "";
        $this->_m_strFilePath = $this->findStatFilePath();
        $this->_m_arrData = array();

        // We expect the HOURLY file size
        // add up the expected size in bytes.
        // format: <version (uint8_t)><sent date (uint32_t)><31 record days>
        $datsize = 1;
        // account for the 'sent' timestamp field. this is a header at the
        // beginning of the file (32 bits). NOTE: it's not used for anything.
        $datsize += 4;
        // 24 hour records, 8 integers (4 bytes each) per record.
        $datsize += 24 * 8 * 4;

        $this->_fileSize = $datsize;
        $this->_validVersions = array(1);
    }


    //! accessor to get the last buffered error message.
    /*!
     * @returns string  the last error message (if any)
     */
    public function error() {
        return $this->_m_strError;
    }

    //! generate path to the new format stat file for the given month.
    /*!
     * @param int $month The month to look for stats for.
     *
     * @returns mixed string The stat file to work with (full path),
     *   or NULL if some kind of error occurred.
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * SIDE EFFECT: We'll set the instance variable _m_strError if an
     *   error occurred.
     *
     * we have two stat file formats. this method builds the path to the
     * new format one.
     */
    private function _getNewStatFilePath() {
        global $home;

        if (!defined('COUNTER_PATH')) {
            $this->_m_strError = 'Statistics counter path is not available.';
            return NULL;
        }
        $path = sprintf('%s.hourly', COUNTER_PATH);
        return $path;
    }


    //! determine which stat file to work on.
    /*!
     * @param int $month The month to look for stats for.
     *
     * @returns mixed string The stat file to work with (full path),
     *   or NULL if some kind of error occurred.
     *
     * @author Logan Borean <loganb@linuxmagic.com>
     *
     * SIDE EFFECT: We'll set the instance variable _m_strError if an
     *   error occurred.
     *
     */
    private function findStatFilePath() {

        if (!defined('COUNTER_PATH')) {
            $this->_m_strError = 'Statistics counter path is not available.';
            return NULL;
        }

        // we have two possible stat files: old format, and new format.
        $newPath = $this->_getNewStatFilePath();

        if ($newPath === NULL) {
            // we expect we already set an error.
            return NULL;
        }

        // if we have a new stat file for it, then we use that.
        if (file_exists($newPath)) {
            return $newPath;
        }
        return NULL;
    }


    //! method to get a specific count value, or 'all'.
    /*! we need to override this because RCPTCounter checks dates
     *
     * @param   integer the day of the month to grab
     * @param   integer the count type (see defines at top of file).
     *
     * @returns mixed   the count value found or 0 if nothing is loaded. IF the
     *                  count type is 'ALL', then returns the full array of counts
     *
     * @author Logan Borean <loganb@linuxmagic.com>
     */
    public function get_value($hour, $type) {
        // sanity check.
        if (!is_numeric($hour) || $hour < 0 || $hour > 23) {
            return 0;
        }
        if ( !is_numeric($type) ||
            // TODO: how about a function to check if it's a valid type?
            ( $type != RCPT_TYPE_TOTAL &&
              $type != RCPT_TYPE_HAM &&
              $type != RCPT_TYPE_SPAM &&
              $type != RCPT_TYPE_EXEMPT &&
              $type != RCPT_TYPE_RATELIMIT_IP &&
              $type != RCPT_TYPE_RATELIMIT_USER &&
              $type != RCPT_TYPE_ALL ) ) {
            return 0;
        }
        // make sure our internal data is set up
        // TODO: just return if it's not... reduce nesting!
        if (is_array($this->_m_arrData)) {
            // and that there is data for the requested day index.
            if (isset($this->_m_arrData[$hour])) {

                    $ts = $this->_m_arrData[$hour]['timestamp'];

                    $howLongAgo = time() - $ts;
                    // if its more than 24 hours ago
                    if ($ts === 0 || ($howLongAgo > (24*60*60))) {
                        return 0;
                    }
                    
                    //echo  '(' .$tsDate . '   ' . date('G n o') . ')';

                    switch($type) {
                    case RCPT_TYPE_TOTAL:
                        $total = $this->_m_arrData[$hour]['ham'];
                        $total += $this->_m_arrData[$hour]['spam'];
                        $total += $this->_m_arrData[$hour]['exempt'];
                        // TODO: should we add the counts for IP/user ratelimit
                        //   in here?
                        return $total;
                        break;
                    case RCPT_TYPE_HAM:
                        return $this->_m_arrData[$hour]['ham'];
                        break;
                    case RCPT_TYPE_SPAM:
                        return $this->_m_arrData[$hour]['spam'];
                        break;
                    case RCPT_TYPE_EXEMPT:
                        return $this->_m_arrData[$hour]['exempt'];
                        break;
                    case RCPT_TYPE_RATELIMIT_IP:
                        return $this->_m_arrData[$hour]['ip_ratelimit'];
                        break;
                    case RCPT_TYPE_RATELIMIT_USER:
                        return $this->_m_arrData[$hour]['user_ratelimit'];
                        break;
                    case RCPT_TYPE_INBOUND:
                        return $this->_m_arrData[$day]['inbound'];
                        break;
                    case RCPT_TYPE_OUTBOUND:
                        return $this->_m_arrData[$day]['outbound'];
                        break;
                    case RCPT_TYPE_ALL:
                        return $this->_m_arrData[$hour];
                        break;
                    }
            }
        }
        return 0;
    }

}
