<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2006-2018 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Dylan Mills <dylan@wizard.ca>
 *            Michael Peddemors <michael@wizard.ca>
 *            Shaun A. Johnson <shaun@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *            Rares Harnagea <rares@linuxmagic.com>
 *
 * $Id: lib.wizard.php 159 2017-03-28 23:15:31Z rares $
 */


//! definitions for display formats: useful for setting flags.
if (!defined('DISPLAY_TEXT')) {
    define('DISPLAY_TEXT', 0);
}
if (!defined('DISPLAY_HTML')) {
    define('DISPLAY_HTML', 1);
}

require_once(__DIR__ . '/lib.wizard_debug.php');
require_once(__DIR__ . '/lib.wizard_validator.php');
require_once(__DIR__ . '/interface.wiz_db_abstract.php');

//!
/*! WizardObject::
 *
 * This is the base class which all table classes will extend.
 *
 * @category   Database
 * @author     Dylan Mills <dylan@wizard.ca>
 * @copyright  2006 LinuxMagic Inc. All Rights Reserved.
 * @version    Release: $Id: lib.wizard.php 159 2017-03-28 23:15:31Z rares $
 * @link       http://www.linuxmagic.com/
 */
class WizardObject extends WizardDebug implements WizDbBase {

    // @var object
    public $db;

    //! If this is true then all debug messages will be printed.
    // @var bool
    public $DEBUG = FALSE;

    //! mixed. this is a shorthand way to indicate an error.
    // this may be a string, or bool true to indicate there is an error
    // but supplying no error code, or it will be NULL if there is no error
    // set.
    // this should be set in conjunction with $_error_text.
    protected $_error = NULL;

    //! mixed. this is a full text method to indicate the error problem.
    // this may be a string, or NULL to indicate no error set.
    // this should be set in conjunction with $_error.
    // it is intended to be used as a fallback if no i18n translation is
    // to be done on $_error.
    protected $_error_text = NULL;
       
    //! @string the db_type PEAR or PDO
    protected static $db_type = '';
    
    //!
    /*! WizardObject()
     *
     * This constructor should be called with the PEAR::DB or PDO object.
     *
     * @param object $dbinstance  A value containing a valid PEAR::DB object.
     *
     * @return void
     */
    public function __construct($dbinstance='') {
        parent::__construct();
        if (is_object($dbinstance)) {
            $this->loadInstance($dbinstance);
        }
    }

    //! set a DB instance
    /*!
     * @param object $dbinstance  A value containing a valid PEAR::DB object.
     * @return bool
     *
     * Note: This allows us to set $db to a db connection.
     * When we move this class object into and out of session variables, we serialize it.
     * However, we do lose the db connection so we must reset it after we unserialize.
     */
    public function setDB($dbinstance) {
        // when we serialize/unserialize this class so we can store it in the Session vars,
        // we need to reassing the db connect variable since it gets lost.
        return $this->loadInstance($dbinstance);
    }

    //! Method used to load a DB instance on $this->db, this->db is a wrapper 
    /*!
     * @param object $instance
     * 
     * @return bool success
     *
     * @author Rares Harnagea 
     */
    protected function loadInstance($instance) {
        if (!isset($instance) || !is_object($instance)) {
            return NULL;
        }
        if ($instance instanceof PDO || $instance instanceof WizDbPdo) {

            static::$db_type = 'PDO';
            
            if ($instance instanceof PDO) {
                $this->db = new WizDbPdo($instance);
            } else {
                $this->db = new WizDbPdo($instance->getDb());
            } 

            return TRUE;
            // PEAR instances 
        } elseif ($instance instanceof DB_pgsql || $instance instanceof WizDbPear) {

            static::$db_type = 'PEAR';

            if ($instance instanceof DB_pgsql) {
                $this->db = new WizDbPear($instance);
            } else {
                $this->db = new WizDbPear($instance->getDb());
            }
            return TRUE;
        } else {
            $this->debug('Invalid variable passed to instance [' . get_class($instance) . ']');
        }
    }

    //! This is a convenience function to begin a transaction in the database
    /*!
     * @return bool success
     *
     * @author Anonymous
     */
    public function beginTransaction() {
        return $this->db->begin();
    }

    //! This is a convenience function to commit a transaction in the database
    /*!
     * @return bool success
     *
     * @author Anonymous
     */
    public function commitTransaction() {
        return $this->db->commit();
    }

    //! This is a convenience function to rollback a transaction in the database
    /*!
     * @return bool success
     *
     * @author Anonymous
     */
    public function rollbackTransaction() {
        return $this->db->rollback();
    }

    //! This function return server now timestamp or false if a error occours
    /*!
     * @return mixed string with timestamp or bool false if we can't get the now() time
     *
     * @author Anonymous
     */
    public function getDatabaseTime() {
        $sql = "SELECT now()";
        return $this->db->getOne($sql);
    }

    //! This is a helper function to run a query and check for errors
    /*!
     * @param $sql the $sql that you want to run
     *
     * @param $table_name, this argument does not make any sense it's mainly used for debug
     * 
     * @return mixed the query or 0 in case the query is not valid
     *
     * @author Anonymous
     * XXX Marked for future deprecation use ::query($sql, $params = array(), $bindType = FALSE);
     */
    public function &checkQuery($sql, $table_name = '') {
        $query = $this->db->query($sql);

        // when there is an error
        if (static::isDBError($query)) {
            $this->debug("$table_name failed [ <pre> $sql </pre> ]: ". $query->getMessage());
            $zero = 0;
            return $zero;
        } else {
            return $query;
            
        }
    }

    //! This function check if there was a PDO error
    /*!
     * @param reference &$object ref to object $object the object that we want to check
     * 
     * @return bool TRUE if the variable is not set or error code is no equal with PDO::ERR_NONE 
     *              FALSE if the object erroCode === PDO::ERR_NONE
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     *      
     * Sideeffect: This function might change the value of your argument to a new PDOStatement object 
     * XXX: this is implemented only for PDO
     */
    public static function isDBError(&$object) {
        // this function works for object only 
        if (!is_object($object)) {
            return FALSE;
        }
        if (static::$db_type === 'PDO') {
            return WizDbPdo::isError($object);
        } else {
            return WizDbPear::isError($object);
        }
    }

    //! Check if the value passed in is a positive number.
    /*!
     * @param string $num  The value to test.
     *
     * @return bool True or false.
     */
    public function positiveNumber($num){
        return (strlen($num) && is_numeric($num) && $num >= 0);
    }

    //! If the value passed in is numeric then return it, otherwise return 'NULL'
    /*!
     * @param string $num  The value to evaluate.
     *
     * @return string The number or the string "NULL".
     */
    public function numberOrNull($num) {
        if (strlen($num) && is_numeric($num)) {
            return $num;
        } else {
            return 'NULL';
        }
    }

    //! If the value passed in is a string then quote it for db use and return
    /*!it, otherwise return  NULL.
     *
     * @param string $str  The value to evaluate.
     *
     * @return string A quoted string or the string "NULL".
     */
    public function quoteOrNull($str) {
        if(strlen($str)) {
            if (get_magic_quotes_gpc()) {
                return $this->db->quote(stripslashes($str));
            } else {
                return $this->db->quote($str);
            }
        } else {
            return "NULL";
        }
    }

    //! retrieve the next value in a named sequence.
    /*!
     * @param string $sequence
     *
     * @return mixed int the next value, or bool FALSE if failure.
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     *
     * NOTE: we use this instead of PEAR::DB::nextId() due to the limitations
     *       on that method. see the documentation about that. specifically
     *       I am referring to the fact that this method changes the name of
     *       the sequence and documents to not use sequences created manually.
     *
     */
    public function getNextSequenceValue($sequence) {
        if (!isset($sequence) || !WizardValidator::isStringNotEmpty($sequence) === 0) {
            $this->debug("invalid parameter");
            return FALSE;
        }

        $sql = 'SELECT NEXTVAL(?) AS value';
        $params = array($sequence);
        $nextVal = $this->db->getOne($sql, $params);
        if (!isset($nextVal) ||  !WizardValidator::isIntPositiveOrZero($nextVal)) {
            return FALSE;            
        }
        return intval($nextVal);
    }

    //! set an error message/code or flag.
    /*!
     * @param mixed $msg This may be a string to set the error code, or bool TRUE to 
     *                   indicate there is an error (but to set no error code).
     *
     * @return bool whether successful
     *
     * @author William Storey <william@linuxmagic.com>
     */
    public function setError($msg) {
        if (is_string($msg) && strlen($msg) > 0) {
            $this->_error = $msg;
            return TRUE;
        }
        if ($msg === TRUE) {
            $this->_error = TRUE;
            return TRUE;
        }
        $this->debug(".setError: invalid parameter");
        return FALSE;
    }

    //! set a new error message: full text.
    /*!
     * @param string $msg The full error message to set.
     *
     * @return bool whether successful
     *
     * @author William Storey <william@linuxmagic.com>
     */
    public function setErrorText($msg) {
        if (is_string($msg) && strlen($msg) > 0) {
            $this->_error_text = $msg;
            return TRUE;
        }
        $this->debug(".setErrorText: invalid parameter");
        return FALSE;
    }

    //! get the last error (shorthand).
    /*!
     * @return mixed string if the last error message is recorded, TRUE if
     *               there is an error set but no code is recorded, or FALSE
     *               if there are no errors.
     *
     * @author William Storey <william@linuxmagic.com>
     */
    public function getError() {
        // no error set.
        if ($this->_error === NULL) {
            return FALSE;
        }
        return $this->_error;
    }

    //! get the last error (full text).
    /*!
     * @return string The last error message recorded, or bool FALSE if there
     *                are no error messages.
     *
     * @author William Storey <william@linuxmagic.com>
     */
    public function getErrorText() {
        if ($this->_error_text === NULL) {
            return FALSE;
        }
        return $this->_error_text;
    }

    //! unset error state.
    /*!
     * @author William Storey <william@linuxmagic.com>
     *
     * we clear both error instance variables so that errors are unset.
     */
    public function unsetErrors() {
        $this->_error = NULL;
        $this->_error_text = NULL;
    }   
}

//!
/*! WizardObject::
 *
 * The config class, loads the config values and has utility functions
 * to create a db connection
 *
 * @category   Database
 * @author     Dylan Mills <dylan@wizard.ca>
 * @copyright  2006 LinuxMagic Inc. All Rights Reserved.
 * @version    Release: $Id: lib.wizard.php 159 2017-03-28 23:15:31Z rares $
 * @link       http://www.linuxmagic.com/
 */
class WizardConfig extends WizardObject {
    //! Source file, defaults to "config.php"
    // @var string
    protected $source = "config.php";

    //! an assoc array of elements
    // @var array
    public $elements = NULL;
        
    //! path to the file where output is going to be redirected in the log to file is enabled
    const LOG_FILE = '/var/log/magicmail-debug/magicmail_interface.log';

    //! control file to enable log to file, 1 to enable 0 disable
    const ENABLE_LOG_TO_FILE = '/etc/magic-mail/control/magicmail_force_log_to_file';

    //! control file to force a SQL scheme, by default we go with PEAR if there is no 
    // sql_scheme on the config file, suggested values 'pgsql_pdo' or 'pgsql'
    const FORCE_SQL_SCHEME = '/etc/magic-mail/control/magicmail_force_db_scheme';

    //! Constructor, sets the source file and loads it.
    /*!
     * @param string $filename  Optional, the name of the config file to load.
     *
     * @return void
     */
    function __construct($filename="config.php") {
        parent::__construct();

        if ($filename) {
            $this->source = $filename;
        }

        if ($filename) {
            $this->loadfile($filename);
        }
    }

    //! Reset the loaded config values.
    /*!
     */
    public function reset() {
        $this->source = "";
        $this->elements = array();
    }

    //! Get a config element.
    /*!
     * @param string $key The name of the config item to retrieve.
     *
     * @return string The config item's value.
     */
    public function get($key) {
        if (isset($this->elements[$key])) {
            return $this->elements[$key];
        }
        return "";
    }

    //! Set a config element.
    /*!
     * @param string $key The name of the config item to set.
     *
     * @param mixed $val The value that we want to set.
     *
     * @return mixed The value that was set
     */
    public function set($key, $val) {
        return $this->elements[$key] = $val;
    }

    //! Based on the values loaded into the elements array from the config file, make a db connection
    /*!
     * @param string $override_dbname  Optional, name of a different
     *                                 db to load.
     * @param bool $persistent Whether or not to make a peresistent
     *                             connection.  This defaults to false and
     *                             should be used with caution.
     *
     * @return object If the db connection was successful, return a PEAR::DB
     *                 object, otherwise it returns NULL.

     * Note: Supported db's:
     * - MySQL
     * - PostgreSQL
     * - Oracle
     * Hint:  Use $this->elements['sql_XXX'], return a PEAR DB instance
     */
    public function getDBInstance($override_dbname = NULL, $persistent = FALSE) {
        $scheme = $this->get("sql_scheme");

        if (!$scheme) {
            $scheme = "pgsql_pdo";
        }

        // optional contol file to overwrite the scheme
        // this is useful if we want all PHP application to use PEAR or PDO
        $scheme = self::getControl(self::FORCE_SQL_SCHEME, $scheme);

        // using this control file we can redirect the $obj->debug() messsage to a file 
        // this not affect the dumpDebug() and/or showDebugMessage() functions
        $logToFile = self::getControl(self::ENABLE_LOG_TO_FILE, NULL);
        if ($logToFile === '1') {
            // Make sure that Apache has write permission to this file
            WizardDebug::setLogFile(self::LOG_FILE);
        }

        $elements = $this->elements;
        if ($override_dbname) {
            $elements['sql_database'] = $override_dbname;
        }

        $elements['sql_persistent'] = $persistent;
        $elements['sql_scheme'] = $scheme;

        // if is not PDO, the it must be PEAR
        if (strpos($scheme, 'pdo') === FALSE) {
            // load the libraries for abstract PEAR
            require_once(__DIR__ . '/lib.wizard_db_pear.php');
            require_once(__DIR__ . '/lib.wizard_db_pear_sth.php');

            $db = WizDbPear::getInstance($elements);
            $this->loadInstance($db);
            return $this->db;
        } else {
        
            // load the libraries for abstract PEAR
            require_once(__DIR__ . '/lib.wizard_db_pdo.php');
            require_once(__DIR__ . '/lib.wizard_db_pdo_sth.php');

            $db = WizDbPdo::getInstance($elements);
            $this->loadInstance($db);
            return $this->db;
        }
    }

    //! retrieve a control file from the underlying filesystem
    /*!
     * @param string $file the file name
     *
     * @param mixed $fallback the return value if failed to get controls
     *
     * @returns mixed the value in the file or $fallback value
     */
    public static function &getControl($file, $fallback = '') {
        if (!file_exists($file) || !is_readable($file)) {
        return $fallback;
        }
        $data = file_get_contents($file);
        if ($data === FALSE) {
            return $fallback;
        }
        $data = trim($data);
        return $data;
    }


    //! Load a configuration from key: value entries in a local file
    /*!
     * @param string $filename Name of the config file to load.
     *
     * @return bool True if the file was loaded successfully.
     */
    public function loadfile($filename = NULL) {
        if ($filename) {
            $use_filename = $filename;
        } else {
            $use_filename = $this->source;
        }

        if (!@$data = file($this->source)) {
            $this->debug(".loadfile unable to load $use_filename");
            return FALSE;
        }
        // Reset internal state to clean
        $this->reset();

        $this->source = $filename;

        for ($i = 0; $i < count($data); $i++) {
            $val = trim($data[$i]);

            if (preg_match('/^\s*#/', $val)
             || preg_match('/^\s*$/', $val)) {
                continue;
            }

            // This wasn't good enough, added to get rid of junk
            if (!preg_match('/^[A-Za-z0-9]/',$val)) {
                continue;
            }

            list($key, $v) = explode(":", $val, 2);
            if (preg_match("/%\{(.+)\}/",$v,$matches)){
               $val = trim($this->elements[$matches[1]]);
               $v = preg_replace("/%\{$matches[1]\}/",$val,$v);
            }
            $this->elements[$key] = trim($v);
        }

        return TRUE;
    }

    //! Set DB username
    /*!
     * @param string $data username for DB connction
     *
     * @return void
     */
    public function setUsername($data) {
        $this->elements["sql_username"] = $data;
    }

    //! Set DB password
    /*!
     * @param string $data password for DB connection
     *
     * @return void
     */
    public function setPassword($data) {
        $this->elements["sql_password"] = $data;
    }
}
