<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2016-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Rares Harnagea <rares@linuxmagic.com>
 *
 * $Id$
 * 
 * Note: The starting point of this library was: https://github.com/PHPMailer/PHPMailer
 * class.smtp.php
 */


class WizardSmtp {
    //! SMTP line break constant.
    /*!
     * @var string
     */
    const CRLF = "\r\n";

    //! The SMTP port to use if one is not specified.
    /*!
     * @var integer
     */
    const DEFAULT_SMTP_PORT = 25;

    //! The maximum line length allowed by RFC 2822 section 2.1.1
    /*!
     * @var integer
     */
    const MAX_LINE_LENGTH = 998;

    //! Debug level for no output
    /*!
     */
    const DEBUG_OFF = 0;

    //! Debug level to show client -> server messages
    /*!
     */
    const DEBUG_CLIENT = 1;

    //! Debug level to show client -> server and server -> client messages
    /*!
     */
    const DEBUG_SERVER = 2;

    //! Debug level to show connection status, client -> server and server -> client messages
    /*!
     */
    const DEBUG_CONNECTION = 3;

    //! Debug level to show all messages
    /*!
     */
    const DEBUG_LOWLEVEL = 4;


    //! Whether to use VERP.
    /*!
     * @link http://en.wikipedia.org/wiki/Variable_envelope_return_path
     * @link http://www.postfix.org/VERP_README.html Info on VERP
     * @var bool
     */
    protected $_doVerp;

    //! The timeout value for connection, in seconds.
    /*!
     * Default of 5 minutes (300sec) is from RFC2821 section 4.5.3.2
     * This needs to be quite high to function correctly with hosts using greetdelay as an anti-spam measure.
     * @link http://tools.ietf.org/html/rfc2821#section-4.5.3.2
     * @var integer
     */
    protected $_timeout;

    //! The socket for the server connection.
    /*!
     * @var resource
     */
    protected $_smtpConn;

    //! Error information, if any, for the last SMTP command.
    /*!
     * @var array
     */
    protected $_error;

    //! The reply the server sent to us for HELO.
    /*!
     * If null, no HELO string has yet been received.
     * @var string|null
     */
    protected $_heloReply;

    //! The set of SMTP extensions sent in reply to EHLO command. capabilities
    /*!
     * Indexes of the array are extension names.
     * Value at index 'HELO' or 'EHLO' (according to command that was sent)
     * represents the server name. In case of HELO it is the only element of the array.
     * Other values can be bool TRUE or an array containing extension options.
     * If null, no HELO/EHLO string has yet been received.
     * @var array|null
     */
    protected $_serverCaps;

    //! The most recent reply received from the server.
    /*!
     * @var string
     */
    protected $_lastReply;

    //! The current debug level 
    /*!
     * @var integer
     */
    protected $_debugLevel;

    //! If transfer layer security is on/off
    /*!
     * @var bool
     */
    protected $_tls; 

    //! The hosting server.
    /*!
     * @var string
     */
    protected $_host;

    //! The port number.
    /*!
     * @var integer
     */
    protected $_port;

    //! The helo/ehlo.
    /*!
     * @var string
     */
    protected $_helo;

    //! Flag to store .
    /*!
     * @var bool
     */
    protected $_tls_support;


    //! Constructor
    /*!
     * @return void
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function __construct() {
        // initialize member variables
        $this->_doVerp      = FALSE;
        $this->_timeout     = 300;
        $this->_error       = array(
            'error'         => '',
            'detail'        => '',
            'smtp_code'     => '',
            'smtp_code_ex'  => ''
        );
        $this->_heloReply   = NULL;
        $this->_serverCaps  = NULL;
        $this->_lastReply   = '';
        $this->_debugLevel  = 0;
        $this->_tls         = FALSE; 
        $this->_tls_support = FALSE; 
        $this->_host        = NULL;
        $this->_port        = self::DEFAULT_SMTP_PORT;

        // By default, use the HTTP_HOST for the HELO host, or localhost if
        // HTTP_HOST is empty or invalid
        if (!isset($_SERVER['HTTP_HOST']) || !WizardValidator::isDomain($_SERVER['HTTP_HOST'])) {
            $this->_helo = 'localhost';
        } else {
            $this->_helo = $_SERVER['HTTP_HOST'];
        }
    }

    //! Destructor
    /*!
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function __destruct() {
        $this->quit();
    }

    //! Connect to an SMTP server.
    /*!
     * @param string $host SMTP server IP or host name
     * @param integer $port The port number to connect to
     * @param integer $timeout How long to wait for the connection to open
     * @param array $options An array of options for stream_context_create()
     *
     * @return bool Whether success
     */
    public function connect($options = array()) {
        // sanity check
        if (!isset($this->_host)) { 
            $this->setError('There is no host set, unable to connect');
            return FALSE;
        }

        if (!is_array($options)) { 
            $this->setError('The stream context must be an array');
            return FALSE;
        }
        

        static $streamok;
        //This is enabled by default since 5.0.0 but some providers disable it
        //Check this once and cache the result
        if (is_null($streamok)) {
            $streamok = function_exists('stream_socket_client');
        }

        // Make sure we are __not__ connected
        if ($this->connected()) {
            // Already connected, generate error
            $this->setError('Already connected to a server');
            return FALSE;
        }

        // Connect to the SMTP server
        $this->logMessage(
            "Connection: opening to $this->_host:$this->_port, timeout=$this->_timeout, options=".var_export($options, TRUE),
            self::DEBUG_CONNECTION
        );

        $errno = 0;
        $errstr = '';

        if ($streamok) {
            $this->_tls_support = TRUE;
            $socketContext = stream_context_create($options);
            stream_context_set_option($socketContext, 'ssl', 'verify_peer_name', false);
            $this->_smtpConn = stream_socket_client(
                $this->_host . ":" . $this->_port,
                $errno,
                $errstr,
                $this->_timeout,
                STREAM_CLIENT_CONNECT,
                $socketContext
            );
        } else {
            $this->_tls_support = FALSE;
            //Fall back to fsockopen which should work in more places, but is missing some features
            $this->logMessage(
                "Connection: stream_socket_client not available, falling back to fsockopen",
                self::DEBUG_CONNECTION
            );
            $this->_smtpConn = fsockopen(
                $host,
                $port,
                $errno,
                $errstr,
                $timeout
            );
        }

        // Verify we connected properly
        if (!is_resource($this->_smtpConn)) {
            $this->setError(
                'Failed to connect to server',
                $errno,
                $errstr
            );
            $this->logMessage(
                'SMTP ERROR: ' . $this->_error['error']
                . ": $errstr ($errno)",
                self::DEBUG_CLIENT
            );
            return FALSE;
        }
        $this->logMessage('Connection: opened', self::DEBUG_CONNECTION);

        // Get any announcement
        $announce = $this->getLines();
        $this->logMessage('SERVER -> CLIENT: ' . $announce, self::DEBUG_SERVER);
        return TRUE;
    }

    //! Initiate a TLS session.
    /*!
     * @return bool Whether Success
     */
    protected function startTLS() {
        if (!$this->sendCommand('STARTTLS', 'STARTTLS', 220)) {
            $this->logMessage('Server does not support TLS Connection', self::DEBUG_CONNECTION);
            return FALSE;
        }

        $this->logMessage('Starting TLS Connection', self::DEBUG_CONNECTION);
        //Allow the best TLS version(s) we can
        $crypto_method = STREAM_CRYPTO_METHOD_TLS_CLIENT;

        //PHP 5.6.7 dropped inclusion of TLS 1.1 and 1.2 in STREAM_CRYPTO_METHOD_TLS_CLIENT
        //so add them back in manually if we can
        if (defined('STREAM_CRYPTO_METHOD_TLSv1_2_CLIENT')) {
            $crypto_method |= STREAM_CRYPTO_METHOD_TLSv1_2_CLIENT;
            $crypto_method |= STREAM_CRYPTO_METHOD_TLSv1_1_CLIENT;
        }

        // Begin encrypted connection
        if (!stream_socket_enable_crypto(
            $this->_smtpConn,
            TRUE,
            $crypto_method
        )) {
            return FALSE;
        }
        return TRUE;
    }


    //! wrapper function to log / debug data and messages 
    /*!
     * @param string $message message that we want to log
     * @param int $level debug level
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    function logMessage($message, $level = 2) {
        // sanity check
        if (!isset($message)    || !WizardValidator::isStringNotEmpty($message)   ||
            !isset($level)      || !WizardValidator::isIntPositive($level)        ||
            $level > $this->_debugLevel) {

            return;
        } 

        // trim the message
        $message = trim($message);
        $message = preg_replace('/(\r\n|\r|\n)/ms', "", $message);
        // get the caller function
        $backtrace = debug_backtrace();

        if (count($backtrace) < 2) {
            $fn = "unknown function";
            $line = $backtrace[0]['line'];
        } else {
            $fn = $backtrace[1]['function'];
            // seems that may not have line in some cases...
            // if we go higher up the tree we do, but is it reliable
            // to do so?
            // this at least occurs when calling via a callback to this debug
            // function in a non-class
            if (isset($backtrace[0]['line'])) {
                $line = $backtrace[0]['line'];
            } else {
                $line = 'unknown';
            }
        }

        $message = $fn . " (line: $line) " . $message;

        print $message . "\n";
    }


    //! Perform SMTP authentication.
    /*!
     * @param string $username The user name
     * @param string $password The password
     * @param string $authtype The auth type (PLAIN, LOGIN, NTLM, CRAM-MD5, XOAUTH2)
     *
     * @return bool True if successfully authenticated.* @access public
     *
     * @note: Must be run after hello().
     * @see hello()
     */
    public function authenticate($username, $password, $authtype = NULL) {
        // sanity check
        if (!isset($username)   || !WizardValidator::isStringNotEmpty($username)  ||
            !isset($password)   || !WizardValidator::isStringNotEmpty($password)) {

            // $authtype is permitted to be NULL at this point
            $this->setError('Sanity check failed');
            return FALSE;
        }

        if (!$this->_serverCaps) {
            $this->setError('Authentication is not allowed before HELO/EHLO');
            return FALSE;
        }

        if (array_key_exists('EHLO', $this->_serverCaps)) {
        // SMTP extensions are available. Let's try to find a proper authentication method

            if (!array_key_exists('AUTH', $this->_serverCaps)) {
                $this->setError('Authentication is not allowed at this stage');
                // 'at this stage' means that auth may be allowed after the stage changes
                // e.g. after STARTTLS
                return FALSE;
            }

            $this->logMessage('Auth method requested: ' . ($authtype ? $authtype : 'UNKNOWN'), self::DEBUG_LOWLEVEL);
            $this->logMessage(
                'Auth methods available on the server: ' . implode(',', $this->_serverCaps['AUTH']),
                self::DEBUG_LOWLEVEL
            );

            if (empty($authtype)) {
                foreach (array('CRAM-MD5', 'LOGIN', 'PLAIN') as $method) {
                    if (in_array($method, $this->_serverCaps['AUTH'])) {
                        $authtype = $method;
                        break;
                    }
                }
                if (empty($authtype)) {
                    $this->setError('No supported authentication methods found');
                    return FALSE;
                }
                $this->logMessage('Auth method selected: '.$authtype, self::DEBUG_LOWLEVEL);
            }

            if (!in_array($authtype, $this->_serverCaps['AUTH'])) {
                $this->setError("The requested authentication method \"$authtype\" is not supported by the server");
                return FALSE;
            }
        } elseif (empty($authtype)) {
            $authtype = 'LOGIN';
        }
        switch ($authtype) {
            case 'PLAIN':
                // Start authentication
                if (!$this->sendCommand('AUTH', 'AUTH PLAIN', 334)) {
                    return FALSE;
                }
                // Send encoded username and password
                if (!$this->sendCommand(
                    'User & Password',
                    base64_encode("\0" . $username . "\0" . $password),
                    235)) {
                    return FALSE;
                }
                break;
            case 'LOGIN':
                // Start authentication
                if (!$this->sendCommand('AUTH', 'AUTH LOGIN', 334)) {
                    return FALSE;
                }
                if (!$this->sendCommand("Username", base64_encode($username), 334)) {
                    return FALSE;
                }
                if (!$this->sendCommand("Password", base64_encode($password), 235)) {
                    return FALSE;
                }
                break;
            case 'CRAM-MD5':
                // Start authentication
                if (!$this->sendCommand('AUTH CRAM-MD5', 'AUTH CRAM-MD5', 334)) {
                    return FALSE;
                }
                // Get the challenge
                $challenge = base64_decode(substr($this->_lastReply, 4));

                // Build the response
                $response = $username . ' ' . $this->hmac($challenge, $password);

                // send encoded credentials
                return $this->sendCommand('Username', base64_encode($response), 235);
            default:
                $this->setError("Authentication method \"$authtype\" is not supported");
                return FALSE;
        }
        return TRUE;
    }

    //! Helper function that will send the SMTP CID command if the server advertise CID
    /*!
     * @param string $cidType, the CID type e.g. COOKIE, HTML5CANVAS
     * @param string $cidKey, the CID key hash   
     *
     * @returns bool whether CID is enabled and successfully submitted
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     *
     * Note: XXX This needs to be called before Auth
     */
    public function sendCID($cidType, $cidKey) {
        if (!$this->getServerExt('CLIENTID')) {
            $this->logMessage('Server Extension CLIENTID NOT Advertised', self::DEBUG_LOWLEVEL);
            return FALSE;
        }
        // Validating the CID TYPE and KEY as per RFC
        // https://datatracker.ietf.org/doc/draft-storey-smtp-client-id/?include_text=1
        if (!isset($cidType) || !WizardValidator::isStringNotEmpty($cidType)) {
            $this->logMessage('No CID type provided!', self::DEBUG_LOWLEVEL);
            return FALSE;
        }
        if (!ctype_alnum($cidType)) {
            $this->logMessage('The cid type MUST include only Alphanumeric chars!', self::DEBUG_LOWLEVEL);
            $this->setError("The CID TYPE must include only alphanumeric chars");
            return FALSE;
        }
        if (strlen($cidType) > 16) {
            $this->logMessage('The cid type exceeded the maximum 16 chars!', self::DEBUG_LOWLEVEL);
            $this->setError("The CID TYPE must not exceed 16 chars");
            return FALSE;
        }
        if (!ctype_print($cidKey)) {
            $this->logMessage('The cid must include only printable chars!', self::DEBUG_LOWLEVEL);
            $this->setError("The CID MUST include only printable chars");
            return FALSE;
        }
        if (strlen($cidKey) > 128) {
            $this->logMessage('The cid exceeded the maximum 128 chars!', self::DEBUG_LOWLEVEL);
            $this->setError("The CID must not exceed 128 chars");
            return FALSE;
        }
        // XXX Add additional validation, like length allowed chars ... 
        if (!isset($cidKey) || !WizardValidator::isStringNotEmpty($cidKey)) {
            $this->logMessage('No CID hash provided!', self::DEBUG_LOWLEVEL);
            return FALSE;
        }
        

        if (!$this->sendCommand("CLIENTID Auth", "CLIENTID $cidType $cidKey", 250)) {
            $this->logMessage('The CLIENTID Auth was not accepted by the Server!', self::DEBUG_LOWLEVEL);
            return FALSE;
        }
        return TRUE;
    }

    //! Calculate an MD5 HMAC hash.
    /*!
     * @param string $data The data to hash
     * @param string $key  The key to hash with
     *
     * @return string the hash
     *
     * @note: works like hash_hmac('md5', $data, $key)
     *        in case that function is not available
     */
    protected function hmac($data = '', $key = '') {
        // sanity check
        if (!isset($data)   || !WizardValidator::isString($data)  ||
            !isset($key)    || !WizardValidator::isString($key)) {

            $this->setError('Sanity check failed');
            return '';
        }

        if (function_exists('hash_hmac')) {
            return hash_hmac('md5', $data, $key);
        }

        // The following borrowed from
        // http://php.net/manual/en/function.mhash.php#27225

        // RFC 2104 HMAC implementation for php.
        // Creates an md5 HMAC.
        // Eliminates the need to install mhash to compute a HMAC
        // by Lance Rushing

        $bytelen = 64; // byte length for md5
        if (strlen($key) > $bytelen) {
            $key = pack('H*', md5($key));
        }
        $key = str_pad($key, $bytelen, chr(0x00));
        $ipad = str_pad('', $bytelen, chr(0x36));
        $opad = str_pad('', $bytelen, chr(0x5c));
        $kIpad = $key ^ $ipad;
        $kOpad = $key ^ $opad;

        return md5($kOpad . pack('H*', md5($kIpad . $data)));
    }

    //! Check connection state.
    /*!
     * @return bool True if connected.
     */
    public function connected() {
        if (is_resource($this->_smtpConn)) {
            $sock_status = stream_get_meta_data($this->_smtpConn);
            if ($sock_status['eof']) {
                // The socket is valid but we are not connected
                $this->logMessage(
                    'SMTP NOTICE: EOF caught while checking if connected',
                    self::DEBUG_CLIENT
                );
                $this->close();
                return FALSE;
            }
            return TRUE; // everything looks good
        }
        return FALSE;
    }

    //! Close the socket and clean up the state of the class.
    /*!
     * @see quit()
     *
     * @note: Don't use this function without first trying to use QUIT.
     */
    public function close() {
        $this->setError('');
        $this->_serverCaps = NULL;
        $this->_heloReply = NULL;
        if (is_resource($this->_smtpConn)) {
            // close the connection and cleanup
            fclose($this->_smtpConn);
            $this->_smtpConn = NULL; //Makes for cleaner serialization
            $this->logMessage('Connection: closed', self::DEBUG_CONNECTION);
        }
    }

    //! Send an SMTP DATA command. Issues a data command and sends the msg_data 
    //  to the server, finalizing the mail transaction. $msgData is the message
    //  that is to be send with the headers. Each header needs to be on single line
    //  followed by a <CRLF> with the message headers and the message body being 
    //  separated by and additional <CRLF>. Implements rfc 821: DATA <CRLF>
    /*!
     * @param string $msgData Message data to send
     *
     * @return bool Whether success
     */
    public function data($msgData) {
        // sanity check
        if (!isset($msgData) || !WizardValidator::isString($msgData)) {

            $this->setError('Sanity check failed');
            return FALSE;
        }

        //This will use the standard timelimit
        if (!$this->sendCommand('DATA', 'DATA', 354)) {
            return FALSE;
        }

        /* The server is ready to accept data!
         * According to rfc821 we should not send more than 1000 characters on a single line (including the CRLF)
         * so we will break the data up into lines by \r and/or \n then if needed we will break each of those into
         * smaller lines to fit within the limit.
         * We will also look for lines that start with a '.' and prepend an additional '.'.
         * NOTE: this does not count towards line-length limit.
         */
            
        // Normalize line breaks before exploding
        $lines = explode("\n", str_replace(array("\r\n", "\r"), "\n", $msgData));

        /* To distinguish between a complete RFC822 message and a plain message body, we check if the first field
         * of the first line (':' separated) does not contain a space then it _should_ be a header and we will
         * process all lines before a blank line as headers.
         */

        $field = substr($lines[0], 0, strpos($lines[0], ':'));
        $inHeaders = FALSE;
        if (!empty($field) && strpos($field, ' ') === FALSE) {
            $inHeaders = TRUE;
        }

        foreach ($lines as $line) {
            $linesOut = array();
            if ($inHeaders and $line == '') {
                $inHeaders = FALSE;
            }
            //Break this line up into several smaller lines if it's too long
            //Micro-optimisation: isset($str[$len]) is faster than (strlen($str) > $len),
            while (isset($line[self::MAX_LINE_LENGTH])) {
                //Working backwards, try to find a space within the last MAX_LINE_LENGTH chars of the line to break on
                //so as to avoid breaking in the middle of a word
                $pos = strrpos(substr($line, 0, self::MAX_LINE_LENGTH), ' ');
                //Deliberately matches both false and 0
                if (!$pos) {
                    //No nice break found, add a hard break
                    $pos = self::MAX_LINE_LENGTH - 1;
                    $linesOut[] = substr($line, 0, $pos);
                    $line = substr($line, $pos);
                } else {
                    //Break at the found point
                    $linesOut[] = substr($line, 0, $pos);
                    //Move along by the amount we dealt with
                    $line = substr($line, $pos + 1);
                }
                //If processing headers add a LWSP-char to the front of new line RFC822 section 3.1.1
                if ($inHeaders) {
                    $line = "\t" . $line;
                }
            }
            $linesOut[] = $line;

            //Send the lines to the server
            foreach ($linesOut as $lineOut) {
                //RFC2821 section 4.5.2
                if (!empty($lineOut) and $lineOut[0] == '.') {
                    $lineOut = '.' . $lineOut;
                }
                $this->clientSend($lineOut . self::CRLF);
            }
        }

        //Message data has been sent, complete the command
        $result = $this->sendCommand('DATA END', '.', 250);
        return $result;
    }

    //! Send an SMTP HELO or EHLO command. 
    //  Used to identify the sending server to the receiving server.
    //  This makes sure that client and server are in a known state. 
    //  Implements RFC 821: HELO <sp> <domain> and RFC 2821 EHLo.
    /*!
     * @return bool Whether success
     */
    public function hello() {
        // sanity check
        //Try extended hello first (RFC 2821)
        return (boolean)($this->sendHello('EHLO', $this->_helo) or $this->sendHello('HELO', $this->_helo));
    }

    //! Send an SMTP HELO or EHLO command. Low-level implementation used by hello()
    /*!
     * @see hello()
     *
     * @param string $hello The HELO string
     * @param string $host The hostname to say we are
     *
     * @return bool Whether success
     */
    protected function sendHello($hello, $host) {
        // sanity check
        if (!isset($hello)  || !WizardValidator::isString($hello) ||
            !isset($host)   || !WizardValidator::isString($host)) {

            $this->setError('Sanity check failed');
            return FALSE;
        }

        $helohost = $host;

        // if $host is an IP address...
        if (WizardValidator::isIPv4($host)) {
            // then we want to play nice and 'wrap' it
            $helohost = '[' . $host . ']';
        }

        $noerror = $this->sendCommand($hello, $hello . ' ' . $helohost, 250);
        $this->_heloReply = $this->_lastReply;
        if ($noerror) {
            $this->parseHelloFields($hello);
        } else {
            $this->_serverCaps = NULL;
        }
        return $noerror;
    }

    //! Parse a reply to HELO/EHLO command to discover server extensions.
    //  In case of HELO, the only parameter that can be discovered is a server name.
    /*!
     *
     * @param string $type - 'HELO' or 'EHLO'
     */
    protected function parseHelloFields($type) {
        // sanity check
        if (!isset($type) || !WizardValidator::isString($type) ||
            ($type !== 'HELO' && $type !== 'EHLO')) {

            $this->setError('Sanity check failed');
            return;
        }

        $this->_serverCaps = array();
        $lines = explode("\n", $this->_heloReply);

        foreach ($lines as $n => $s) {
            //First 4 chars contain response code followed by - or space
            $s = trim(substr($s, 4));
            if (empty($s)) {
                continue;
            }
            $fields = explode(' ', $s);
            if (!empty($fields)) {
                if (!$n) {
                    $name = $type;
                    $fields = $fields[0];
                } else {
                    $name = array_shift($fields);
                    switch ($name) {
                        case 'SIZE':
                            $fields = ($fields ? $fields[0] : 0);
                            break;
                        case 'AUTH':
                            if (!is_array($fields)) {
                                $fields = array();
                            }
                            break;
                        default:
                            $fields = TRUE;
                    }
                }
                $this->_serverCaps[$name] = $fields;
            }
        }
    }

    //! Send an SMTP MAIL command. starts a mail transaction from the email address
    //  specified in $from. Returns true if successful or false otherwise. If True the
    //  mail transaction is started adn then one or more recipient commands may be called
    //  followed by a data command. Implemnts rfc 821: MAIL <sp> FROM: <reverse-path> <CRLF>
    /*!
     * @param string $from Source address of this message
     *
     * @return bool Whether success
     */
    public function mailFrom($from) {
        // sanity check
        if (!isset($from) || !WizardValidator::isEmail($from)) {

            $this->setError('Sanity check failed');
            return FALSE;
        }

        $useVerp = ($this->_doVerp ? ' XVERP' : '');
        return $this->sendCommand(
            'MAIL FROM',
            'MAIL FROM:<' . $from . '>' . $useVerp,
            250
        );
    }

    //! Send an SMTP QUIT command. closes the socket if there is no error or the $close_on_error
    //  argument is true. Implements from rfc 821: QUIT <CRLF>
    /*!
     * @param bool $close_on_error Should the connection close if an error occurs?
     * 
     * @return bool Whether success
     */
    public function quit($close_on_error = TRUE) {
        // sanity check
        if (!isset($close_on_error) || !WizardValidator::isBool($close_on_error)) {

            $this->setError('Sanity check failed');
            return FALSE;
        }

        $noerror = $this->sendCommand('QUIT', 'QUIT', 221);
        $err = $this->_error; //Save any error
        if ($noerror or $close_on_error) {
            $this->close();
            $this->_error = $err; //Restore any error from the quit command
        }
        return $noerror;
    }

    //! Send an SMTP RCPT command. Sets the TO argument to $toaddr. Implements from rfc 821:
    //  RCPT <SP> TO:<forward-path> <CRLF>
    /*!
     * @param string $address The address the message is being sent to
     *
     * @return bool Whether success
     */
    public function recipient($address) {
        // sanity check
        if (!isset($address) || !WizardValidator::isEmail($address)) {

            $this->setError('Sanity check failed');
            return FALSE;
        }

        return $this->sendCommand(
            'RCPT TO',
            'RCPT TO:<' . $address . '>',
            array(250, 251)
        );
    }

    //! Send an SMTP RSET command. Abort any transaction that is currently is progress.
    //  Implements rfc 821: RSET <CRLF>
    /*!
     * @return bool True on success.
     */
    public function reset() {
        return $this->sendCommand('RSET', 'RSET', 250);
    }

    //! Send a command to an SMTP server and check its return code.
    /*!
     * @param string $command The command name - not sent to the server
     * @param string $commandstring The actual command to send
     * @param integer|array $expect One or more expected integer success codes
     *
     * @return bool True on success.
     */
    protected function sendCommand($command, $commandstring, $expect) {
        // sanity check
        if (!isset($command)        || !WizardValidator::isStringNotEmpty($command)        ||
            !isset($commandstring)  || !WizardValidator::isStringNotEmpty($commandstring)  ||
            !isset($expect)         || 
            (!WizardValidator::isInt($expect) && !WizardValidator::isArray($expect))){

            $this->setError('Sanity check failed');
            return FALSE;
        }

        if (!$this->connected()) {
            $this->setError("Called $command without being connected");
            return FALSE;
        }
        //Reject line breaks in all commands
        if (strpos($commandstring, "\n") !== FALSE or strpos($commandstring, "\r") !== FALSE) {
            $this->setError("Command '$command' contained line breaks");
            return FALSE;
        }
        $this->clientSend($commandstring . self::CRLF);

        $this->_lastReply = $this->getLines();
        // Fetch SMTP code and possible error code explanation
        $matches = array();
        // 550 User [rares1@linuxmagic.com] does not exist (#5.5.5)
        if (preg_match("/^([0-9]{3})[ -](.*)/", $this->_lastReply, $matches)) {
            $code = $matches[1];
            if (is_int($expect)) {
                $code_ex = $expect;
            } else {
                $code_ex = join(', ', $expect);
            }
            // Cut off error code from each response line
            $detail = $matches[2];
        } else {

            $this->logMessage('without regex');
            // Fall back to simple parsing if regex fails
            $code = substr($this->_lastReply, 0, 3);
            $code_ex = NULL;
            $detail = substr($this->_lastReply, 4);
        }

        $this->logMessage('SERVER -> CLIENT: ' . $this->_lastReply, self::DEBUG_SERVER);

        if (!in_array($code, (array)$expect)) {
            $this->setError(
                "$command command failed",
                $detail,
                $code,
                $code_ex
            );
            $this->logMessage(
                'SMTP ERROR: ' . $this->_error['error'] . ': ' . $this->_lastReply,
                self::DEBUG_CLIENT
            );

            $this->setError("Mail server replied $this->_lastReply, $code, $code_ex");
            $this->setError('Mail server replied', $this->_lastReply, $code, $code_ex);
            return FALSE;
        }

        $this->setError('');
        return TRUE;
    }

    //! Send an SMTP SAML command. Starts a mail transaction from the email address
    //  specified in $from. On success, the mail transaction is started and then one or
    //  more recipient will sned the message to the users terminal if they are logged in
    //  and send them an email. Implements rfc 821: SAML <SP> FROM:<reverse-path> <CRLF>
    /*!
     * @param string $from The address the message is from
     *
     * @return bool Whether success
     */
    public function sendAndMail($from) {
        // sanity check
        if (!isset($from) || !WizardValidator::isEmail($from)) {

            $this->setError('Sanity check failed');
            return FALSE;
        }

        return $this->sendCommand('SAML', "SAML FROM:$from", 250);
    }

    //! Send an SMTP VRFY command.
    /*!
     * @param string $name The name to verify
     *
     * @return bool Whether success
     */
    public function verify($name) {
        // sanity check
        if (!isset($name) || !WizardValidator::isStringNotEmpty($name)) {

            $this->setError('Sanity check failed');
            return FALSE;
        }

        return $this->sendCommand('VRFY', "VRFY $name", array(250, 251));
    }

    //! Send an SMTP NOOP command. Used to keep keep-alives alive, 
    //  doesn't actually do anything
    /*!
     * @return bool Whether success
     */
    public function noop() {
        return $this->sendCommand('NOOP', 'NOOP', 250);
    }

    //! Send raw data to the server.
    /*!
     * @param string $data The data to send
     * 
     * @return integer|bool The number of bytes sent to the server or false on error
     */
    public function clientSend($data) {
        // sanity check
        if (!isset($data) || !WizardValidator::isString($data)) {

            $this->setError('Sanity check failed');
            return FALSE;
        }

        $this->logMessage("CLIENT -> SERVER: $data", self::DEBUG_CLIENT);
        return fwrite($this->_smtpConn, $data);
    }

    //! Get the latest error.
    /*!
     * @return array the error
     */
    public function getError() {
        return $this->_error;
    }

    //! Get SMTP extensions available on the server
    /*!
     * @return array|null the array of available extensions
     */
    public function getServerExtList() {
        return $this->_serverCaps;
    }

    //! A multipurpose method
    /*!
     * @param string $name Name of SMTP extension or 'HELO'|'EHLO'
     *
     * @return mixed 
     *
     * @note: The method works in three ways, dependent on argument value and current state
     *          1. HELO/EHLO was not sent - returns null and set up $this->_error
     *          2. HELO was sent
     *            $name = 'HELO': returns server name
     *            $name = 'EHLO': returns bool false
     *            $name = any string: returns null and set up $this->_error
     *          3. EHLO was sent
     *            $name = 'HELO'|'EHLO': returns server name
     *            $name = any string: if extension $name exists, returns bool True
     *              or its options. Otherwise returns bool False
     *        In other words, one can use this method to detect 3 conditions:
     *         - null returned: handshake was not or we don't know about ext (refer to $this->_error)
     *         - false returned: the requested feature exactly not exists
     *         - positive value returned: the requested feature exists
     */
    public function getServerExt($name = '') {
        // sanity check
        if (!isset($name) || !WizardValidator::isString($name)) {

            $this->setError('Sanity check failed');
            return FALSE;
        }

        if (!$this->_serverCaps) {
            $this->setError('No Server capabilities was sent');
            return NULL;
        }

        // the tight logic knot ;)
        if (!array_key_exists($name, $this->_serverCaps)) {
            if ($name == 'HELO') {
                return $this->_serverCaps['EHLO'];
            }
            if ($name == 'EHLO' || array_key_exists('EHLO', $this->_serverCaps)) {
                return FALSE;
            }
            $this->setError('HELO handshake was used. Client knows nothing about server extensions');
            return NULL;
        }

        return $this->_serverCaps[$name];
    }

    //! Get the last reply from the server.
    /*!
     * @return string the last reply from the server
     */
    public function getLastReply() {
        return $this->_lastReply;
    }

    //! Read the SMTP server's response.
    /*!
     * @return string The server's response
     */
    protected function getLines() {
        // If the connection is bad, give up straight away
        if (!is_resource($this->_smtpConn)) {
            return '';
        }

        $data = '';
        $endtime = 0;
        stream_set_timeout($this->_smtpConn, $this->_timeout);
        while (is_resource($this->_smtpConn) && !feof($this->_smtpConn)) {
            $str = @fgets($this->_smtpConn, 515);
            $this->logMessage("SMTP -> getLines(): \$data is \"$data\"", self::DEBUG_LOWLEVEL);
            $this->logMessage("SMTP -> getLines(): \$str is  \"$str\"", self::DEBUG_LOWLEVEL);
            $data .= $str;
            // If 4th character is a space, we are done reading, break the loop, micro-optimisation over strlen
            if ((isset($str[3]) and $str[3] == ' ')) {
                break;
            }
            // Timed-out? Log and break
            $info = stream_get_meta_data($this->_smtpConn);
            if ($info['timed_out']) {
                $this->logMessage(
                    'SMTP -> getLines(): timed-out (' . $this->Timeout . ' sec)',
                    self::DEBUG_LOWLEVEL
                );
                break;
            }
        }
        return $data;
    }

    //! Enable or disable VERP address generation.
    /*!
     * @param bool $enabled TRUE to set enable, FALSE to set disable
     */
    public function setVerp($enabled = FALSE) {
        // sanity check
        if (!isset($enabled) || !WizardValidator::isBool($enabled)) {

            $this->setError('Sanity check failed');
            return;
        }

        $this->_doVerp = $enabled;
    }

    //! Get VERP address generation mode.
    /*!
     * @return bool Whether address generate is enabled or not
     */
    public function getVerp() {
        return $this->_doVerp;
    }

    //! Set error messages and codes.
    /*!
     * @param string $message The error message
     * @param string $detail Further detail on the error
     * @param string $smtp_code An associated SMTP error code
     * @param string $smtp_code_ex Extended SMTP code
     */
    protected function setError($message, $detail = '', $smtp_code = '', $smtp_code_ex = '') {
        // sanity check
        if (!isset($message)        || !WizardValidator::isString($message)            ||
            !isset($detail)         || !WizardValidator::isString($detail)             ||
            !isset($smtp_code)      || !WizardValidator::isString((string)$smtp_code)  ||
            !isset($smtp_code_ex)   || !WizardValidator::isString((string)($smtp_code_ex))) {

            return;
        }

        $this->_error = array(
            'error'         => $message,
            'detail'        => $detail,
            'smtp_code'     => $smtp_code,
            'smtp_code_ex'  => $smtp_code_ex
        );
    }

    //! Set debug level
    /*!
     * @param integer $level debug level
     *
     * @author anonymous
     */
    public function setDebugLevel($level) {
        // sanity check
        if (!isset($level)              || !WizardValidator::isInt($level) ||
            $level < self::DEBUG_OFF    || $level > self::DEBUG_LOWLEVEL) {
            return FALSE;
        }
        $this->_debugLevel = $level;
        return TRUE;
    }

    //! Get debug level
    /*!
     * @return debug level
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function getDebugLevel() {
        return $this->_debugLevel;
    }

    //! Set tls
    /*!
     * @param bool $tls the bool value to enable to disable tls 
     *
     * @return bool TRUE if successfully set the prefix
     *              FALSE otherwise
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function setTls($tls) {
        // sanity check
        if (!isset($tls) || !WizardValidator::isBool($tls)) {
            return FALSE;
        }
        if ($tls === TRUE && !defined('OPENSSL_ALGO_SHA1')) {
            $this->logMessage("Unable to use TLS, PHP module openssl is not installed!");
            return FALSE;
        }
        $this->_tls = $tls;
        
        return TRUE;
    }

    //! Get prefix
    /*!
     * @return string prefix
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function getTls() {
        return $this->_tls;
    }

    //! Set host
    /*!
     * @param string $host host
     *
     * @return bool TRUE if successfully set the host
     *              FALSE otherwise
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function setHost($host) {
        if (!isset($host) || !WizardValidator::isDomain($host)) {
            return FALSE;
        }
        $this->_host = $host;
        return TRUE;
    }

    //! Get host
    /*!
     * @return string host
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function getHost() {
        return $this->_host;
    }

    //! Set port
    /*!
     * @param string $port port
     *
     * @return bool TRUE if successfully set the port
     *              FALSE otherwise
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function setPort($port) {
        if (!isset($port) || !WizardValidator::isInt($port)) {
            return FALSE;
        }
        $this->_port = $port;
        return TRUE;
    }

    //! Get port
    /*!
     * @return string port
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function getPort() {
        return $this->_port;
    }

    //! Set helo
    /*!
     * @param string $helo
     *
     * @return bool TRUE if successfully set the helo
     *              FALSE otherwise
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function setHelo($helo) {
        if (!isset($helo) || !WV::isDomain($helo)) {
            $this->logMessage("The provided HELO [$helo] is invalid!");
            return FALSE;
        }
        $this->_helo = $helo;
        return TRUE;
    }

    //! Get helo
    /*!
     * @return string helo
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function getHelo() {
        return $this->_helo;
    }
}
