<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicMail (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2017-2018 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Rares Harnagea <rares@linuxmagic.com>
 *
 * $Id$
 *
 * Usage:
 * // construct the object and load the file optional
 * // you can call the constructor without arguments and use the method 
 * // WizardSource::loadFile('path-to-file'); to load the file
 * $sourceObj = new WizardSource('/etc/magic-mail/control/mmadmin_config.php');
 * 
 * // Helper method to get all the elements, returns a hash array key=>value
 * $myArray = $sourceObj->getElements();
 * print_r($myArray);
 * // Method to retrieve one element, if the element does not exist will return an empty string 
 * // Note that any element will be casted to string
 * $sourceObj->get('sql_host');
 */

//require_once('lib.wizard_validator.php');
//WizardValidator');

// require_once('/usr/share/magicmail/php/library/lib.wizard_validator.php');

class WizardSource {

    //! the max file size for reading with the getControl
    // 5 MB, this function may be used by other  
    // if the file size use a different mechanism 
    const MAX_CONTROL_FILE_SIZE = 5000000;

    //! Holdes the member variables
    /*!
     * @var array
     */
    protected $elements = array();

    //! Holds the source path
    /*!
     * @var string
     */
    protected $source = '';

    public function __construct($filename = '') {
        if (!empty($filename)) {
            $this->loadfile($filename);
        }
    }

    //! Get a config element.
    /*!
     * @param string $key The name of the config item to retrieve.
     * @param mixed $default the value that will be returned if $key does not exist
     *
     * @return string The config item's value or default.
     */
    public function get($key = '', $default = '') {
        return  !WizardValidator::isStringNotEmpty($key)    ||
                !isset($this->elements[$key])               ? 
                $default : $this->elements[$key];
    }

    //! Get config elements
    /*!
     * @return array The config elements
     */
    public function getElements() {
        return $this->elements;
    }

    //! Set a config element.
    /*!
     * @param string $key The name of the config item to set.
     * @param string $val the value that we want to set.
     *
     * @return bool if the value was successfully set
     */
    public function set($key = '', $val = '') {
        // sanity check
        if (!WizardValidator::isStringNotEmpty((string) $key) ||
            !WizardValidator::isString((string) $val)) {
            return FALSE;
        }

        $this->elements[$key] = $val;
        return TRUE;
    }

    //! Load a configuration from key: value entries in a local file
    /*!
     * @param string $filename Name of the config file to load.
     *
     * @return bool True if the file was loaded successfully.
     */
    public function loadfile($filename = '') {
        // sanity check
        if (!WizardValidator::isStringNotEmpty((string) $filename) ||
            !is_readable($filename)) {
            return FALSE;
        }

        // store the source config file path
        $this->source = $filename;

        // get data from source config file
        if (!$data = file($this->source)) {
            // Reset internal state to clean
            $this->reset();
            return FALSE;
        }

        // loop through the data
        for ($i = 0; $i < count($data); $i++) {
            $val = trim($data[$i]);
            // If empty line or does not start with alpha char let's ignore the line
            // check if we have the ':' since this is required to know which part is 
            // the key and which one is the value
            if ($val === '' || ctype_alpha($val[0]) === FALSE || strpos($val, ':') === FALSE) {
                continue;
            }

            list($key, $value) = explode(":", $val, 2);
            // trim the parts
            $key = trim($key);
            $value = trim($value);
            // set the element
            $this->elements[$key] = $value;
        }

        return TRUE;
    }
    
    //! Reset the loaded config values.
    /*!
     */
    public function reset() {
        $this->source = "";
        $this->elements = array();
    }

    //! retrieve a control file from the underlying filesystem
    /*!
     * @param string $file the file name
     * @param mixed $fallback fallback in case there is problem getting the value of the control file
     *
     * @returns mixed the value in the file or NULL if no value is set
     */
    public static function &getControl($file, $fallback = '') {
        if (!isset($file) || !WizardValidator::isStringNotEmpty($file) || !is_readable($file)) {
            // This is to noise
            // WizardDebug::log("The file [$file] path argument is not set properly or not readable");
            return $fallback;
        }   
        $filesize = filesize($file);
        if ($filesize === FALSE || $filesize > self::MAX_CONTROL_FILE_SIZE) {
            WizardDebug::log("The file [$file] exceeded the limit for this method");
            return $fallback;
        } 

        if (($data = file_get_contents($file)) === FALSE) {
            return $fallback;
        }         
        $data = trim($data);
        // if the file is empty
        if ($data === '') {
            return $fallback;
        }
        return $data;
    }

    //! retrieve the content of a file as JSON from the underlying filesystem
    /*!
     * @param string $file the file name
     * @param mixed $fallback fallback in case there is problem getting the value of the control file
     *
     * @returns mixed the value in the file or NULL if no value is set
     */
    public static function &getJSONFile($file, $fallback = NULL) {
        // make sure that the argument is string, self::getControl will check if is a file
        if (!isset($file) || !WizardValidator::isStringNotEmpty($file)) {
            WizardDebug::log('The file path argument is not set properly');
            return $fallback;
        }   
        // get the content 
        if (($content = self::getControl($file, $fallback)) === $fallback) {
            WizardDebug::log("Unable to get the file [$file] content");
            return $fallback;
        }
        // decode the content to array
        $json = json_decode($content, TRUE);
        // json_decode could return FALSE or NULL if there are problem decoding the JSON 
        if (empty($json)) {
            WizardDebug::log("Unable to decode the file [$file] to JSON, last error " .  json_last_error());
            return $fallback;
        }
        return $json;
    }    
    
}
