<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2016-2018 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Rares Harnagea <rares@linuxmagic.com>
 *
 * $Id: lib.wizard_validator.php 154 2017-03-07 18:24:37Z rares $
 */

// we create this class so that we have a shorter name for our WizardValidator class
// this could be usefull in development, but for production let's do a replace
// s/WV/WizardValidator/g
class WV extends WizardValidator {}

class WizardValidator {

    //! Validate if a variable is boolean 
    /*!
     * @param string $bool the variable that you want to check 
     *
     * @return bool whether the variable is a boolean
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public static function isBool($bool) {
        return  isset($bool) &&
                is_bool($bool);
    }

    //! Validate if a variable is string could be an empty string
    /*!
     * @param string $string the variable that you want to check 
     *
     * @return bool whether the variable is a string 
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     * 
     * SIDE EFFECTS: this function will return FALSE if you pass an integer that 
     * is NOT cast to string 
     */
    public static function isString($string) {
        return  isset($string) &&
                is_string($string);
    }

    //! Validate if a variable is string and is not empty
    /*!
     * @param string $string the variable that you want to check 
     *
     * @return bool whether the variable is a string 
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     * 
     * SIDE EFFECTS: this function will return FALSE if you pass an integer that 
     * is NOT cast to string 
     */
    public static function isStringNotEmpty($string) {
        return  isset($string)      &&
                is_string($string)  &&
                !empty($string)     &&
                strlen(trim($string)) !== 0;
    }

    //! Validate if a variable is type integer
    /*!
     * @param mixed $int the variable that you want to check 
     *
     * @return bool whether the variable is an int 
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public static function isTypeInt($int) {
        // Do not use empty() here since it will trigger FALSE on zero 
        return  isset($int)         &&
                is_int($int)        &&
                $int <= PHP_INT_MAX;
    }

    //! Validate if a variable is integer
    /*!
     * @param mixed $int the variable that you want to check 
     *
     * @return bool whether the variable is an int 
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public static function isInt($int) {
        // Do not use empty() here since it will trigger FALSE on zero 
        return  isset($int)                 &&
                is_numeric($int)            &&
                intval($int) == $int        &&
                $int <= PHP_INT_MAX;
    }

    //! Validate if a variable is integer and greater than 0
    /*!
     * @param mixed $int the variable that you want to check 
     *
     * @return bool whether the variable is an int that's greater than 0
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public static function isIntPositive($int) {
        return  isset($int)         &&
                static::isInt($int)   &&
                intval($int) > 0; 
    }

    //! Validate if a variable is integer and greater or equal to 0
    /*!
     * @param mixed $int the variable that you want to check 
     *
     * @return bool whether the variable is an int that's greater or equal to 0
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public static function isIntPositiveOrZero($int) {
        return  isset($int)         &&
                static::isInt($int)   &&
                intval($int) >= 0; 
    }

    //! Validate if a variable an array 
    /*!
     * @param mixed $array the variable that you want to check 
     *
     * @return bool whether the variable is an array 
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public static function isArray($array) {
        return  isset($array) &&
                is_array($array);
    }

    //! Validate if a variable an array that is not empty 
    /*!
     * @param mixed $array the variable that you want to check 
     *
     * @return bool whether the variable is an array that not empty
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public static function isArrayNotEmpty($array) {
        return  isset($array)       &&
                is_array($array)    &&
                !empty($array);
    }

    //! Validate if a variable an object
    /*!
     * @param mixed $object the variable that you want to check 
     *
     * @return bool whether the variable is an object
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public static function isObject($object) {
        return  isset($object) &&
                is_object($object);
    }

    //! Validate if a variable is a well formed email address 
    /*!
     * @param string $email the variable that you want to check 
     *
     * @return bool whether the variable is a email
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public static function isEmail($email) {
        return  isset($email)                   &&
                static::isStringNotEmpty($email)  &&
                strlen($email) > 4              &&
                filter_var($email, FILTER_VALIDATE_EMAIL);
    }

    //! Helper function that Validate if $email is a well formed email address and set split the email into local part and domain
    /*!
     * @param string $email the variable that you want to check 
     * @param string $user reference where we set the local part if the email is valid
     * @param string $domain reference where we set the domain part if the email is valid 
     *
     * @return bool whether the variable is a well formed email
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public static function splitEmail($email, &$user, &$domain) {
        if (!isset($email) || !static::isEmail($email)) {
            return FALSE;
        }
        list($user, $domain) = explode('@', $email);
        return TRUE;
    }


    //! Validate if a variable is a well formed email address 
    /*!
     * @param string $email the variable that you want to check 
     *
     * @return bool whether the variable is a email
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public static function isEmailMX($email) {
        // First check if is well formed
        if (!isset($email) || !static::splitEmail($email, $user, $domain)) {
            return FALSE;
        }

        return checkdnsrr($domain, 'MX');
    }

    //! Validate if a variable is a well formed domain name 
    /*!
     * @param string $domain the variable that you want to check 
     *
     * @return bool whether the variable is a domain
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public static function isDomain($domain) {
        return  isset($domain)                  &&
                static::isStringNotEmpty($domain) &&
                strlen($domain) > 5             &&
                static::isUrl('http://' . $domain);
    }
    
    //! Validate if a variable is a well formed URL 
    /*!
     * @param string $url the variable that you want to check 
     *
     * @return bool whether the variable is a URL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public static function isUrl($url) {
        return  isset($url)                     &&
                static::isStringNotEmpty($url)    &&
                strlen($url) > 8                &&
                filter_var($url, FILTER_VALIDATE_URL);
    }
   
    //! Validate if a variable is a IP address V4
    /*!
     * @param string $ip the variable that you want to check 
     *
     * @return bool whether the variable is a valid IP addres v4
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     * 
     * Note: the IP may be bogon or on a reserved range 
     */
    public static function isIpv4($ip) {
        return  isset($ip)                     &&
                static::isStringNotEmpty($ip)    &&
                strlen($ip) >= 7               &&
                filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4);
    }
    
    //! Validate if a variable is a valid IP CIDR V4
    /*!
     * @param string $ip the variable that you want to check 
     *
     * @param bool $returnLowAddr set to true if you want this function 
     *                            to return the correct CIDR in case the last
     *                            byte you passed is incorrect
     *
     * @return mixed bool whether the variable is a valid CIDR, or the correct CIDR in case you set 
     *                    the second arg
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     * 
     * Note: the IP may be bogon or on a reserved range 
     */
    public static function isCidr($cidr, $returnLowAddr = FALSE) {
        if (!isset($cidr) || !static::isStringNotEmpty($cidr) || strlen($cidr) < 7) {
            return FALSE;
        }               
        $cidr = trim($cidr);
        // if we don't have a / then is just an IP address
        if (strpos($cidr, '/') === FALSE) {
            return static::isIPv4($cidr);
        }
        // use the helper function to get the parts of the CIDR
        // if the CIDR is not valid this function will return FALSE,otherwise will return an array
        // with network, broadcase and subnet

        if (($cidrParts = static::_getCidrParts($cidr)) === FALSE) {
            return FALSE;
        }

        // now we have to deal with a network and a mask
        list($ip, $subnet) = explode('/', $cidr, 2);
        list($network, $broadcast, $subnet) = $cidrParts;

        // convert the low or network back to a address to make sure that
        // it's matching the IP part
        $networkIp = long2Ip($network);
        if ($returnLowAddr === FALSE) {
            return $networkIp === $ip;
        } else {
            if ($networkIp !== $ip) {
                // this return is the correct format of the network, probably the user had the last byte
                // incorrect but we were able to determine the network
                return "$networkIp/$subnet";
            }
            // Everything is ok the CIDR is valid
            return $cidr;
        }
    }

    //! helper function that will return the parts of a CIDR
    /*!
     * @param string $cidr the CIDR that need to be splitted to parts
     *
     * @return mixed the array with $network, $broadcast, $subnet or bool FALSE if there was a problem
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    protected static function _getCidrParts($cidr) {
        if (!isset($cidr) || !static::isStringNotEmpty($cidr) || strlen($cidr) < 7) {
            return FALSE;
        }               
        $cidr = trim($cidr);
        // if we don't have a / then is just an IP address
        if (strpos($cidr, '/') === FALSE) {
            return FALSE;
        }
        // now we have to deal with a network and a mask
        list($ip, $subnet) = explode('/', $cidr, 2);
        // make sure that we have something for both IP and subnet
        if (!isset($ip) || empty($ip) || !isset($subnet) || empty($subnet)) {
            return FALSE;
        }
        // make sure that the IP part is valid
        if (static::isIpV4($ip) === FALSE) {
            return FALSE;
        }
        // the subnet needs to be between 1 and 32
        if ($subnet < 1 || $subnet > 32) {
            return FALSE;
        }

        // conver the IP to long     
        $longIp = ip2long($ip);
    
        // convert the mask to long and shift the bits
        $mask = (0xFFFFFFFF << (32 - $subnet)) & 0xFFFFFFFF;

        // now we need to determine the low and high or network and broadcast address
        $network = $longIp & $mask;

        $broadcast  = $longIp | (~$mask & 0xFFFFFFFF);
        $ret = array($network, $broadcast, $subnet);
        return $ret;
    }

    //! helper function to check if an IP is in a network
    /*!
     * @param string $ip the IP address that need to be checked if is in the CIDR network
     *
     * @param string $cidr the CIDR that need to be checked
     *
     * @return bool TRUE if the IP is the CIDR range, FALSE otherwise
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public static function isIpInCidr($ip, $cidr) {
        if (!isset($ip) || !static::isStringNotEmpty($ip) || strlen($ip) < 7) {
            return FALSE;
        }               
        $ip = trim($ip);
        if (static::isIpV4($ip) === FALSE) {
            return FALSE;
        }
        if (!isset($cidr) || !static::isStringNotEmpty($cidr) || strlen($cidr) < 7) {
            return FALSE;
        }               
        $cidr = trim($cidr);

        // if both are IP address
        if ($cidr === $ip) {
            return TRUE;
        }

        // use the helper function to get the parts of the CIDR
        // if the CIDR is not valid this function will return FALSE,otherwise will return an array
        // with network, broadcase and subnet
        if (($cidrParts = static::_getCidrParts($cidr)) === FALSE) {
            return FALSE;
        }

        list($network, $broadcast, $subnet) = $cidrParts;
        // now we know that we have a valid IP and a valid CIDR
        $ipLong = ip2long($ip);

        if ($ipLong < $network || $ipLong > $broadcast) {
            return FALSE;
        }
        return TRUE;
    }
}
