<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Lucas Nincao <lucasn@linuxmagic.com>
 *
 * $Id: countryAuthStatsModule.php 10602 2019-11-12 18:07:12Z deiony $
 */

require_once('wizard/lib.wizard_validator.php');
require_once('lib.basestatsmodule.php');
require_once('lib.authlogsearch.php');
require_once('lib.country.php');

class countryAuthStatsModule extends BaseStatsModule {
    // values for time period filter
    const PERIOD_ALL_TIME = 0;
    const PERIOD_CURR_DAY = 1;

    // default values for filters
    const DEFAULT_PERIOD = countryAuthStatsModule::PERIOD_CURR_DAY;
    const DEFAULT_TOP_NUM = 5;

    // arrays to store the allowed values posted from the form dropdown menus
    protected $allowedTopNum = array(0, 5, 10, 15, 20, 25, 30);
    protected $allowedTimePeriod = array(
        countryAuthStatsModule::PERIOD_ALL_TIME,
        countryAuthStatsModule::PERIOD_CURR_DAY
    );

    // values posted on any of the dropdrowns change
    protected $topNum;
    protected $timePeriod;

    // array to store all the country auth stats to avoid parsing twice
    protected $authStatsData;

    //! class constructor
    public function __construct($l) {
        parent::__construct($l, FALSE);
        $this->topNum = countryAuthStatsModule::DEFAULT_TOP_NUM;
        $this->timePeriod = countryAuthStatsModule::DEFAULT_PERIOD;
        $this->authStatsData = NULL;
    }

    //! method to process any actions and react accordingly
    public function actionProcessor($action, &$errors, &$notices) {
        // set the posted filter values
        if (isset($_POST['top-auth'], $_POST['period-auth'])) {
            if (!$this->setTopNum($_POST['top-auth'])
                || !$this->setTimePeriod($_POST['period-auth'])) {
                $errors[] = intl('invalid_entry');
                return;
            }

            // save the POST in the SESSION for the future
            $_SESSION['auth_form'] = $_POST;
        }
    }

    //! method to render the module
    public function renderModule() {
        global $module;

        // Authentication: Top X countries: Pass and Block
        $dataAuth = array();

        $dataAuth[0]['table'] = $this->countryAuthStatsTable('PASS');
        $dataAuth[0]['headerLabel'] = intl('authCountryStatsHeaderPass');
        $dataAuth[0]['JSlabel'] = 'authStatsHelpPass';

        $dataAuth[1]['table'] = $this->countryAuthStatsTable('BLOCK');
        $dataAuth[1]['headerLabel'] = intl('authCountryStatsHeaderBlock');
        $dataAuth[1]['JSlabel'] = 'authStatsHelpBlock';

        // help bubble dictionary for the JS
        $this->_i18n_js[] = 'authStatsHelpPass';
        $this->_i18n_js[] = 'authStatsHelpBlock';

        renderTemplate('countryAuthStats', array(
            'module'        => $module,
            'submodule'     => 'countryStats',
            'showSelection' => TRUE,
            'topNum'        => $this->topNum,
            'timePeriod'    => $this->timePeriod,
            'data'          => $dataAuth,
            'help'          => intl('help'),
        ));
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    public function getCSS() {
        return array();
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    public function getJS() {
        return array();
    }

    //! Creates a graph with the authentication stats, divided by country
    /*!
     * @param   string          PASS or BLOCK
     *
     * @returns string          stats graph html
     */
    protected function countryAuthStatsTable($status) {
        // if either of these variables isn't set, return default empty table
        if ($this->topNum === NULL || $this->timePeriod === NULL) {
            return $this->getDefaultTableHTML();
        }

        // check if already have this data before going through the logs again
        if ($this->authStatsData === NULL) {
            $this->authStatsData = $this->getAuthByCountryLogs();
        }

        // assign a friendly name
        $data = $this->authStatsData;

        // returns an empty table if there are
        // no countries for this authentication status
        if (empty($data[$status])) {
            return $this->getDefaultTableHTML();
        }

        // generate authentication stats table
        $table = $this->generateTable($data[$status]);

        // build the authentication stats table html string
        $html = $table->getRenderString();

        // return default empty table if an error occurs
        return $html ? $html : $this->getDefaultTableHTML();
    }

    //! Creates an array with all the authentication stats
    /*!
     * @returns array with all the email stats from the logs
     *
     * NOTE: separated by PASS/BLOCK -> countries
     */
    protected function getAuthByCountryLogs() {
        $authLogSearch = new AuthLogSearch();

        // determine how far back in time to parse the log files
        // NOTE: by default, only parse the log for the current day
        //       otherwise parse up to the log days control set by user
        $stop = 0;
        if ($this->timePeriod == countryAuthStatsModule::PERIOD_ALL_TIME) {
            $stop = (int)getControl("logdays", 7, CONTROLDIR);
        }

        return $authLogSearch->getTopAuthStatsByCountryRange(0, $stop, $this->topNum);
    }

    //! generates the label for the authentication country stats tables
    /*!
     * @param   string          country code index
     *
     * @returns html            string
     *
     * NOTE: overloaded to handle countries with names too long
     */
    protected function generateLabelForStatsTable($countryCode) {
        $countryName = Country::getCountryNameFromCode($countryCode);

        return "<div class='country-name' title='$countryName'>"
             . $countryName
             . "</div>\n"
             . "<div class='country-code' title='$countryName'>"
             . "($countryCode)"
             . "</div>\n";
    }

    //! validates the top number
    /*!
     * @returns boolean         whether the number of selections was valid
     */
    protected function validateTopNum($top_num) {
        return isset($top_num)
            && WV::isIntPositiveOrZero($top_num)
            && in_array($top_num, $this->allowedTopNum);
    }

    //! validates the time period
    /*!
     * @returns boolean         whether the time period was valid
     */
    protected function validateTimePeriod($time_period) {
        return isset($time_period)
            && WV::isIntPositiveOrZero($time_period)
            && in_array($time_period, $this->allowedTimePeriod);
    }

    //! set the number of selections
    /*!
     * @param   int             the number of selections
     *
     * @returns boolean         whether the number of selections was set
     */
    public function setTopNum($top_num) {
        if (!isset($top_num) || !$this->validateTopNum($top_num)) {
            return FALSE;
        }
        $this->topNum = $top_num;
        return TRUE;
    }

    //! set the time period filter value
    /*!
     * @param   int             the time period filter value
     *
     * @returns boolean         whether the time period was set
     */
    public function setTimePeriod($time_period) {
        if (!isset($time_period) || !$this->validateTimePeriod($time_period)) {
            return FALSE;
        }
        $this->timePeriod = $time_period;
        return TRUE;
    }

    //! Returns the number of selections
    /*!
     * @returns int              the number of selections
     */
    public function getTopNum() {
        return $this->topNum;
    }

    //! Returns the time period
    /*!
     * @returns int             the time period
     */
    public function getTimePeriod() {
        return $this->timePeriod;
    }
}
