<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2014-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Robert Bisewski <robertb@linuxmagic.com>
 *
 * $Id: logSnipModule.php 9538 2019-03-14 23:30:28Z deiony $
 */

require_once('lib.logsearch.php');

//! defines the log limit
define('LOG_SNIP_LIMIT', 15);

class logSnipModule extends Module {

    protected $log_search_handle;

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
        $this->log_search_handle = new LogSearch();
    }

    //! method to process any actions and react accordingly
    public function actionProcessor($action, &$errors, &$notices) {
        // the log snip module does not process any actions
    }

    //! method to render the log snip module
    public function renderModule() {
        // initialize all the variables needed for log search
        $base_file = LOGDIR . "mslog";
        $max_days = getControl("logdays", 7) + 1;
        $data = array();
        $morethanrequested = FALSE;

        // parse the log files for up to the 15 newest log entries
        if (!$this->log_search_handle->parseLogFiles($base_file, LOG_SNIP_LIMIT, $max_days, $data, $morethanrequested)) {
            return;
        }

        // render the log snip module based on the template
        renderTemplate('logSnip', array(
            'label' => intl('logsniplabel'),
            'header' => $this->build_log_snip_header(),
            'table' => $this->build_log_snip_body($data),
        ));
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering
    public function getCSS() {
        $css = array('log-search.css', 'log-snip.css');
        return $css;
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering
    /*!
     * @returns array   an array of the files to include
     */
    public function getJS() {
        $js = array(
            'help-popup.js',
            'magicspam.js',
            'magicspam-menu.js',
            'limit-integer.js',
            'check-enabled.js',
            'ajax.js',
            'form-submit.js',
            'log-search.js'
        );
        return $js;
    }

    //! build the html for the log snip header
    /*!
     * @returns string  the html for the log snip header
     */
    protected function build_log_snip_header() {
        $snip_header = new magicspamDataTable;
        $snip_header->setHeader(array(
                    intl("timestamp"),
                    intl("type"),
                    intl("ip"),
                    intl("from"),
                    intl("rcpt")
        ));
        $snip_header->setClass("headerRow", array(
                    "header_timestamp",
                    "header_type",
                    "header_ip",
                    "header_from",
                    "header_rcpt"
        ));
        return $snip_header->getRenderString();
    }

    //! build the html for the log snip body
    /*!
     * @param   array   the log search results
     *
     * @returns string  the html for the log snip table
     */
    protected function build_log_snip_body($data) {
        if (empty($data)) {
            // default to a warning message in table
            $log_table = '<div id="logNoResults">'
                        . '    <div class="warnings logSnipWarning">'
                        . intl('logResultsNone')
                        . '    </div>'
                        . '</div>';
        } else {
            // process the log search results into a data table
            $snip_body = $this->process_log_search_results($data);
            // wrap the data table html with a div
            $log_table = '<div id="logSnipLogsWrapper">'
                        . $snip_body->getRenderString()
                        . '</div>';
        }

        return $log_table;
    }

    //! process log search results into a data table
    /*!
     * @param   array   the log search results
     *
     * @returns object  the data table of processed results
     */
    protected function process_log_search_results($data) {
        $snip_body = new magicspamDataTable;
        $snip_body->setClass( "row", array(
                    "log_timestamp log_cell",
                    "log_type log_cell",
                    "log_ip log_cell",
                    "log_from log_cell",
                    "log_rcpt log_cell"
        ));
        // then we generate the rows
        foreach ($data as $row) {
            // initialize the processed row
            $prow = array();

            // loop through each data item in the row
            foreach ($row as $index => $data) {
                // we will not be using the data from these indexes
                if ($index === INDEX_RULE || $index === INDEX_MUA ||
                    $index === INDEX_HOSTNAME || $index === INDEX_COUNTRY ||
                    $index === INDEX_HELO) {
                    continue;
                }

                // reset the class
                $class = "";

                // process the data appropriately based on the index
                switch ($index) {
                case INDEX_DATETIME:
                    // format the datetime into a friendlier form
                    $unixTime = strtotime($data);
                    if ($unixTime !== FALSE) {
                        $data = date("D M d @ g:ia", $unixTime);
                    }
                    break;
                case INDEX_TYPE:
                    // generate the class based on the type
                    $class = "is" . ucfirst(strtolower($data));
                    // append the rule name if any
                    if ($row[INDEX_RULE] !== '') {
                        $data = $data . "[" . $row[INDEX_RULE] . "]";
                    }
                    break;
                case INDEX_IP:
                case INDEX_SENDER:
                case INDEX_RECIP:
                default:
                    // no processing required for these data
                    break;
                }

                // encode the data
                $encoded_data = htmlspecialchars($data);

                // build the html
                $html = "<a title=\"$encoded_data\" class=\"$class\">"
                       . $encoded_data
                       . "</a>";

                // append the html to the processed row
                $prow[] = $html;
            }

            // add the processed row to the table
            $snip_body->addRow($prow);
        }

        return $snip_body;
    }
}
