<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Product line and is
 * Copyright (c) LinuxMagic Inc. 2014-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Kyle Macdonald <kylem@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: magicspamAlertModule.php 10935 2020-01-09 18:05:08Z deiony $
 */
require_once('lib.modulecontainer.php');

class magicspamAlertModule extends Module {

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     *
     * @constructor
     */
    public function __construct($l) {
        parent::__construct($l, TRUE);
        $this->submodules = array(
            'system_alert' => 'systemAlert',
            'news'         => 'news',
        );

        // if this suite has the package updater built in then add it to the stack.
        if (defined('PKG_UPDATER') && is_executable(PKG_UPDATER)) {
            $this->submodules['package_ugprade'] = 'packageUpgrade';
        }

        // TODO: does this belong in the parent constructor? seems quite a
        //   common thing we'd want...
        global $modules;
        foreach ($this->submodules as $m) {
            if (!array_key_exists($m, $modules)) {
                loadModule($m);
            }
        }
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     *
     * @return void
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $modules;
        $subModule = isset($_POST['submodule']) ? $_POST['submodule'] : NULL;
        if ($subModule && isset($modules[$subModule]) 
           && in_array($subModule, $this->submodules)) {
            $modules[$subModule]->actionProcessor($action, $errors, $notices);
        }
    }

    //! method to render the full page module.
    /*!
     * @return void
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     *
     * SIDE EFFECT: Generates HTML to stdout
     */
    public function renderModule() {
        global $module;
        global $modules;
        $firstExpanded = FALSE;

        renderMenu();

        $submodule = NULL;
        if (array_key_exists('submodule', $_POST)
            && strlen($_POST['submodule']) > 0) {
            $submodule = $_POST['submodule'];
        }

        // Prints out the Master blurb for the page
        echo "<div class='masterBlurb'>";
        echo "<h2>" . intl('msa_header') . "</h2>";
        echo "</div>";

        $html = '';

        // print out the individual submodules
        foreach ($this->submodules as $idx => $m) {
            if (!isset($modules[$m])) {
                continue;
            }

            // Validation for the SESSION variable that holds our collapse state
            if (isset($_SESSION[$m . "__collapse"]) && $_SESSION[$m . "__collapse"] != '') {
                // Do nothing as the value is already set
            } else {
                // Else define it as open by default 
                $_SESSION[$m . "__collapse"] = 1;
            }

            // Renders the submodule
            $modules[$m]->renderModule();
        }
    }

    //! count how many unread feed items there are (all feeds right now).
    /*!
     * @return int The count of unread feed items (all feeds included).
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * we total up the number of unread items from each feed we have contained.
     */
    public function countUnread() {
        global $modules;
        $count = 0;
        foreach ($this->submodules as $module) {
            if (!array_key_exists($module, $modules)) {
                // while we should already have loaded by now, it won't hurt.
                loadModule($module);
            }
            if (!is_a($modules[$module], 'NotificationModule')) {
                continue;
            }
            $count += $modules[$module]->countUnread();
        }
        return $count;
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     */
    public function getCSS() {
        $css = array('system-alerts.css');
        return array_merge($css, $this->submoduleCSS());
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     */
    public function getJS() {
        $js = array('form-submit.js', 'limit-integer.js','alerts.js');
        return array_merge($js, $this->submoduleJS());
    }

    //! method to return the title of the page to renderHeader
    /*!
     * @return  string  the value of the <title> of this page
     *
     * @author  Kyle Macdonald <kylem@linuxmagic.com>
     */
    public function getPageTitle() {
        return intl('msa_header');
    }
}
