<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2019-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Deion Yu <deiony@linuxmagic.com>
 *
 * $Id: migrationModule.php 10932 2020-01-09 02:26:26Z deiony $
 */

require_once('lib.module.php');

require_once('wizard/lib.wizard_validator.php');
require_once('wizard/lib.wizard_debug.php');
require_once('wizard/lib.http_var.php');

class migrationModule extends Module {

    //! the name of the exported migration file
    const MIGRATION_FILE = 'magicspam_migration.json';

    //! the list of migratable modules
    protected $migratable_modules;

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);

        // build the list of migratable modules
        $this->migratable_modules = array(
            "settings",
        );
        if (isProfessional()) {
            $this->migratable_modules[] = "securitySettings";
        }
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $errors, $notices;
        if ($action == "export") {
            // retrieve the ie input if any otherwise default to false
            $isIE = HttpVar::getPost("ie", FALSE);

            // retrieve the configurations of the migratable modules
            $configurations = $this->getMigrationData();
            if (!$configurations) {
                $errors[] = intl('migration_export_data_failure');
                return FALSE;
            }

            // export the migration file with the configurations
            $this->exportMigrationFile($configurations, $isIE);
        } else if ($action == "import") {
            // ensure the migration file was uploaded
            if (!isset($_FILES['migration_file'])) {
                $errors[] = intl('migration_file_invalid');
                return;
            }

            // import the configurations from the migration file
            if ($this->importMigrationFile($_FILES['migration_file'])) {
                $notices[] = intl('migration_import_file_success');
            }
        }
    }

    //! method to render the full page module.
    public function renderModule() {
        global $module;
        renderTemplate('migration', array(
            'module' => $module,
            'migratable_list' => $this->buildMigratableListHtml(),
        ));
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getCSS() {
        return array('migration.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getJS() {
        return array('migration.js', 'css_browser_selector.js', 'ajax.js');
    }

    //! builds the html for the list of migratable configurations
    /*!
     * @returns string  the migratable configurations list html
     */
    protected function buildMigratableListHtml() {
        // initialize html variable with default value
        $html = "";

        // list of migratable configurations
        $configurations = array(
            "system_options",
            "exemptions",
            "server_policies",
            "bms_lists",
            "rbl_lists",
        );

        //! append professional configurations if necessary
        if (isProfessional()) {
            $configurations[] = "security_settings";
        }

        // build the html which consists of list items
        foreach($configurations as $configuration) {
            $html .= "<li>" . intl("migration_export_list_$configuration") . "</li>";
        }
        return $html;
    }

    //! exports the migration file from the configurations
    /*!
     * @param   array  the configuration for exporting the migration file
     * @param   bool   whether the browser is Internet Explorer
     */
    protected function exportMigrationFile($configurations, $isIE) {
        // turn off output compression
        ini_set('zlib.output_compression', 'Off');

        // determine the octet-stream MIME type based on IE browser
        $octetstream = ($isIE) ? "octetstream" : "octet-stream";

        // full list of potential HTTP headers for CSV download
        $csv_headers = array(
            "HTTP/1.1 200 OK",
            "Pragma: public",
            "Last-Modified: " . gmdate('D, d M Y H:i:s') . " GMT",
            "Cache-Control: no-store, no-cache, must-revalidate",
            "Cache-Control: pre-check=0, post-check=0, max-age=0",
            "Content-Transfer-Encoding: none",
            "Content-Type: application/$octetstream; name=\"" . self::MIGRATION_FILE . "\"",
            "Content-Type: application/force-download",
            "Content-Disposition: attachment; filename=\"" . self::MIGRATION_FILE . "\"",
        );

        // remove the Cache-Control headers when not IE browser
        if (!$isIE) {
            array_splice($csv_headers, 3, 2);
        }

        // remove the status code header when not building the CSV headers string
        if (!defined("LS_CSV_HEADERS_STRING")) {
            array_splice($csv_headers, 0, 1);
        }

        // send the HTTP headers or build the CSV headers string
        $csv_headers_string = "";
        foreach ($csv_headers as $csv_header) {
            if (defined("LS_CSV_HEADERS_STRING")) {
                $csv_headers_string .= "$csv_header\n";
            } else {
                header($csv_header);
            }
        }

        // open the output buffer mechanism for writing
        $handle = fopen('php://output', 'w');

        // write the CSV headers string to the output buffer if necessary
        if (defined("LS_CSV_HEADERS_STRING")) {
            fwrite($handle, "$csv_headers_string\n");
        }

        // write the migration data to the output buffer in JSON format
        fwrite($handle, json_encode($configurations));
        fclose($handle);
    }

    //! imports the configurations from the migration file
    /*!
     * @param   array  the uploaded migration file
     *
     * @returns bool   whether the migration file was imported successfully
     */
    protected function importMigrationFile($file) {
        global $errors;

        // validate the migration file
        if (!$this->validateMigrationFile($file)) {
            return FALSE;
        }

        // retrieve the contents which is expected to be in JSON format
        $content = file_get_contents($file['tmp_name']);
        if ($content === FALSE) {
            $errors[] = intl('migration_file_read_failure');
            return FALSE;
        }

        // retrieve the configurations by decoding the JSON
        $configurations = json_decode($content, TRUE);
        if (is_null($configurations)) {
            $errors[] = intl('migration_file_decode_failure');
            return FALSE;
        }

        // import the migration configurations
        return $this->loadMigrationData($configurations);
    }

    //! validates the migration file
    /*!
     * @param   array  the migration file to validate
     *
     * @returns bool   whether the migration file is valid
     */
    protected function validateMigrationFile($file) {
        global $errors;
        if (!isset($file) || !WV::isArrayNotEmpty($file)) {
            $errors[] = intl('migration_file_invalid');
            return FALSE;
        }
        if ($file['error'] === UPLOAD_ERR_NO_FILE) {
            $errors[] = intl('migration_file_no_upload');
            return FALSE;
        }
        if ($file['error'] !== UPLOAD_ERR_OK) {
            $errors[] = intl('migration_file_upload_failure');
            return FALSE;
        }
        if ($file['type'] !== 'application/json'
            && $file['type'] !== 'application/octetstream'
            && $file['type'] !== 'application/octet-stream') {
            $errors[] = intl('migration_file_unexpected_type');
            return FALSE;
        }
        if ($file['size'] == 0) {
            $errors[] = intl('migration_file_empty');
            return FALSE;
        }
        if (!file_exists($file['tmp_name'])) {
            $errors[] = intl('migration_file_missing');
            return FALSE;
        }
        return TRUE;
    }

    //! retrieve the migratable configurations of the migratable modules
    /*!
     * @returns array the list of migratable configurations
     *
     * NOTE: similar to the implementation in Module except the function
     *       operates on $migratable_modules rather than $submodules
     */
    public function getMigrationData() {
        // the list of all loaded Module objects
        global $modules;

        // the list of migratable configurations
        $configurations = array();

        // go through each submodule within the module
        foreach ($this->migratable_modules as $module) {
            // skip submodules that are not loaded or migratable
            if (!array_key_exists($module, $modules)
                || !$modules[$module]->isMigratable()) {
                continue;
            }

            // retrieve the configuration of the migratable module
            $configuration = $modules[$module]->getMigrationData();
            if ($configuration === FALSE) {
                // XXX: maybe error handling, halt export process?
                continue;
            }

            // append the configuration to the list of configurations
            $configurations[$module] = $configuration;
        }

        // return the list of migratable configurations or FALSE if none
        return $configurations;
    }

    //! override function to prevent infinite loop
    public function renderOutput($modules) {
        // XXX: legacy code workaround
    }
}
