<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Product line and is
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: protectionKeyModule.php 10395 2019-09-16 18:11:15Z deiony $
 */
require_once('lib.listmanager.php');
require_once('lib.modulecontainer.php');
require_once('lib.register.php');

if (isProfessional()) {
    require_once("lib.spamassassinmanagerutility.php");
}

class protectionKeyModule extends Module {
    //! internal variable to indicate if we are in edit mode for the license
    private $editMode = FALSE;

    //! internal variable to indicate if we are in edit mode for an addon license.
    private $addonEdit = FALSE;

    //! internal variable to indicate if first installation
    private $firstInstallation = FALSE;

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
        $this->submodules = array('serviceStatus' => 'serviceStatus');
        foreach ($this->submodules as $m) {
            loadModule($m);
        }
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $admin_email, $license, $pages, $modules;

        $subModule = isset($_POST['subsubmodule']) ? $_POST['subsubmodule'] : NULL;
        if (  $subModule && isset($modules[$subModule])
           && in_array($subModule, $this->submodules)) {
            return $modules[$subModule]->actionProcessor($action, $errors, $notices);
        }

        // ok.. we have several different types of processing available
        $status = getControl("enabled", 0);
        // toggle the enabled/disabled status
        if ($action === 'toggleStatus') {
            if(!isFreemium() && !$license->isValid() && $status == "0") {
                $errors[] = intl('upgrade_required');
            // before enabling we want to ensure that the admin email is set.
            } else if ($status == "0" && $admin_email === '') {
                $errors[] = intl('ae_please_complete');
            // otherwise, we use our register object to toggle
            } else {
                $register = Register::getInstance();
                if (!$register->toggle()) {
                    $errors[] = $register->error();
                } else {
                    $this->_runUpdater();
                }

                // XXX: only the PRO suites need to worry about SpamAssassin
                //      pull this block of code out into PRO specific directory
                if (defined("SA_MANAGER")) {
                    $sa_status = getControl("sa_status", 0); 
                    // nothing to do when SA setting is already OFF 
                    if ($sa_status !== "0") { 
                        $buffer = ""; 
                        $saManagerCmd = SpamAssassinManagerUtility::getInstance(); 
                        $saManagerCmd->callSpamAssassinManagerUtility($status ? "d" : "e", $buffer); 
                    } 
                }
            }
            return;
        }
        // process request to edit the license
        if ($action === 'editLicense') {
            $this->editMode = TRUE;
        }
        if ($action === 'remove_license') {
            // safety - this is only supposed to exist in freemium
            if (!defined('ISFREEMIUM')) {
                return;
            }
            $rv = $license->removeLicense();
            if ($rv === -1) {
                $errors[] = $license->error();
            } else {
                $notices[] = intl('license_removed');
            }
        }
        // process request to edit an addon license.
        if ($action === 'editAddonLicense') {
            $this->addonEdit = TRUE;
        }
        
        // process request to save an addon license
        if ($action == 'saveAddon') {
            $lcode = isset($_POST['licenseKey']) ? $_POST['licenseKey'] : NULL;
            // if this is 'freemium' then don't bother.
            if (isFreemium()) {
                return;
            }

            if ($lcode) {
                $aLicense = new LicenseManager();
                $rv = $aLicense->updateAddon($lcode);

                if ($rv === -1) {
                    $errors[] = $aLicense->error();
                    $this->addonEdit = TRUE;
                } else {
                    // before we go hog wild and assume everything updated correctly
                    if (!$aLicense->isValid()) {
                        if (!$aLicense->isExpired()) {
                            $errors[] = intl('empty_license');
                        } else {
                            $errors[] = intl('expired_license');
                        }
                    } else {
                        $notices[] = intl('license_saved');
                    }
                }
            }
            return;
        }

        // process request to save the license
        if ($action === 'saveLicense') {
            $lcode = isset($_POST['licenseKey']) ? $_POST['licenseKey'] : NULL;
            // add a hook to save the admin email too?

            // we want to know if we're in freemium mode prior to updating
            // the license because we can take separate steps if we are
            // transitioning from freemium to licensed mode.
            $isFreemium = isFreemium();

            // XXX: should an empty license be indicator to reset to freemium?
            if ($lcode) {
                // request an update
                $rv = $license->update($lcode);

                if ($rv === -1) {
                    $errors[] = $license->error();
                    $this->editMode = TRUE;
                } else {
                    // before we go hog wild and assume everything updated correctly
                    if (!$license->isValid()) {
                        if (!$license->isExpired()) {
                            $errors[] = intl('empty_license');
                        }
                        // expiry warnings are handled at top level index.php
                    } else {
                        $notices[] = intl('license_saved');
                        if ($admin_email === '') {
                            $notices[] = intl('ae_please_complete');
                        } else {
                            // auto enable magicspam when all conditions are met
                            if ($status == "0") {
                                $register = Register::getInstance();
                                if (!$register->toggle()) {
                                    $errors[] = $register->error();
                                } else {
                                    // only display the first time popup message when auto enabled
                                    $this->firstInstallation = TRUE;
                                }
                            }

                            // always run the updater when a VALID license is saved
                            $this->_licenseUpdatedActions($isFreemium);
                        }
                    }
                }
            }
        }
    }

    //! method to render the full page module.
    /*!
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function renderModule() {
        global $license, $module, $modules;

        // prepare some labels  and values
        $version = MAGICSPAM_VERSION;
        $licensekey = '';
        $expires = 0;
        $status = getControl("enabled", 0);
        $licenseAction = 'editLicense';
        $aLicense = new LicenseManager();
        $aLicense->clear();
        $aLicense->loadAddon();

        $service_status = 1;
        if (isset($modules['serviceStatus'])) {
            $service_status = $modules['serviceStatus']->isRunning();
        }
        $subsubmodule = '';

        // the license manager has built in encoding specific to
        // FREEMIUM detection so...
        $licensekey = $license->getCode();
        if ($aLicense->isValid()) {
            $licensekey .= " <a class=\"expiryLink\""
                         . " onclick=\"MS.showHelpPopup(this, 'X', 'addon_descr');\""
                         . " title=\"" . intl('addon_blurb') . ": " . $aLicense->getCode() . "\">*</a>";
        }
        $expires = $license->getExpiry();
        $licenseType = intl($license->getSKUType());

        if (!$license->isNew() && strlen($licensekey) > 0 && !$license->isExpired()) {
                $this->_i18n_js[] = 'expiry_blurb';
                $expires = "<a class=\"expiryLink\" "
                         . "onclick=\"MS.showHelpPopup(this, 'X', 'expiry_blurb');\" "
                         . "title=\""
                         . intl('expiry_blurb')
                         . "\">"
                         . htmlspecialchars($license->getExpiry())
                         . "</a>";
        }

        if ($license->isExpired()) {
            $expires = intl('expired');
            $this->editMode = TRUE;
        } else if ($license->isNew() && strlen($licensekey) > 0) {
            $expires = intl('never');
            $version .= ' Free Edition';
        } else {
            // this mode indicates this is a license required version
            // and a license is not set so
            if (strlen($licensekey) === 0) {
                $this->editMode = TRUE;
            }
        }

        // now we prep our tables
        $licenseTable = new magicspamDataTable();
        $statusRow = '';
        $toggleButton = '';
        $statusButton = '';
        $formAction = 'toggleStatus';

        if (!$service_status) {
            $statusBlurbRow = intl('serviceBlurb');
            $statusButton = "<button class=\"msbutton\" type=\"submit\"><span>"
                          . intl('start_service')
                          . "</span></button>\n";
            $subsubmodule = 'serviceStatus';
            $formAction = 'start_service';
        } else {
            $statusBlurbRow = intl('statusBlurb');
        }

        if ($status === "1" && $service_status) {
            $statusRow = '<span>'. intl('statusLabel') . '</span>'
                       . '<span class="enabled">'
                       . intl('enabled')
                       . '&nbsp;<img src="' . IMAGEPATH . 'icon-success.png"/></span>';
        } else {
            $statusRow = '<span>'. intl('statusLabel') .'</span>'
                       . '<span class="disabled">'
                       . intl('disabled')
                       . '&nbsp;<img src="' . IMAGEPATH . 'icon-warning.png"></span>';
        }
        
        if ($status === "1") {
            $toggleButton = "<button class=\"msbutton\" type=\"submit\"";
            if (!$service_status) {
                $toggleButton .= " disabled=\"disabled\"";
            }

            $toggleButton .= "><span>"
                           . intl('disable')
                           . "</span></button>\n";
        } else {
            $toggleButton = "<button class=\"msbutton\" type=\"submit\"";
            if (!$service_status) {
                $toggleButton .= " disabled=\"disabled\"";
            }
            $toggleButton .= "><span>"
                           . intl('enable')
                           . "</span></button>\n";
        }

        $buttons = '';
        $licenseForm = '';
        if ($this->editMode) {
            $licenseValue = '';
            if (!isFreemium()) {
                $licenseValue = $license->getCode();
            }
            $licenseForm = "<span class=\"msStatusHead\">"
                         . intl('license_key_title')
                         . "</span>\n"
                         . "<span class=\"msStatusBlurb\">\n"
                         . intl('please_enter_key') . "\n"
                         . "<input type=\"text\" name=\"licenseKey\" "
                         . "id=\"licenseKeyInput\" value=\""
                         . $licenseValue
                         . "\" maxlength=\"38\" onkeyup=\"formatLicense(this);\"/>\n"
                         . "</span>";

            $buttons = '<button type="button" class="msbutton" '
                     . 'onclick="document.forms[\'licenseForm\'].ms_action.value=\'\';'
                     . 'document.forms[\'licenseForm\'].submit();"/>'
                     . '<span>' . intl('cancel') . '</span></button>'
                     . '<button type="reset" class="msbutton" '
                     . 'onclick="clearLicenseForm();return false;">'
                     . '<span>' . intl('clear') . '</span></button>'
                     . '<button type="submit" class="msbutton">'
                     . '<span>' . intl('save') . '</span></button>';

            $licenseAction = 'saveLicense';
        } else if ($this->addonEdit) {
            $licenseValue = '';
            if (!isFreemium()) {
                $aLicense = new LicenseManager();
                $aLicense->clear();
                if ($aLicense->loadAddon() != -1) {
                    $licenseValue = $aLicense->getCode();
                }
            }
            $licenseForm = "<span class=\"msStatusHead\">"
                         . intl('addon_key_title')
                         . "</span>\n"
                         . "<span class=\"msStatusBlurb\">\n"
                         . intl('please_enter_addon') . "\n"
                         . "<input type=\"text\" name=\"licenseKey\" "
                         . "id=\"licenseKeyInput\" value=\""
                         . $licenseValue
                         . "\" maxlength=\"38\" onkeyup=\"formatLicense(this);\"/>\n"
                         . "</span>";
            $buttons = '<button type="button" class="msbutton" '
                     . 'onclick="document.forms[\'licenseForm\'].ms_action.value=\'\';'
                     . 'document.forms[\'licenseForm\'].submit();"/>'
                     . '<span>' . intl('cancel') . '</span></button>'
                     . '<button type="reset" class="msbutton" '
                     . 'onclick="clearLicenseForm();return false;">'
                     . '<span>' . intl('clear') . '</span></button>'
                     . '<button type="submit" class="msbutton">'
                     . '<span>' . intl('save') . '</span></button>';
            $licenseAction = 'saveAddon';
        } else {
            $licenseTable->addRow(array(intl('license'), $licensekey));
            $licenseTable->addRow(array(intl('version'), $version));
            $licenseTable->addRow(array(intl('expires'), $expires));
            if (!isFreemium()) {
                $licenseTable->addRow(array(intl('licenseType'), $licenseType));
            } else {
                $licenseTable->addRow(array('&nbsp;', '&nbsp;'));
            }
            $licenseForm = $licenseTable->getRenderString();
            $buttons = "";
            // IF this is a freemium suite....
            if (defined('ISFREEMIUM')) {
                // we want to know if the license is from us or from plesk
                $requester = $license->getRequester();
                // so if it has a registered license that is not from plesk license manager
                if (!isFreemium() && $requester != 'PARALLELS-KA') {
                    $buttons = '<button class="msbutton" type="button" onclick="javascript:removeLicenseKey(this.form);"><span>' . intl('remove_license_key') . '</span></button>';
                }
            }
            $buttons .= '<button class="msbutton" type="submit"><span>' . intl('enter_license_key') . '</span></button>';
            // determine if we should render the addon code
            $render_addon = getControl('addon_enabled', 0);
            if ($render_addon === "1" && !isFreemium()) {
                $buttons .= '<button class="msbutton" type="submit" onclick="this.form.ms_action.value=\'editAddonLicense\';this.form.submit();"><span>' . intl('enter_wl_license_key') . '</span></button>';
            }
        }

        // and the 'help' text
        $this->_i18n_js[] = 'protection_help';
        $this->_i18n_js[] = 'addon_descr';
        $this->_i18n_js[] = 'confirm_license_remove';

        $renderArgs = array(
                         'module' => $module,
                         'header' => intl('protectionKey'),
                         'licensetable' => $licenseForm,
                         'statusHead' => $statusRow,
                         'statusBlurb' => $statusBlurbRow,
                         'toggleButton' => $toggleButton,
                         'statusButton' => $statusButton,
                         'licenseButtons' => $buttons,
                         'help' => intl('help'),
                         // help popup i18n key.
                         'helpI18n' => 'protection_help',
                         'licenseAction' => $licenseAction,
                         'subsubmodule' => $subsubmodule,
                         'formAction' => $formAction,
        );
        
        renderTemplate('protectionKey', $renderArgs);

        // render the first installation popup if necessary
        if ($this->firstInstallation) {
            $this->__first_time_install_popup();
        }
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getCSS() {
        return array('protectionKey.css', 'splash.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getJS() {
        // XXX: merge in include for serviceStatus
        return array('license-key.js', 'form-submit.js');
    }

    //! run the updater.
    /*!
     * @return bool Whether successful
     *
     * @author William Storey <william@linuxmagic.com>
     */
    private function _runUpdater() {
        // get the current status
        $status = getControl("enabled", 0);
        // if the status is 1 then that means enabled
        if ($status == 1) {
            // and then we can run updates..but...
            // we run things different if it is a WINDOWS installation.
            if (defined('ISWINDOWS') && ISWINDOWS == 1) {
                pclose(popen("start /b " . UPDATER, "r"));
            } else {
            // vs. a Linux installation!
                exec(UPDATER . " >/dev/null 2>&1 &");
            }
        }
    }

    //! disable RATS RBL lists.
    /*!
     * @return bool Whether successful
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * find all enabled RBLs, and disable them if they are RATS RBLs.
     */
    private function _disableRATSRBLs() {
        // TODO: the below logic might be better to abstract in some kind of RBL
        //   management class...
        $listMgr = ListManager::getInstance();
        $res = $listMgr->load('-E', DATA_PATH, $lists);
        if ($res !== TRUE) {
            // TODO: log?
            return FALSE;
        }
        // see rbls in rblListsModule.
        $listsToRemove = array(
            'noptr.spamrats.com',
            'dyna.spamrats.com',
            'spam.spamrats.com',
            'psbl.surriel.com',
        );
        foreach ($lists as $list) {
            if (!in_array($list, $listsToRemove, TRUE)) {
                continue;
            }
            $args = array('-E', '-d');
            $args[] = $list;
            $res = $listMgr->delete($args);
            if ($res !== TRUE) {
                // TODO: log?
                // I'm not returning here... though we could. try our best.
                continue;
            }
        }
        return TRUE;
    }

    //! remove PSBL from the available RBLs
    private function _removePSBL() {
        $listMgr = ListManager::getInstance();
        $res = $listMgr->load('-R', DATA_PATH, $lists);
        if ($res !== TRUE) {
            // TODO: log?
            return FALSE;
        }
        $listsToRemove = array(
            'psbl.surriel.com PSBL Passive Spam Block List (http://psbl.org)',
        );
        foreach ($lists as $list) {
            if (!in_array($list, $listsToRemove, TRUE)) {
                continue;
            }
            $args = array('-R', '-d', $list);
            $res = $listMgr->delete($args);
            if ($res !== TRUE) {
                // TODO: log?
                continue;
            }
        }
        return TRUE;
    }

    //! add CBL list to available RBLs.
    /*!
     * @return bool Whether successful
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * the CBL list is one that is available by default in the regular
     * non-freemium MagicSpam. so when we switch to license, make it available
     * too.
     */
    private function _addCBLToAvailableRBLs() {
        // first find if it's already present in the list.
        // TODO: the below logic might be better to abstract in some kind of RBL
        //   management class...
        $listMgr = ListManager::getInstance();
        $res = $listMgr->load('-E', DATA_PATH, $lists);
        if ($res !== TRUE) {
            // TODO: log?
            return FALSE;
        }
        $enabledHosts = array();
        foreach ($lists as $list) {
            // form is: <host> <description>
            $parts = explode(' ', $list, 2);
            if (count($parts) !== 2) {
                // corrupt entry somehow?
                // TODO: we could abort or log...
                continue;
            }
            $enabledHosts[] = $parts[0];
        }
        // TODO: this host and description are also in certain package control
        //   files such as in rbl.available in the magicspam-plesk suite...
        //   we might be able to consolidate that somehow...
        $cblHost = 'cbl.abuseat.org';
        $cblDesc = 'The CBL Composite Blocking List (http://cbl.abuseat.org)';
        if (in_array($cblHost, $enabledHosts, TRUE)) {
            // it's already there!
            return TRUE;
        }
        // add it.
        $args = array('-R');
        if (defined('PERUSER_ON')) {
            $args[] = '-h';
            $args[] = $listMgr->getFolder();
        }
        $args[] = '-a';
        $args[] = $cblHost . ' ' . $cblDesc;
        $res = $listMgr->add($args);
        if ($res !== TRUE) {
            // TODO: log?
            return FALSE;
        }
        return TRUE;
    }

    //! enable rules during the upgrade from freemium to licensed.
    /*!
     * @return bool Whether successful
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * @pre We are upgrading from freemium mode and have just entered a valid
     *   license.
     *
     * at this time we only enable the rule check_reverse_dns_list. why?
     * because in the licensed package it is recommended and enabled by default.
     * in freemium it is not available.
     *
     * the only other rule that becomes available right now is KSF. while it
     * is recommended we do not enable it by default anywhere, so we do not
     * enable it here either. why? because it needs some manual confirmation
     * that you actually want it enabled given the risks of there being a
     * something in front of the mail server.
     */
    private function _enableRulesFreemiumToLicensed() {
        // load current enabled rules.
        $rulesRaw = getControl(RULES_ENABLED, '');
        if (strlen($rulesRaw) > 0) {
            $rules = explode("\n", $rulesRaw);
        } else {
            $rules = array();
        }

        // we want to enable both regex rules
        $ruleToEnables = array('check_dynamic_reverse_dns', 
                               'check_reverse_dns_list');

        // if enabled, then there's nothing to do.
        if (in_array($ruleToEnables[0], $rules, TRUE) 
        && in_array($ruleToEnables[1], $rules, TRUE)) {
            return TRUE;
        }

        // we have to enable them
        $rules[] = $ruleToEnables[0];
        $rules[] = $ruleToEnables[1];

        // write out the control file with our rules.
        $rulesRawUpdate = implode("\n", $rules);
        $ret = setControl(RULES_ENABLED, $rulesRawUpdate);
        if ($ret !== TRUE) {
            // TODO: log?
            return FALSE;
        }
        return TRUE;
    }

    //! take actions upon entering a valid license.
    /*!
     * @param bool $isFreemium True if the license is being entered from
     *   freemium mode.
     *
     * @return bool Whether we succeeded in all we wanted or encountered
     *   an error.
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * @pre We just updated our license to a valid license.
     *
     * we start our update process.
     *
     * we also remove any RATS RBL lists from the RBL enabled list.
     * why? because they can use BMS!
     *
     * also add in CBL as an available list because it's usually there in the
     * licensed package.
     *
     * if we're upgrading from freemium we also enable certain spam rules.
     */
    private function _licenseUpdatedActions($isFreemium) {
        // parameter checks.
        if (!is_bool($isFreemium)) {
            // TODO: log?
            return FALSE;
        }
        $success = TRUE;
        // we want to start up the first set of updates in the background.
        $this->_runUpdater();
        // clear out any enabled RATS RBL lists. why? because we can use BMS.
        if (!$this->_disableRATSRBLs()) {
            // try our best to do everything, so don't return here.
            $success = FALSE;
        }
        if (!$this->_removePSBL()) {
            $success = FALSE;
        }
        // add in CBL to available list (because it's there in licensed package
        // usually).
        if (!$this->_addCBLToAvailableRBLs()) {
            // try our best to do everything, so don't return here.
            $success = FALSE;
        }
        // if we're upgrading from freemium then enable certain rules that are
        // usually enabled by default but were not available until now.
        if ($isFreemium && !$this->_enableRulesFreemiumToLicensed()) {
            // try our best to do everything, so don't return here.
            $success = FALSE;
        }
        return $success;
    }

    private function __first_time_install_popup() {
        // build header of first time install popup
        $html  = '<script type="text/javascript">';
        $html .= '  function closeSplash() {';
        $html .= '      var modal = document.getElementById("splash-screen");';
        $html .= '      modal.classList.add("fadeout");';
        $html .= '  }';
        $html .= '</script>';

        $html .= '<!-- Modal -->' . "\n";
        $html .= '<div class="modal fade" id="splash-screen" role="dialog">' . "\n";
        $html .= '    <div class="modal-dialog">' . "\n";
        $html .= '        <!-- Modal content-->' . "\n";
        $html .= '        <div class="modal-content">' . "\n";
        $html .= '            <div class="modal-header">' . "\n";
        $html .= '                <button type="button" class="close" onclick="closeSplash();" data-dismiss="modal">&times;</button>' . "\n";
        $html .= '                <h3 class="modal-title">' . intl('firstInstallTitle') . '</h3>' . "\n";
        $html .= '            </div>' . "\n";
        $html .= '            <div class="modal-body">' . "\n";

        // build content of first time install popup
        $html .= '                <h1>' . intl('firstInstallMainHeader') . '</h1>';
        $html .= '                <p>' . intl('firstInstallMainBlurb') . '</p>';
        $html .= '                <h1>' . intl('firstInstallTipsHeader') . ':</h1>';
        $html .= '                <ul>';
        $html .= '                    <li>' . intl('firstInstallTipsActivity') . '</li>';
        $html .= '                    <li>' . intl('firstInstallTipsSettings') . '</li>';
        $html .= '                    <li>' . intl('firstInstallTipsAdminEmail') . '</li>';
        $html .= '                </ul>';
        $html .= '                <h1>' . intl('firstInstallQuestionHeader') . '</h1>';
        $html .= '                <p>' . intl('firstInstallQuestionBlurb') . '</p>';


        // footer content of first time install popup
        $html .= '            </div>' . "\n";
        $html .= '        </div>' . "\n";
        $html .= '    </div>' . "\n";
        $html .= '</div>' . "\n";

        echo $html;
    }
}
