<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Product line and is
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *            Robert Bisewski <robertb@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: rblListsModule.php 10395 2019-09-16 18:11:15Z deiony $
 */
require_once('lib.module.php');
require_once('lib.datatable.php');
require_once('lib.listmanager.php');

class rblListsModule extends Module {
    //! internal array of all available RBL lists.
    private $lists, $lists_original_length;

    // TODO: document
    private $rbl_enabled;

    // TODO: document
    private $radio_name;

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
        // set module to be migratable
        $this->migratable = TRUE;
        global $errors;
        // set our static lists that are always included with MagicSpam free
        if (isFreemium()) {
            $this->lists = array(
                'Rats-NOPTR' => array(
                    'description' => 'Rats-NOPTR',
                    'rbl'         => 'noptr.spamrats.com',
                    'help'        => 'rblRatsNoptr',
                ),
                'Rats-Dyna' => array(
                    'description' => 'Rats-Dyna',
                    'rbl'         => 'dyna.spamrats.com',
                    'help'        => 'rblRatsDyna',
                ),
                'Rats-SPAM' => array(
                    'description' => 'Rats-SPAM',
                    'rbl'         => 'spam.spamrats.com',
                    'help'        => 'rblRatsSpam',
                ),
            );
            // TODO: why not count them?
            $this->lists_original_length = 3;
        } else {
            $this->lists = array();
            $this->lists_original_length = 0;
        }

        // Load up any RBL available (-R -l)
        $listMgr = ListManager::getInstance();
        $ret = $listMgr->load('-R', DATA_PATH, $data);
        if ($ret == 1) {
            foreach ($data as $row) {
                // split line in 2 strings: name and host
                $available = preg_split('/\s/', $row, 2);
                // sanity check in case of no description
                if (count($available) < 2 || !$available[1]) {
                    $available[1] = $available[0];
                }
                $this->lists[$available[1]] = array('description' => $available[1],
                                        'rbl'        => $available[0],);
            }
        } else {
            // TODO: what if error() is blank?
            $errors[] = htmlspecialchars($listMgr->error());
        }

        // Load up RBL enabled (-E -l)
        $ret = $listMgr->load('-E', DATA_PATH, $data);
        if ($ret == 1) {
            $this->rbl_enabled = $data;
        } else {
            // TODO: what if error() is blank?
            $errors[] = htmlspecialchars($listMgr->error());
        }
        $this->radio_name = "rbllist";
    }

    //! handle request action 'add_rbl_list'.
    /*!
     * @return Associative array with keys 'notices' - an array
     *   of notice messages, or 'errors' - an array of error
     *   messages.
     *
     * @author William Storey <william@linuxmagic.com>
     */
    private function _actionAddRBLList() {
        $errors = array();
        $notices = array();

        // Checks if the listName and listHost are valid
        if (!array_key_exists('listName', $_POST)
            || !is_string($_POST['listName'])) {
            $errors[] = intl("rblBlankListName");
            return array('errors' => $errors, 'notices' => $notices);
        }
        // remove excessive spaces on list name
        $listName = trim($_POST['listName']);
        if (strlen($listName) === 0) {
            $errors[] = intl("rblBlankListName");
            return array('errors' => $errors, 'notices' => $notices);
        }
        if (!array_key_exists('listHost', $_POST)
            || !is_string($_POST['listHost'])) {
            $errors[] = intl("rblBlankHostName");
            return array('errors' => $errors, 'notices' => $notices);
        }
        // remove excessive spaces on list host before validating it
        $listHost = strtolower(trim($_POST['listHost']));

        if (strlen($listHost) === 0) {
            $errors[] = intl("rblBlankHostName");
            return array('errors' => $errors, 'notices' => $notices);
        }

        // NOTE: this is a 'hacky' way of using the domain part validation for
        //   email addresses
        //   as FILTER_VALIDATE_URL is not in line with what we want.
        if ( filter_var("user@" . $listHost, FILTER_VALIDATE_EMAIL) === FALSE
          || filter_var("user@" . $listHost, FILTER_VALIDATE_EMAIL) === NULL ) {
            $errors[] = intl("rblInvalidHostName");
            return array('errors' => $errors, 'notices' => $notices);
        }

        // check if we already have a list with this name or host.
        $res = $this->existsRBL($listName, $listHost);
        if ($res !== FALSE) {
            $errors[] = $res;
            return array('errors' => $errors, 'notices' => $notices);
        }

        // Add the new RBL list to the list of arrays
        $newRBLList = array(
            'description' => $listName,
            'rbl'         => $listHost,
        );
        $this->lists[$listName] = $newRBLList;

        // now we need to generate an update of the 'availableList' array
        // in our internal format.
        $availableList = array();

        // we now add the appropriate information offset from our original
        // 'static' indexes.
        for ($index = $this->lists_original_length;
            $index < count($this->lists); $index++) {
            $availableList[$listName] = array(
                'description' => $this->lists[$listName]['description'],
                'rbl'         => $this->lists[$listName]['rbl'],
            );
        }

        // Use listManager to write the new list
        $listMgr = ListManager::getInstance();
        $args = array('-R');
        // prepare listHost and listName to be inserted as a single entry
        $newEntry = $listHost . " " . $listName;
        $args[] = '-a';
        $args[] = $newEntry;
        $args[] = '-h';
        $args[] = DATA_PATH;
        $retAdd = $listMgr->add($args);

        // write to the rbl.available file
        if ($retAdd !== TRUE) {
            $errors[] = intl("rblCannotWriteToFile");
            return array('errors' => $errors, 'notices' => $notices);
        }
        $notices[] = htmlspecialchars($newRBLList['description'])
            . intl("rblAddSuccess");
        return array('errors' => $errors, 'notices' => $notices);
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     *
     * @return void
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $recommended_rbl;
        $ret = TRUE;

        if (!isFreemium()) {
            $recommended_rbl = array();
        }

        if ($action === 'remember_expand_state') {

                $module = isset($_POST['changed_module']) ? $_POST['changed_module'] : NULL;
                $value  = isset($_POST['expanded']) ? $_POST['expanded'] : NULL;
                $var = $module . "__collapse";
                $_SESSION[$var] = $value;
       }

    // if we were asked to reset the RBL lists to default recommended...
        if ($action === 'set_rbl_list_to_defaults') {
                // generate a 'string' list of the recommended RBLs
                $default_enabled = array();

                // load recommended lists, if any
                if (sizeof($recommended_rbl) != 0) {
                    foreach ($recommended_rbl as $i) {
                        $default_enabled[] = $this->lists[$i]['rbl'];
                    }
                }
                $this->rbl_enabled = $default_enabled;

                // Erases existing list
                $listMgr = ListManager::getInstance();
 
                // If Professional and we've got flags
                if (isProfessional() && defined('RBL_FLAG')) {
                    // clear flags constrol file 
                    $flagRet = clearControl(RBL_FLAG);  
                    if ($flagRet !== TRUE) {
                        $errors[] = intl("rblCannotWriteToFile");                           
                    }
                }
                $ret = $listMgr->load('-E', DATA_PATH, $lists);
                if ($ret == 1) {
                    foreach ($lists as $list) {
                        $args = array('-E');
                        $args[] = '-d';
                        $args[] = $list;
                        $args[] = '-h';
                        $args[] = DATA_PATH;
                        $retDel = $listMgr->delete($args);
                        if ($retDel != 1) {
                            // TODO: what if error() is blank?
                            $errors[] = htmlspecialchars($listMgr->error());
                            return;
                        }
                    }
                } else {
                    // TODO: what if error() is blank?
                    $errors[] = htmlspecialchars($listMgr->error());
                    return;
                }
                // adds current selected lists
                foreach ($default_enabled as $entry) {
                    $args = array('-E');
                    $args[] = '-a';
                    $args[] = $entry;
                    $args[] = '-h';
                    $args[] = DATA_PATH;
                    $retAdd = $listMgr->add($args);
                    if ($retAdd !== TRUE ) {
                        $errors[] = intl("rblCannotWriteToFile");
                        return;
                    }
                }
                $notices[] = intl("rblDefaults");
                return;
        // otherwise if we are saving from the form post...
        } else if ($action === 'save_lists') {
            $selected_arr = array();
            $flagarr = array();

            // Grab the posted values of the slider buttons
            if (isset($_POST[$this->radio_name])) {
                foreach ($_POST[$this->radio_name] as $key => $value) {
                    if ($value == "true" || $value == 'flag') {
                        $selected_arr[] = $key;
                    }
                    if ( isProfessional()
                       && $value == 'flag'
                       && isset($this->lists[$key])
                       && isset($this->lists[$key]['rbl'])) {
                        $flagarr[] = $this->lists[$key]['rbl'];
                    }
                }
            }

            // convert the posted values to their URL counterparts
            $aHyperlinks = array();
            if (isset($selected_arr) && is_array($selected_arr)) {
                foreach ($selected_arr as $s) {
                    if (isset($this->lists[$s]) && isset($this->lists[$s]['rbl'])) {
                        $aHyperlinks[] = $this->lists[$s]['rbl'];
                    }
                }
            }

            // let write the flags
            $flagRet = TRUE;

            // ListManager is invoked to write
            $listMgr = ListManager::getInstance();

            // If using the Professional version and we have flags...
            if (isProfessional() && defined('RBL_FLAG')) {

                // if PRO write the flag rules
                $flagstr = implode("\n", $flagarr);
                if (!strlen($flagstr)) {
                    $flagRet = clearControl(RBL_FLAG);
                } else {
                    $flagRet = setControl(RBL_FLAG, $flagstr);
                }

                // get any errors
                if ($flagRet !== TRUE) {
                    $errors[] = $flagRet;            
                }
            }

            $ret = $listMgr->load('-E', DATA_PATH, $lists);
            if ($ret == 1) {
                foreach ($lists as $entry) {
                    $args = array('-E');
                    $args[] = '-d';
                    $args[] = $entry;
                    $args[] = '-h';
                    $args[] = DATA_PATH;
                    $retDel = $listMgr->delete($args);
                    if ($retDel != 1) {
                        // TODO: what if error() is blank?
                        $errors[] = htmlspecialchars($listMgr->error());
                        return;
                    }
                }
            } else {
                // TODO: what if error() is blank?
                $errors[] = htmlspecialchars($listMgr->error());
                return;
            }

            // writes selection to RBL enabled
            foreach ($aHyperlinks as $entry) {
                $args = array('-E');
                $args[] = '-a';
                $args[] = $entry;
                $args[] = '-h';
                $args[] = DATA_PATH;
                $retAdd = $listMgr->add($args);
                if ($retAdd !== TRUE) {
                    $errors[] = intl("rblCannotWriteToFile");
                    return;
                }
            }
            
            $notices[] = intl("rblLists") . " " . intl("saved");
            return;
        // Add new list (name, host)
        } else if ($action === 'add_rbl_list') {
            $res = $this->_actionAddRBLList();
            $errors = array_merge($errors, $res['errors']);
            $notices = array_merge($notices, $res['notices']);
            return;
        } else if ($action === 'delete_rbl') {
            // Determines which delete button was pressed
            $listToDelete = urldecode($_POST['delete']);
            // If not a user defined list, don't delete
            if ($this->isDefaultlist($listToDelete)) {
                $errors[] = intl("rblCannotDeletePredefinedList");
                return;
            }

            // remove the RBL from the list
            $rblURL = $this->lists[$listToDelete]['rbl'];
            $rblName = $this->lists[$listToDelete]['description'];
            unset($this->lists[$listToDelete]);

            // remove it from the enabled list if it's there.
            $wasEnabled = FALSE;
            foreach ($this->rbl_enabled as $i => $v) {
                if ($v === $rblURL) {
                    unset($this->rbl_enabled[$i]);
                    $wasEnabled = TRUE;
                    break;
                }
            }

            // Prepare command to remove (-d -R "host name")
            $listMgr = ListManager::getInstance();
            $args = array('-R');
            // prepare entry to be deleted
            $entryToDel = $rblURL." ".$rblName;
            $args[] = '-d';
            $args[] = $entryToDel;
            $args[] = '-h';
            $args[] = DATA_PATH;
            $retDel = intval($listMgr->delete($args));
            if ($retDel == ListManager::ENTRY_NOT_FOUND) {
                $errors[] = intl("rblEntryNotFound");
                return;
            } else if ($retDel == ListManager::INVALID_FLAG) {
                $errors[] = intl("rblInvalidFlagError");
                return;
            } else if ($retDel == ListManager::FLAG_ERROR) {
                $errors[] = intl("rblFlagError");
                return;
            } else if ($retDel < 1) {
                $errors[] = intl("rblCannotWriteToFile");
                return;
            }
            // and from enabled list.
            if ($wasEnabled) {
                $args = array('-E');
                $args[] = '-d';
                $args[] = $rblURL;
                $args[] = '-h';
                $args[] = DATA_PATH;
                $retDel = intval($listMgr->delete($args));
                if ($retDel == ListManager::ENTRY_NOT_FOUND) {
                    $errors[] = intl("rblEntryNotFound");
                    return;
                } else if ($retDel == ListManager::INVALID_FLAG) {
                    $errors[] = intl("rblInvalidFlagError");
                    return;
                } else if ($retDel == ListManager::FLAG_ERROR) {
                    $errors[] = intl("rblFlagError");
                    return;
                } else if ($retDel < 1) {
                    $errors[] = intl("rblCannotWriteToFile");
                    return;
                }
            }
            $notices[] = htmlspecialchars($rblName) . intl("rblDeleteSuccess");
            return;
        } else {
            $errors[] = intl("rblUnknownAction");
        }
    }

    //! method to render the full page module.
    /*!
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function renderModule() {
        global $module, $recommended_rbl, $controller;
        if (!isFreemium()) {
            $recommended_rbl = array();
        }

        // add the i18n we need in javascript.
        $arrayi18n = array('rblCloseLabel','rblHelpPopupLabel','rblHelpList','rblConfirmDelete','rblRatsNoptr','rblRatsDyna','rblRatsSpam');
        foreach ($arrayi18n as $rblpop) {
            $this->_i18n_js[] = $rblpop;
        }

        // checks if the table is empty
        $isEmpty = count($this->lists) === 0;

        // Defines the help button which gets attached to the table
        $helpButton = "<span class=\"collapseActions\">\n"
                    . "<a href=\"#\" onclick=\"MS.showHelpPopup(this, 'rblCloseLabel', 'rblHelpPopupLabel'); return false;\">\n"
                    . "  <span class=\"helplink\"></span>\n"
                    . "  <strong>" . intl('help') . "</strong>\n"
                    . "</a></span>\n";

        // Defines the mini-help button on the left-hand side
        $miniHelpButton = "<a id=\"mini-help\" href=\"#\""
                        . "onclick=\"MS.showHelpPopup(this, 'rblCloseLabel',"
                        . "'rblHelpPopupLabel'); return false;\">\n"
                        . "     <span class=\"helplink\"></span>\n"
                        . "</a>\n";

        // set up the table style
        $table = new magicspamDataTable();
        $table->setClass("row", array("leftColumn",
                                      "centerColumn",
                                      "centerColumn",
                                      "actionColumn"));
        $table->setClass("headerRow", array("leftColumn",
                                            "centerColumn",
                                            "centerColumn",
                                            "actionColumn"));
        $table->setHeader(array(intl('rblListName'),
                                '&nbsp;',
                                intl('status'),
                                $helpButton));

        // load the enabled items from rules.enabled
        // array will be empty if file is missing or empty
        $lists_enabled = array();
        $recommendedlabel = intl('recommended');

        $listMgr = ListManager::getInstance();
        $lists_flag =  array();

        // Get RBL flags for professional only 
        if (isProfessional() && defined('RBL_FLAG')) {
            $tmp = getControl(RBL_FLAG, "");
            if (strlen($tmp) > 0) { // something was returned, set the rules and do the count
                $lists_flag = explode("\n", $tmp);
            }
        }
        $ret = $listMgr->load('-E', DATA_PATH, $lists_enabled);
        if ($ret != 1) {
            // TODO: what if error() is blank?
            $errors[] = htmlspecialchars($listMgr->error());
        }

        // set the default and enabled value for each rule
        foreach ($this->lists as $rule => &$data) {
            $data['default'] = in_array($rule, $recommended_rbl);
        }

        // Prints out the page subtitle
        $listlabel = intl('rblListLabel');

        // Prints out the individual rows of the table
        $rowNum = 0;

        // if this is freemium, then we want to parse out the first static lists first
        $sortList = array(); 
        $displayList = array(); 
        if (isFreemium()) { 
            
            // Grab the static entries and do no sort them 
            foreach ($this->lists as &$entryProperties)  { 
                if ($this->isDefaultlist($entryProperties['description'])) {
                    $displayList[$entryProperties['description']] = $entryProperties; 
                } else {
                    $sortList[$entryProperties['description']] = $entryProperties;
                }
            }
 
            // Sort the added entries 
            uasort($sortList, array($this, '_descCmp')); 
            
            // Add the sorted entries back into the list we plan on displaying 
            foreach ($sortList as &$entryProperties)  { 
                $displayList[$entryProperties['description']] = $entryProperties; 
            }

        // Else this is not freemium, so we sort the whole list
        } else {
            $displayList = $this->lists; 
            uasort($displayList, array($this, '_descCmp')); 
        } 

        // Cycle thru the list and render the content
        foreach ($displayList as $rule => &$data) {
            $desc = "";
            $url = $target = $onclick = NULL;
            if (isset($data['help']) && $data['help']) {
                $help = $data['help'];
                $onclick = "MS.showHelpPopup(this,'X','".$help."'); return false;";
            } else {
                $onclick = "MS.showHelpPopup(this,'X', 'rblHelpList'); return false;";
            }
            $description = htmlspecialchars($data['description']);
            $internal = "({$listlabel}: " . htmlspecialchars($data['rbl']) . ")";

            $desc .= renderListDescription($description, $internal, $url, $onclick, NULL);

            if (isset($data['bymaster']) && $data['bymaster'] === TRUE) {
                $defstat = generateLabel("enforced");
            } else {
                $defstat = generateLabel("recommended", $data['default']);
            }

            // Determines if the user / Mastercontroller enabled a list
            $isEnabled = FALSE;
            if ((isset($data['rbl']) && in_array($data['rbl'], $lists_enabled))
               || (isset($data['enabled']) && $data['enabled'] === TRUE)) {
                $isEnabled = TRUE;

                // determine if this is 'flag'
                if (isProfessional() && in_array($data['rbl'], $lists_flag)) {
                    $isEnabled = 'flag';
                }
            }
            // Determines if Mastercontroller is "locking" a specific setting
            // NOTE: technically this should never happen
            $isLocked = FALSE;
            if (isset($data['bymaster']) && $data['bymaster'] === TRUE) {
                $isLocked = TRUE;
            }

            // professional (and plus) we render a three state slider
            if (isProfessional()) {
                $sliderWidget = renderSliderThree($isEnabled, $isLocked, $this->radio_name . "[{$rule}]", 'rblListsForm');
            // everything else gets a two state slider
            } else {
                $sliderWidget = renderSlider($isEnabled, $isLocked, $this->radio_name . "[{$rule}]", 'rblListsForm');
            }
            
            // Checks if a row has been user defined, then prints a
            // delete icon.
            $deleteRowButton = "<img class=\"trashcan_grey\" alt=\"" .
                               intl('rblNoDelete') . "\"" .
                               "title=\"" . intl('rblNoDelete') .
                               "\" src=\"images/icon-trashcan-grey.png\"></img>";
            if (!$this->isDefaultList($data['description'])) {
                // delete button with function to confirm deletion.
                $deleteRowButton = '<button class="deleteRBL"';
                $deleteRowButton .= ' title="' . intl('rblDelete');
                $deleteRowButton .= '" onclick="return ';
                $deleteRowButton .= 'confirmDeleteRBL(';
                // NOTE: to avoid issues with ' and \ characters in the
                //   string here we're applying rawurlencode()... which
                //   is a bit of a workaround I think... but works!
                $deleteRowButton .= '\'' . htmlspecialchars(rawurlencode($data['description'])) . '\');"></button>';
            }
            // Attaches the new row to the table
            $table->addRow(array($desc, $defstat, $sliderWidget, $deleteRowButton));
            $rowNum++;
        }
        if (!isFreemium()) {
            $blurb = intl('rblBlurbLicensed');
        } else {
            $blurb = intl('rblBlurbFree');
        }
        // Prints out the rbl table
        echo "<div class=\"enforceGrey\">" . "\n";
        renderTemplate('rblLists', array(
                                  'header'         => intl('rblLists'),
                                  'blurb'          => $blurb,
                                  'table'          => $table->getRenderString(),
                                  'module'         => $module,
                                  'save'           => intl('save'),
                                  'add'            => intl('add'),
                                  'defaults'       => intl("defaults"),
                                  'isEmpty'        => $isEmpty,
                                  'miniHelpButton' => $miniHelpButton,
                                  'isFreemium'     => isFreemium(),
                                  'rblListName'    => intl('rblBlankListName'),
                                  'rblHostName'    => intl('rblBlankHostName'),
                                  'rblAddNewTitle' => intl('rblAddNewTitle'),
                                ));
        echo "</div><!-- end #enforceGrey -->" . "\n";
    }

    //! method to check the existence of repeated rules in RBL
    /*!
     * @param  listName      the list we want to check existence
     * @param  listHost      the list url to be checked for existence
     *
     * @returns mixed FALSE If the list does not exist. Otherwise
     *   we will return an error string to indicate why we're
     *   saying failure.
     *
     * @author Daniel Peitl <danielp@linuxmagic.com>
     */
    public function existsRBL($listName, $listHost) {
        if (!is_string($listName) || strlen($listName) === 0
            || !is_string($listHost) || strlen($listHost) === 0) {
            return intl('rblInvalidNameOrHost');
        }
        // lowercase and trim both strings
        $listName = strtolower(trim($listName));
        $listHost = strtolower(trim($listHost));

        // checks each rule's description and host
        foreach ($this->lists as $rule => &$data) {
            $dataDescription = strtolower(trim($data['description']));
            $dataRBL = strtolower(trim($data['rbl']));

            // if matches, return
            if ($listName === $dataDescription || $listHost === $dataRBL) {
                return intl('rblRepeatedEntry');
            }
        }
        // it's indeed a new entry
        return FALSE;
    }

    //! method to retreive the title of the current module
    /*!
     * @returns string  the title of the current module
     *
     * @author Kyle Macdonald <kylem@linuxmagic.com>
     *
     * SIDE EFFECTS: this function may call exit(1) if the conditions xyz are met.
     * TODO: write better validation for the arguments
     * etc. etc.
     */
    public function getModuleTitle() {
        return intl('rblLists');
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getCSS() {
        return array('rbl.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page
    // rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     */
    public function getJS() {
        return array(
            'set-action.js',
            'help-popup.js',
            'ip-form.js',
        );
    }

    //! method converts data to JSON format & writes data to a file
    /*!
     * @param  file      the file in question being written to
     * @param  contents  string/array to write to file
     *
     * @returns boolean  comes back true if the file write was successful
     *
     * @author Robert Bisewski <robertb@linuxmagic.com>
     */
    public function setJSONFileData($file, $contents) {
        try {
            $result = file_put_contents($file, json_encode($contents), LOCK_EX);
            if ($result !== FALSE) {
                return TRUE;
            }
        } catch (Exception $e) {
            // file write was unsuccessful, ergo must return false...
        }
        return FALSE;
    }

  //! method to check if list is default
    /*!
     * @param  listName      the list we want to check
     *
     * @returns true if name matches any of default lists
     *
     * @author Daniel Peitl <danielp@linuxmagic.com>
     */
    public function isDefaultList($listName) {
        return is_string($listName) &&
            ($listName === 'Rats-Dyna' || $listName === 'Rats-NOPTR' || $listName === 'Rats-SPAM');
    }

    //! utility function for comparing descriptions - used for uasort.
    /*!
     * @param   array   the first object to compare
     * @param   array   the second object to compare
     *
     * @returns integer < 1 if a < b, > 0 if a > b, 0 if both are equal
     */
    public function _descCmp($a, $b) {
        // sanity check: bad args = 0
        if (  !is_array($a) || !is_array($b)
           || !isset($a['description'])
           || !isset($b['description'])
           || !isset($a['rbl'])
           || !isset($b['rbl']) ) {
            return 0;
        }
        // first compare the descriptions.
        $cmp1 = strcasecmp($a['description'], $b['description']);
        // if the descriptions are different, then that's enough
        if ($cmp1 !== 0) {
            return $cmp1;
        }
        // otherwise, we fall through to the RBL list if the
        // descriptions are the same.
        // XXX: do we allow descriptions to be the same?  We shouldn't.. but
        //      this is a good fallthrough anyways.
        return strcasecmp($a['rbl'], $b['rbl']);
    }

    //! retrieve the migratable configurations of the module
    /*!
     * @returns array  the migratable configurations
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    public function getMigrationData() {
        // the list of migratable configurations
        $configurations = array();

        // retrieve and store the available RBLs
        $listMgr = ListManager::getInstance();
        $listMgr->load('-R', DATA_PATH, $available);
        $configurations[RBL_AVAILABLE] = $available;

        // retrieve and store the enabled RBLs
        $listMgr->load('-E', DATA_PATH, $enabled);
        $configurations[RBL_ENABLED] = $enabled;

        // retrieve and store the flag only RBLs
        if (isProfessional() && defined('RBL_FLAG')) {
            $configurations[RBL_FLAG] = getControl(RBL_FLAG, "", MASTERCONTROLDIR);
        }

        // return the migration configurations
        return $configurations;
    }

    //! loads the migratable configurations of the module
    /*!
     * @param   array  the configurations to load
     *
     * @returns bool   whether the configurations were loaded successfully
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    public function loadMigrationData($configurations) {
        // sanity check
        if (!isset($configurations) || !WV::isArrayNotEmpty($configurations)) {
            return FALSE;
        }

        // initialize list manager
        $listMgr = ListManager::getInstance();

        // add and disable all the available RBLs from the configurations
        foreach ($configurations[RBL_AVAILABLE] as $rbl) {
            $args = array('-R', '-a', $rbl);
            $args[] = '-h';
            $args[] = DATA_PATH;
            $ret = $listMgr->add($args);
            $args = array('-E', '-d', explode(" ", $rbl)[0]);
            $args[] = '-h';
            $args[] = DATA_PATH;
            $ret = $listMgr->delete($args);
        }

        // store the enabled RBLs from the configurations
        foreach ($configurations[RBL_ENABLED] as $rbl) {
            $args = array('-E', '-a', $rbl);
            $args[] = '-h';
            $args[] = DATA_PATH;
            $ret = $listMgr->add($args);
        }

        // store the flag only RBLs
        if (isProfessional() && defined('RBL_FLAG')) {
            // find the existing RBLs that are not in the configuration
            $listMgr->load('-R', DATA_PATH, $available);
            $untracked = array_diff($available, $configurations[RBL_AVAILABLE]);
            array_walk($untracked, function(&$rbl) {
                $rbl = explode(" ", $rbl)[0];
            });

            // retrieve the existing RBLs that are flagged
            $flagged = explode("\n", getControl(RBL_FLAG, ""));
            $untracked_flagged = join("\n", array_intersect($untracked, $flagged));

            if (array_key_exists(RBL_FLAG, $configurations) && WV::isStringNotEmpty($configurations[RBL_FLAG])) {
                if (WV::isStringNotEmpty($untracked_flagged)) {
                    setControl(RBL_FLAG, $configurations[RBL_FLAG] . "\n$untracked_flagged");
                } else {
                    setControl(RBL_FLAG, $configurations[RBL_FLAG]);
                }
            } else if (WV::isStringNotEmpty($untracked_flagged)) {
                setControl(RBL_FLAG, $untracked_flagged);
            } else {
                clearControl(RBL_FLAG);
            }
        }

        // success, migratable configurations successfully loaded
        return TRUE;
    }
}
