<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: spamSettingsModule.php 6173 2017-01-25 17:24:37Z shaun $
 */
require_once('functions.php');
require_once('lib.module.php');
require_once('lib.datatable.php');
require_once('lib.rejections.php');
require_once('lib.globaltoggle.php');

class spamSettingsTemplateModule extends Module {
    protected $rules;
    protected $rules_subset;

    //! string. this is to track the request action that was last run.
    // why do we have it? because for 'set recommended defaults' we want to also
    // be able to show, from javascript, a popup bubble warning. this is a way
    // to communicate with javascript to tell it which action was run.
    private $_action_flag = '';

    private $_flag_rules = array();

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);

        $this->rules = array(
            'block_lists' => array(
                'description' => 'Block messages based on source IP Reputation (BMS&trade;)',
                'help' => 'sp_ipRep',
                'bymaster' => TRUE,
                'enabled' => TRUE,
                'freemium' => FALSE,
            ), 'block_rbl_lists' => array(
                'description' => 'Block messages based on source IP Reputation (RBL)',
                'url' => "http://www.linuxmagic.com/power_of_ip_reputation.html",
                'help' => 'sp_ipRepRBL',
                'bymaster' => TRUE,
                'enabled' => TRUE,
                'freemium' => FALSE,
            ), 'check_dynamic_reverse_dns' => array(
                'description' => 'Block Mail Servers using Probable Dynamic/DUL IP space (DYNA REGEX FULL) ',
                'description_freemium' => 'Block Mail Servers using Probable Dynamic/DUL IP space (DYNA REGEX LITE) ',
                'help' => 'sp_dynaRegex',
                'freemium' => TRUE,
            ), 'check_confirmed_dyna' => array(
                'description' => 'Block Mail Servers using Confirmed Dynamic/DUL IP space (DYNA REGEX) ',
                'help' => 'sp_dynaRegex',
                'freemium' => FALSE,
                'default' => TRUE,
            ), "check_ip_reverse_dns" => array(
                "description" => "Require Mail Servers to have rDNS configured",
                "help" => 'sp_revDns',
                'default' => TRUE,
            ), 'check_reverse_dns_list' => array(
                'description' => 'Block Mail Servers reported as Spam Source (NONDUL REGEX) ',
                'help' => 'sp_dnsList',
                'freemium' => FALSE,
            ), "require_helo" => array(
                "description" => "Sending Server must identify itself (HELO)",
                "help" => 'sp_reqHelo',
                'default' => TRUE,
            ), "valid_helo_domain" => array(
                "description" => "Server Identification should be sane (FQDN HELO) ",
                "help" => 'sp_validHelo',
                'default' => TRUE,
            ), "resolve_helo_domain" => array(
                "description" => "Confirm Server Identification Resolves (HELO) ",
                "help" => 'sp_resolveHelo',
            ), "rfc_mail_from" => array(
                "description" => "MAIL FROM: must meet RFC2822 specifications",
                "help" => 'sp_rfcFrom',
            ), "require_full_addr" => array(
                "description" => "Strict Email Address Parsing (RFC Compliance)",
                "help" => 'sp_fullAdd',
                'default' => TRUE,
            ), "valid_from_domain" => array(
                "description" => "Valid FROM domain (A or MX Record)",
                "help" => 'sp_validFrom',
                'default' => TRUE,
            ), "valid_ptr" => array(
                "description" => "PTR record should be FQDN (Best Practices) ",
                "help" => 'sp_validPtr',
                'default' => TRUE,
            )
        );
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $recommended_rules, $recommended_rules_free;

        // store the action - we want to provide it to the javascript
        // environment in some cases.
        $this->_action_flag = $action;
        // sets the state of the collaspible modules
        if ($action === 'remember_expand_state') {
                $module = isset($_POST['changed_module']) ? $_POST['changed_module'] : NULL;
                $value  = isset($_POST['expanded']) ? $_POST['expanded'] : NULL;
                $var = $module . "__collapse";
                $_SESSION[$var] = $value;
        }

        // action to update what rules are enabled.
        if ($action == 'save_server_policies') {
            if ($this->handlePolicySave() === FALSE) {
                $errors[] = intl('errorFailSave');
            }

        // action to set rules to the defaults.
        } else if ($action == 'set_server_policies_to_defaults') {
            $target_rules = array();

            foreach ($this->rules as $rule => $props) {
                if ($props['default'] === TRUE) {
                    $target_rules[] = $rule;
                }
            }

            $ret = setControl(RULES_TEMPLATE_ENABLED, implode("\n", $target_rules));
            if ($ret !== TRUE) {
                $errors[] = intl('errorRestoreRecommended');
                return;
            }

            $ret = clearControl(RULES_TEMPLATE_FLAG);
            if ($ret !== TRUE) {
                $errors[] = intl('errorRestoreRecommended');
                return;
            }

            $notices[] = intl("serverPolicies"). " " . intl("saved");

        } else if ($action == 'sync_server_policies') {
            // ensure that there is a policy to sync
            if (empty($_POST['policy-name'])) {
                $errors[] = intl('policySyncError');
                return;
            }
            $policy = $_POST['policy-name'];

            // ensure that the individual policy has been saved
            if ($_POST['ori_rules'][$policy] !== $_POST['rules'][$policy]) {
                if ($this->handlePolicySave() === FALSE) {
                    $errors[] = intl('errorFailSave');
                    return;
                }
            }

            // sync the individual policy
            if ($this->handlePolicySync($policy) === FALSE) {
                $errors[] = intl('policySyncError');
                return;
            }

            $notices[] = $_POST['policy-name'] . ' ' . intl('policySynced');
        } else if ($action == 'sync_all_server_policies') {
            // ensure that all the policies have been saved
            foreach ($_POST['ori_rules'] as $policy => $status) {
                if ($status !== $_POST['rules'][$policy]) {
                    if ($this->handlePolicySave() === FALSE) {
                        $errors[] = intl('errorFailSave');
                        return;
                    }
                    break;
                }
            }

            // sync all the policies
            if ($this->handlePolicySyncAll() === FALSE) {
                $errors[] = intl('errorFailSyncAll');
                return;
            }

            $notices[] = intl('noticeSuccessSyncAll');
        } 
    }

    //! Save all the spam policies into template files
    protected function handlePolicySave() {
        // create two empty arrays for keeping track of enabled and flagged rules
        $enabledRules = array();
        $flaggedRules = array();

        // these rules should be ignored
        $ignoreRules = array(
            'block_lists',
            'block_rbl_lists',
        );

        // sanity check
        if (empty($_POST["rules"])) {
            return FALSE;
        }

        // keep track of which rules have been enabled or flagged
        foreach ($_POST["rules"] as $rule=>$status) {
            // skip any ignored rules
            if (in_array($rule, $ignoreRules)) {
                continue;
            }

            // add rule to corresponding array
            if ($status !== "false") {
                $enabledRules[] = $rule;
                if ($status === "flag") {
                    $flaggedRules[] = $rule;
                }
            }
        }

        // regardless of how the rules are set, we always remove all
        // the rules from both the enabled and flagged files
        // start removal from the enabled file
        $ret = clearControl(RULES_TEMPLATE_ENABLED);
        if ($ret !== TRUE) {
            return FALSE;
        }
        // continue on removing from the flagged file
        $ret = clearControl(RULES_TEMPLATE_FLAG);
        if ($ret !== TRUE) {
            return FALSE;
        }

        // at this point, both the enabled and flagged files will be empty
        // so we can add rules to them as necessary and not worry about any
        // more removals

        // start by creating a single string for both enabled and flagged rules
        $enabledString = implode("\n", $enabledRules);           
        $flaggedString = implode("\n", $flaggedRules);           

        // then add enabled rules to the enabled file
        if (strlen($enabledString) > 0) {
            $ret = setControl(RULES_TEMPLATE_ENABLED, $enabledString);
            if ($ret !== TRUE) {
                return FALSE;
            }
        }

        // finally add the flagged rules to the flagged file
        if (strlen($flaggedString) > 0) {
            $ret = setControl(RULES_TEMPLATE_FLAG, $flaggedString);
            if ($ret !== TRUE) {
                return FALSE;
            }
        }

        return TRUE;
    }

    //! Sync all the spam policies with all users
    protected function handlePolicySyncAll() {
        // sanity check
        if (!is_array($_POST['rules'])) {
            return false;
        }

        // go through and sync each spam setting
        foreach ($_POST['rules'] as $policy => $key) {
            if ($this->handlePolicySync($policy) === FALSE) {
                return false;
            }
        }

        return true;
    }

    //! Sync an individual spam policies with all users
    protected function handlePolicySync($policy) {
        // sanity check
        if (empty($_POST['rules'][$policy])) {
            return false;
        }

        // ignore the hidden spam settings
        if ($policy === "block_lists" || $policy === "block_rbl_lists") {
            return true;
        }

        $policySliderOption = $_POST["rules"][$policy];
        if ($policySliderOption === 'true') {
            $cmdOption = GlobalToggle::ENABLE;
        } else if ($policySliderOption === 'false') {
            $cmdOption = GlobalToggle::DISABLE;
        } else if ($policySliderOption === 'flag') {
            $cmdOption = GlobalToggle::FLAG;
        } else {
            return false;
        }

        $toggleCmd = GlobalToggle::getInstance();
        if($toggleCmd->toggleSpamRule($policy, $cmdOption) === FALSE) {
            return false;
        }

        return true;
    }

    //! method to render the full page module.
    /*!
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function renderModule() {
        global $module;

        $enabled_rules = array();

        // and help popups
        foreach ($this->rules as $rule => &$data) {
            $this->_i18n_js[] = $data['help'];
        }

        // load the enabled items from rules.enabled
        $tmp = getControl(RULES_TEMPLATE_ENABLED, "");
        if (strlen($tmp) > 0) { // something was returned, set the rules and do the count
            $enabled_rules = explode("\n", $tmp);
        }

        // set the default and enabled values
        $this->_setDefaultEnabled($enabled_rules);

        $this->_getFlagRules();

        // set up the base column classes for rendering
        $rowClasses = array('ruleDesc', 'recommendation', 'ruleStatus');


        // then we want to allow custom rejections
        // and render a toggle link
        // now, we also modify the classes if we are showing the rejection buttons per request.
        $rowClasses = array('ruleDescSmall',
                            'recommendationSmall',
                            'ruleStatus',
                            'ruleActionsVisible');
        // and set up the header we want rendered
        $header = '<span class="ruleDescSmall">' . intl("spamSettingsHeader") . "</span>\n"
                . '<span class="recommendationSmall">&nbsp;</span>' . "\n"
                . '<span class="ruleStatus">' . intl('status') . "</span>\n"
                . '<span class="ruleActionsVisible">' . intl("sync") . "</span>\n";
        
        // Validate when we determine the state of the tab
        if (isset($_SESSION['serverPolicies__collapse']) && $_SESSION['serverPolicies__collapse'] != '') {
            $collapseState = $_SESSION['serverPolicies__collapse'];
        
        // Else open by default
        } else {
            $collapseState = 1;
        }
 

        // prepare our table
        $table = $this->_prepareDataTable($rowClasses);

        renderTemplate('serverPoliciesTemplate', array(
            'table'         => $table->getRenderString(),
            'module'        => $module,
            'header'        => $header,
            'actionFlag'    => htmlspecialchars($this->_action_flag),
            'collapseState' => $collapseState,
        ));
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     */
    public function getCSS() {
        return array('server-policies.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getJS() {
        return array(
                     'ajax.js'
                     ,'server-policies.js'
                     ,'user-settings.js'
                    );
    }

    //! internal method to prep the 'default' rule status
    /*!
     * @param   array   the array of rules that are currently enabled.
     */
    private function _setDefaultEnabled($enabled_rules) {
        global $recommended_rules, $recommended_rules_free;
        // safety check to ensure we are working with an array.
        if (!is_array($enabled_rules)) {
            $enabled_rules = array();
        }
        // for each rule that is in the list
        foreach ($this->rules as $rule => &$data) {
            // then, set the 'enabled' status based on the passed in array.
            $data['enabled'] = in_array($rule, $enabled_rules);
        }
        // done - all other information pass through is held in internal variable '$rules'
    }


    //! internal method to prepare a data table for rendering
    /*!
     * @param array $columnClasses TODO document
     * @param bool $renderRejections TODO document
     *
     * @return magicspamDataTable
     *
     * @author Anonymous
     * @author William Storey <william@linuxmagic.com>
     */
    private function _prepareDataTable($columnClasses) {
        $defstat = "";

        // safety check
        if (!is_array($columnClasses)) {
            $columnClasses = array();
        }

        // create our table object
        $table = new magicspamDataTable();

        // set the classes
        // XXX: what happens if its empty?
        $table->setClass("row", $columnClasses);

        // and generate our data rows
        foreach ($this->rules as $rule => &$data) {
            if (  !isset($this->rules_subset)
               || (isset($this->rules_subset) && is_array($this->rules_subset))) {

                $help = isset($data['help']) ? $data['help'] : "";
                $description = isset($data['description']) ? $data['description'] : "";
                if (!isset($data['bymaster'])) {
                    $data['bymaster'] = FALSE;
                }

                $internal = htmlspecialchars(intl('rule_name')) . ": " . htmlspecialchars($rule);

                // some rules have note text under certain conditions.
                $noteHTML = NULL;
                if (!isFreemium() && $rule === 'check_dynamic_reverse_dns') {
                    $noteHTML = '<p>';
                    $noteHTML .= intl('serverPoliciesLastUpdate');
                    $noteHTML .= ' ';
                    $noteHTML .= htmlspecialchars(
                        $this->_getRegexUpdateTime('dyna'));
                    $noteHTML .= '</p>';
                }
                if (!isFreemium() && $rule === 'check_confirmed_dyna') {
                    $noteHTML = '<p>';
                    $noteHTML .= intl('serverPoliciesLastUpdate');
                    $noteHTML .= ' ';
                    $noteHTML .= htmlspecialchars(
                        $this->_getRegexUpdatetime('confirmed'));
                    $noteHTML .= '</p>';
                }
                $desc = renderListDescription($description, $internal, NULL,
                    "MS.showHelpPopup(this,'X', '".$help."'); return false;",
                    "_blank", $noteHTML);

                $ruleEnabled = FALSE;
                if (isset($data['enabled']) && $data['enabled'] === TRUE) {
                    $ruleEnabled = TRUE;
                    if (in_array($rule, $this->_flag_rules)) {
                        $ruleEnabled = 'flag';
                    }
                }
                
                $defstat = generateLabel("recommended", isset($data['default']) ? $data['default'] : FALSE);

                $statusWidget = renderSliderThree($ruleEnabled, FALSE, "rules[{$rule}]", 'serverPoliciesForm');

                $syncButton = '<button type="button" class="msbutton" data-name="'. $rule .  '" onclick="syncServerPolicy(event);"  ><span>' . intl('sync') . '</span></button>' . "\n";

                // ok so now we can prep our row.
                $rowArr = array($desc, $defstat, $statusWidget, $syncButton);

                // set an id on the dyna regex rule row.
                $rowID = NULL;
                if ($rule === 'check_dynamic_reverse_dns') {
                    $rowID = 'row_check_dynamic_reverse_dns_freemium';
                } else if ($rule === 'check_confirmed_dyna') {
                    $rowID = 'row_check_confirmed_dyna';
                }
                $table->addRow($rowArr, $rowID);
            }
        }
        return $table;
    }

    //! find the last update time of the dyna regex list.
    /*!
     * @return string Text describing the last update time.
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * TODO: does this belong in this class?
     *
     * we open and read the header of the dyna regex file.
     * we parse out the date and if we can.
     * we then translate this into visible text.
     */
    private function _getRegexUpdateTime($prefix) {
        // TODO: can we have a define to just /etc/magicspam?
        //   this is under /etc/magicspam/dynaregex, but
        //   we appear to only have /etc/magicspam/control as a define.
        // sanity check argument
        if (!is_string($prefix) || !strlen($prefix)) {
            return "";
        }
        $isCompiled = TRUE;
        $regexFile = CONTROLDIR . '/../dynaregex/' . $prefix . '.cregexes';
        if (!file_exists($regexFile)) {
            // compiled not found, look for plain text version
            $regexFile = CONTROLDIR . '/../dynaregex/' . $prefix . '.regexes';
            if (!file_exists($regexFile)) {
                return intl('serverPoliciesRegexFileNotFound');
            } else {
                $isCompiled = FALSE;
            }
        }
        if (!is_readable($regexFile)) {
            return intl('serverPoliciesRegexFileNotReadable');
        }
        // if this is a non compiled version, then test the format
        if (!$isCompiled) {
            // first line of the file has what we want.
            // it looks like:
            // # Dynamic Style Regular Expression Listing, Autogenerated by LinuxMagic, October 1, 2014, 11:34 am
            $fh = fopen($regexFile, 'r');
            if (!$fh) {
                return intl('serverPoliciesRegexFileCouldNotOpen');
            }
            $line = fgets($fh);
            fclose($fh);
            if ($line === FALSE) {
                return intl('serverPoliciesRegexFileCouldNotRead');
            }
            $line = trim($line);
            // parse out the part we want.
            $res = preg_match('/Autogenerated by LinuxMagic, (.*)$/', $line,
                $matches);
            if ($res !== 1 || count($matches) !== 2 || strlen($matches[1]) === 0) {
                return intl('serverPoliciesRegexFileCorrupt');
            }
        }

        // ok.. let's stat that thing
        $fstat = stat($regexFile);
        if (!is_array($fstat) || sizeof($fstat) < 10) {
            return intl('serverPoliciesRegexFileNotReadable');
        }

        $unixtime = $fstat[9];

        // NOTE: see lib.logsearch date() call too.
        $timeFormat = @date("l, M d Y @ g:i A", $unixtime);
        if ($timeFormat === FALSE) {
            return intl('serverPoliciesRegexDateFormat');
        }
        return $timeFormat;
    }

    //! internal method to get flag rules 
    /*!
     * This will work only for Pro version
     * on Freemium this array will be empty
     */
    private function _getFlagRules() {

        // in freemium mode we don't allow flag rules
        if (!isProfessional() || !defined('RULES_FLAG')) {
            $this->_flag_rules = array();
            return;
        }

        // load the flagonly items from rules.flagonly
        $tmp = getControl(RULES_TEMPLATE_FLAG, "");
        // something was returned, explode the rules into an array
        if (strlen($tmp) > 0) {
            $flag_rules = explode("\n", $tmp);
        }
        // make sure that we have acutally something in flag_rules array 
        if (isset($flag_rules) && is_array($flag_rules) && count($flag_rules)) {
            $this->_flag_rules = $flag_rules;
        } else {
            $this->_flag_rules = array();
        }
    }
}
